@@ -21,7 +21,7 @@ set -u
 set -o pipefail
 
 cd /usr
-git clone --depth=1 https://github.com/apache/incubator-tvm tvm --recursive
+git clone https://github.com/apache/incubator-tvm tvm --recursive
 cd /usr/tvm
 # checkout a hash-tag
 git checkout 4b13bf668edc7099b38d463e5db94ebc96c80470
@@ -29,9 +29,6 @@ git checkout 4b13bf668edc7099b38d463e5db94ebc96c80470
 echo set\(USE_LLVM llvm-config-8\) >> config.cmake
 echo set\(USE_CUDA ON\) >> config.cmake
 echo set\(USE_CUDNN ON\) >> config.cmake
-echo set\(USE_RPC ON\) >> config.cmake
-echo set\(USE_SORT ON\) >> config.cmake
-echo set\(USE_GRAPH_RUNTIME ON\) >> config.cmake
 echo set\(USE_BLAS openblas\) >> config.cmake
 mkdir -p build
 cd build
