@@ -6,9 +6,9 @@
 # to you under the Apache License, Version 2.0 (the
 # "License"); you may not use this file except in compliance
 # with the License.  You may obtain a copy of the License at
-# 
+#
 #   http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing,
 # software distributed under the License is distributed on an
 # "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
@@ -25,9 +25,15 @@ apt-get update && apt-get install -y --no-install-recommends curl
 export RUSTUP_HOME=/opt/rust
 export CARGO_HOME=/opt/rust
 # this rustc is one supported by the installed version of rust-sgx-sdk
-curl -s -S -L https://sh.rustup.rs -sSf | sh -s -- -y --no-modify-path --default-toolchain nightly-2019-03-24
+curl -s -S -L https://sh.rustup.rs -sSf | sh -s -- -y --no-modify-path --default-toolchain stable
 . $CARGO_HOME/env
 rustup component add rustfmt
 
+# install wasmtime
+export WASMTIME_HOME=/opt/wasmtime
+curl https://wasmtime.dev/install.sh -sSf | bash
+export PATH="${WASMTIME_HOME}/bin:${PATH}"
+rustup target add wasm32-wasi
+
 # make rust usable by all users
 chmod -R a+w /opt/rust
