@@ -19,317 +19,32 @@
 
 /*!
  * \file tvm/relay/transform.h
- *
- * This file implements a pass manager. The pass manager manages a sequence
- * of Relay-to-Relay transformation passes over a particlar unit of AST. The
- * design is largely inspired from LLVM's pass manager and modern deep learning
- * frameworks that perform tensor->tensor transformations.
- *
- * The responsibilities of a traditional compiler pass manager usually involves:
- *  - Organizing the execution order of optimization passes though not
- * necessarily in the optimal sequence.
- *  - Collecting required analysis information and keep them up-to-date.
- *  - Reducing the effort required to implement new passes for compiler
- * developers, etc.
- *
- * Similar to LLVM's pass manager, we designed the Relay pass manager to work
- * different granularity, i.e. module level, function level, and even sequential
- * passe that contains a host of passes.
- *
- * However, we also extend the functionality of the traditional pass manager
- * with the consideration of requirements/convention from deep learning
- * frameworks, such as Pytorch and Gluon, etc. Each pass in the Relay pass
- * manager performs the Relay.Module -> Relay.Module transformation. All
- * different types of passes, including the sequential-level pass object, are
- * essentially pass objects. This design, therefore, effectively provides users
- * a consistent and convenient interface, i.e. Pass, to play with. It offers a
- * means to ease the development and testing of Relay passes. For example, with
- * the pass manager, external users will be able to have custom passes correctly
- * scheduled without having to modify a single handcrafted pass order.
- *
- * In the future we need to describe constraints between passes. For example,
- * we may want to preserve dependencies between different passes and validate
- * them on the completion of a certain pass.
- *
- * We also need to store side information and import the error reporting system.
+ * \brief Relay specific transformation passes.
  */
 #ifndef TVM_RELAY_TRANSFORM_H_
 #define TVM_RELAY_TRANSFORM_H_
 
-#include <tvm/base.h>
-#include <tvm/packed_func_ext.h>
+#include <tvm/ir/transform.h>
 #include <tvm/relay/attrs/transform.h>
-#include <tvm/relay/error.h>
 #include <tvm/relay/expr.h>
-#include <tvm/relay/module.h>
+#include <tvm/relay/function.h>
 #include <tvm/relay/op.h>
 #include <tvm/relay/op_attr_types.h>
+#include <tvm/runtime/container.h>
+
 #include <string>
-#include <unordered_map>
-#include <vector>
 
 namespace tvm {
 namespace relay {
 namespace transform {
 
-/*
- * \brief The context of pass.
- */
-class PassContext;
-
-/*!
- * \brief PassContextNode contains the information that a pass can rely on,
- * such as analysis results.
- */
-class PassContextNode : public RelayNode {
- public:
-  /*!
-   * \brief The error reporter used to notify users why an optimization fails.
-   */
-  ErrorReporter err_reporter;
-
-  /*! \brief The default optimization level. */
-  int opt_level{2};
-
-  /*! \brief CPU is the default fallback device for heterogeneous execution. */
-  int fallback_device{static_cast<int>(kDLCPU)};
-
-  /*! \brief The list of required passes. */
-  tvm::Array<tvm::Expr> required_pass;
-  /*! \brief The list of disabled passes. */
-  tvm::Array<tvm::Expr> disabled_pass;
-
-  PassContextNode() = default;
-
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("opt_level", &opt_level);
-    v->Visit("fallback_device", &fallback_device);
-    v->Visit("required_pass", &required_pass);
-    v->Visit("disabled_pass", &disabled_pass);
-  }
-
-  static constexpr const char* _type_key = "relay.PassContext";
-  TVM_DECLARE_NODE_TYPE_INFO(PassContextNode, RelayNode);
-};
-
-/*!
- * \brief PassContext that is used to configure the pass behavior.
- *
- * \code
- *
- *  auto new_ctx = PassContext::Create();
- *  ctx->opt_level = 2;
- *  ctx->fallback_device = kDLCPU;
- *  With<PassContext> scope(ctx);
- *  // pass context in effect.
- *
- * \endcode
- */
-class PassContext : public NodeRef {
- public:
-  PassContext() {}
-  explicit PassContext(NodePtr<::tvm::Node> n) : NodeRef(n) {}
-  /*!
-   * \brief const accessor.
-   * \return const access pointer.
-   */
-  const PassContextNode* operator->() const {
-    CHECK(get() != nullptr);
-    return static_cast<const PassContextNode*>(get());
-  }
-  /*!
-   * \brief mutable accessor.
-   * \return mutable access pointer.
-   */
-  PassContextNode* operator->() {
-    CHECK(get() != nullptr);
-    return static_cast<PassContextNode*>(get_mutable());
-  }
-  /*!
-   * \brief Construct a PassContext containing the default configurations.
-   * \return The new PassContext.
-   */
-  TVM_DLL static PassContext Create();
-  /*!
-   * \brief Get the default pass context in the current scope.
-   * \return The pass context.
-   */
-  TVM_DLL static PassContext Current();
-
-  // accessor.
-  using ContainerType = PassContextNode;
-  class Internal;
-
- private:
-  // The entry of a pass context scope.
-  TVM_DLL void EnterWithScope();
-  // The exit of a pass context scope.
-  TVM_DLL void ExitWithScope();
-
-  // Classes to get the Python `with` like syntax.
-  friend class Internal;
-  friend class tvm::With<PassContext>;
-};
-
-/*
- * \brief The meta data of a pass.
- *
- * PassInfo can be extended conveniently in the future if more meta information
- * is needed.
- */
-class PassInfo;
-
-/*!
- * \brief PassInfoNode contains meta data that will be used to help optimization
- * and analysis.
- */
-class PassInfoNode : public RelayNode {
- public:
-  /*! \brief The minimal optimization level that this pass will be enabled. */
-  int opt_level;
-
-  /*! \brief The name of an optimization/analysis pass. */
-  std::string name;
-
-  /*! \brief The passes that are required to perform the current pass. */
-  tvm::Array<tvm::Expr> required;
-
-  PassInfoNode() = default;
-
-  void VisitAttrs(tvm::AttrVisitor* v) {
-    v->Visit("opt_level", &opt_level);
-    v->Visit("name", &name);
-    v->Visit("required", &required);
-  }
-
-  TVM_DLL static PassInfo make(int opt_level,
-                               std::string name,
-                               tvm::Array<tvm::Expr> required);
-
-  static constexpr const char* _type_key = "relay.PassInfo";
-  TVM_DECLARE_NODE_TYPE_INFO(PassInfoNode, RelayNode);
-};
-
-TVM_DEFINE_NODE_REF(PassInfo, PassInfoNode)
-
-class Pass;
-
-/*!
- * \brief PassNode is the base type of differnt types of optimization passes.
- * It is designed as a pure class and implemented by different pass subclasses
- * at different granularity of Relay nodes.
- */
-class PassNode : public RelayNode {
- public:
-  virtual ~PassNode() {}
-  /*!
-   * \brief Get the pass information/meta data. */
-  virtual PassInfo Info() const = 0;
-
-  /*!
-   * \brief Transform mod using the default PassContext in the current scope.
-   *
-   * \param mod The module that an optimization pass runs on.
-   *
-   * \return The transformed module.
-   */
-  Module operator()(const Module& mod) const {
-    return this->operator()(mod, PassContext::Current());
-  }
-
-  /*!
-   * \brief Transform mod using a functor under a given pass context.
-   *
-   * \param mod The module that an optimization pass runs on.
-   * \param pass_ctx The pass context that can provide information for the optimization.
-   *
-   * \return The transformed module.
-   */
-  virtual Module operator()(const Module& mod,
-                            const PassContext& pass_ctx) const = 0;
-
-  void VisitAttrs(tvm::AttrVisitor* v) {}
-
-  static constexpr const char* _type_key = "relay.Pass";
-  TVM_DECLARE_BASE_NODE_INFO(PassNode, RelayNode);
-};
-
-class Pass : public NodeRef {
- public:
-  /*!
-   * \brief Transform mod using the default PassContext in the current scope.
-   *
-   * \param mod The module that an optimization pass runs on.
-   *
-   * \return The transformed module.
-   */
-  Module operator()(const Module& mod) const {
-    const PassNode* node = operator->();
-    CHECK(node != nullptr);
-    return node->operator()(mod);
-  }
-  /*!
-   * \brief Transform mod using a functor under a given pass context.
-   *
-   * \param mod The module that an optimization pass runs on.
-   * \param pass_ctx The pass context that can provide information for the optimization.
-   *
-   * \return The transformed module.
-   */
-  Module operator()(const Module& mod,
-                    const PassContext& pass_ctx) const {
-    const PassNode* node = operator->();
-    CHECK(node != nullptr);
-    return node->operator()(mod, pass_ctx);
-  }
-
-  TVM_DEFINE_NODE_REF_METHODS(Pass, NodeRef, PassNode);
-};
-
-class SequentialNode;
-
-class Sequential : public Pass {
- public:
-  /*!
-   * \brief The constructor of `Sequential`.
-   *
-   * \param passes The passes to apply.
-   * \param pass_info The pass metadata.
-   */
-  TVM_DLL Sequential(tvm::Array<Pass> passes, PassInfo pass_info);
-
-  /*!
-   * \brief The constructor of `Sequential`.
-   *
-   * \param passes The passes to apply.
-   * \param name The name of a sequential pass. It's defaulted to "sequential".
-   *        This allows users to only provide a list of passes and execute them
-   *        under a given context.
-   */
-  TVM_DLL Sequential(tvm::Array<Pass> passes, std::string name = "sequential");
-
-  Sequential() = default;
-  explicit Sequential(tvm::NodePtr<::tvm::Node> n) : Pass(n) {}
-
-  const SequentialNode* operator->() const;
-  using ContainerType = Sequential;
-};
-
-/*
- * \brief Create a module pass.
- *
- * \param pass_func The packed function that contains the optimization.
- * \param opt_level The optimization level of the module pass.
- * \param name The name of the module pass.
- * \param required The list of the passes that the module pass is dependent on.
- *
- * \return The created module pass.
- */
-Pass CreateModulePass(
-    const runtime::TypedPackedFunc<Module(Module, PassContext)>& pass_func,
-    int opt_level,
-    const std::string& name,
-    const tvm::Array<tvm::Expr>& required);
+using Pass = tvm::transform::Pass;
+using PassNode = tvm::transform::PassNode;
+using PassInfo = tvm::transform::PassInfo;
+using PassInfoNode = tvm::transform::PassInfoNode;
+using PassContext = tvm::transform::PassContext;
+using PassContextNode = tvm::transform::PassContextNode;
+using Sequential = tvm::transform::Sequential;
 
 /*
  * \brief Create a function pass.
@@ -341,11 +56,9 @@ Pass CreateModulePass(
  *
  * \return The created function pass.
  */
-TVM_DLL Pass CreateFunctionPass(const runtime::TypedPackedFunc<
-                                Function(Function, Module, PassContext)>& pass_func,
-                                int opt_level,
-                                const std::string& name,
-                                const tvm::Array<tvm::Expr>& required);
+TVM_DLL Pass CreateFunctionPass(
+    const runtime::TypedPackedFunc<Function(Function, IRModule, PassContext)>& pass_func,
+    int opt_level, String name, tvm::Array<String> required);
 
 /*! \brief Remove expressions which does not effect the program result.
  *
@@ -363,6 +76,20 @@ TVM_DLL Pass CreateFunctionPass(const runtime::TypedPackedFunc<
  */
 TVM_DLL Pass DeadCodeElimination(bool inline_once = false);
 
+/*!
+ * \brief Convert all expressions of TensorType into GradCell,
+ * an algebraic data type defined in gradient.rly.
+ *
+ * This will delay or decrease memory usage. All calls to
+ * ones, ones_like, zeros, zeros_like will not immediately instantiate a tensor in memory,
+ * rather only instantiate if needed. It also defines + and * operation
+ * between GradCell types which can increase performance when using
+ * zero-filled or one-filled tensors, which is the case in reverse mode ad.
+ *
+ * \return the pass
+ */
+TVM_DLL Pass LazyGradientInit();
+
 /*!
  * \brief Fold constant expressions.
  *
@@ -389,6 +116,21 @@ TVM_DLL Pass FuseOps(int fuse_opt_level = -1);
  */
 TVM_DLL Pass RewriteAnnotatedOps(int fallback_device);
 
+/*!
+ * \brief Turn an expression to Basic Block Normal Form.
+ *
+ * We define a block as a group of expressions implied by the scope structure.
+ *
+ * Each graph node can only belong to a single block.
+ *
+ * For any value that is being used in multiple blocks, it has to be referred
+ * by a Var which is defined in a block, whose scope is the least common ancestor
+ * of blocks this value is used.
+ *
+ * \return The pass.
+ */
+TVM_DLL Pass ToBasicBlockNormalForm();
+
 /*!
  * \brief turn a dataflow graph into Administrative Normal Form, or A-Normal Form (ANF).
  *
@@ -405,6 +147,15 @@ TVM_DLL Pass RewriteAnnotatedOps(int fallback_device);
  */
 TVM_DLL Pass ToANormalForm();
 
+/*!
+ * \brief ToANormalForm but on incomplete graph.
+ *
+ * \param expr the graph.
+ *
+ * \return The transformed program.
+ */
+TVM_DLL Expr ToANormalForm(const Expr& expr);
+
 /*!
  * \brief Turn an expression into continuation passing style(CPS).
  *
@@ -450,6 +201,13 @@ TVM_DLL Pass PartialEval();
  */
 TVM_DLL Pass SimplifyInference();
 
+/*!
+ * \brief Replaces non linear activation functions with their fast but approximate counterparts.
+ *
+ * \return The Pass.
+ */
+TVM_DLL Pass FastMath();
+
 /*!
  * \brief Infer the type of an expression.
  *
@@ -470,7 +228,7 @@ TVM_DLL Pass InferType();
  *
  * \return The pass.
  */
-TVM_DLL Pass EliminateCommonSubexpr(PackedFunc fskip = nullptr);
+TVM_DLL Pass EliminateCommonSubexpr(runtime::PackedFunc fskip = nullptr);
 
 /*!
  * \brief Combine parallel 2d convolutions into a single convolution if the
@@ -489,10 +247,23 @@ TVM_DLL Pass CombineParallelConv2D(uint64_t min_num_branches = 3);
  * `min_num_branch`.
  *
  * \param min_num_branches The minimun number of branches.
+ * \param to_batch_matmul Whether to combine parallel dense ops to batch matmul.
+ *                        If set false, combine dense ops to single dense op.
  *
  * \return The pass.
  */
-TVM_DLL Pass CombineParallelDense(uint64_t min_num_branches = 3);
+TVM_DLL Pass CombineParallelDense(uint64_t min_num_branches = 3, bool to_batch_matmul = true);
+
+/*!
+ * \brief Combine parallel batch_matmul ops into a single batch_matmul
+ *  if the number of branches of this dense operator is not less than
+ * `min_num_branch`.
+ *
+ * \param min_num_branches The minimun number of branches.
+ *
+ * \return The pass.
+ */
+TVM_DLL Pass CombineParallelBatchMatmul(uint64_t min_num_branches = 3);
 
 /*!
  * \brief Backward fold axis scaling into weights of conv/dense operators.
@@ -532,6 +303,28 @@ TVM_DLL Pass CanonicalizeOps();
  */
 TVM_DLL Pass AlterOpLayout();
 
+/*!
+ * \brief Given a dest layout, this pass transforms the expr such that most of the ops input data
+ * layout is changed to the dest layout. In ideal situation, there are only 2 layout transforms, one
+ * at the start and one at the end.
+ *
+ * This pass is not a part of relay.build and is expected to be called between framework-relay
+ * parser and relay.build call. This is very helpful for hardware backends that support/prefer only
+ * type of data layout.
+ *
+ * RFC - https://discuss.tvm.ai/t/layout-conversion-pass/4009
+ *
+ * This pass uses most of the AlterOpLayout and InferCorrectLayout infrastructure. We can define new
+ * layouts for conv2d ops for now. Most of the other operators try to adapt to their input layout
+ * using the InferCorrectLayout infrastructure.
+ *
+ * \param desired_layouts Specify mapping of op_name to array of desired layouts for each input.
+ *                        For example: Map("nn.conv2d", Array("NHWC", "OHWI")),
+ *                        this specifies the desired layout for data then kernel for nn.conv2d.
+ * \return The pass.
+ */
+TVM_DLL Pass ConvertLayout(const Map<String, Array<String>>& desired_layouts);
+
 /*!
  * \brief Legalizes an expr with another expression.
  * \param legalize_map_attr_name The Op's attr name which corresponds to the legalize rule function.
@@ -542,7 +335,7 @@ TVM_DLL Pass AlterOpLayout();
  *
  * \return The pass.
  */
-TVM_DLL Pass Legalize(const std::string& legalize_map_attr_name = "FTVMLegalize");
+TVM_DLL Pass Legalize(const String& legalize_map_attr_name = "FTVMLegalize");
 
 /*!
  * \brief Canonicalize cast expressions to make operator fusion more efficient.
@@ -568,13 +361,37 @@ TVM_DLL Pass CanonicalizeCast();
 TVM_DLL Pass EtaExpand(bool expand_constructor, bool expand_global_var);
 
 /*!
- * \brief Print the IR for a module to help debugging.
+ * \brief Partition a Relay program into regions that can be executed on
+ * different backends.
  *
- * \param show_meta_data The flag to control if meta data needs to be printed.
- *
- * \return the pass.
+ * \return The pass.
  */
-TVM_DLL Pass PrintIR(bool show_meta_data = true);
+TVM_DLL Pass PartitionGraph();
+
+/*!
+ * \brief Inline the global functions marked as `inline` in a given Relay
+ * IRModule.
+ *
+ * \return The pass.
+ */
+TVM_DLL Pass Inline();
+
+/*!
+ * \brief Remove the unused functions in the Relay IRModule.
+ *
+ * \param entry_functions The entry functions used to search the functions that
+ *        are being used.
+ *
+ * \return The pass.
+ */
+TVM_DLL Pass RemoveUnusedFunctions(Array<runtime::String> entry_functions);
+
+/*!
+ * \brief Simplify the Relay expression.
+ *
+ * \return The pass.
+ */
+TVM_DLL Pass SimplifyExpr();
 
 }  // namespace transform
 
@@ -600,9 +417,7 @@ TVM_DLL Expr Bind(const Expr& expr, const tvm::Map<Var, Expr>& binds);
  * \return A type checked Function with its checked_type field populated.
  * \note this function mutates mod and is not thread-safe.
  */
-TVM_DLL Function InferType(const Function& f,
-                           const Module& mod,
-                           const GlobalVar& var);
+TVM_DLL Function InferType(const Function& f, const IRModule& mod, const GlobalVar& var);
 
 /*!
  * \brief Apply rewrite rules to rewrite the expr in post DFS order. This
@@ -616,9 +431,8 @@ TVM_DLL Function InferType(const Function& f,
  *                           an Expr consumed by multiple callers.
  * \return The rewritten expression.
  */
-TVM_DLL Expr ForwardRewrite(const Expr& expr,
-                            const std::string& rewrite_map_attr_name,
-                            std::function<NodeRef(const Call&)> fcontext = nullptr,
+TVM_DLL Expr ForwardRewrite(const Expr& expr, const String& rewrite_map_attr_name,
+                            std::function<ObjectRef(const Call&)> fcontext = nullptr,
                             std::function<Expr(const Expr&)> fmulti_ref_trigger = nullptr);
 
 /*!
@@ -633,9 +447,8 @@ TVM_DLL Expr ForwardRewrite(const Expr& expr,
  *
  * \return The rewritten expression.
  */
-TVM_DLL Expr ForwardRewrite(const Expr& expr,
-                            const FForwardRewrite& rewrite_func,
-                            std::function<NodeRef(const Call&)> fcontext = nullptr,
+TVM_DLL Expr ForwardRewrite(const Expr& expr, const FForwardRewrite& rewrite_func,
+                            std::function<ObjectRef(const Call&)> fcontext = nullptr,
                             std::function<Expr(const Expr&)> fmulti_ref_trigger = nullptr);
 
 /*!
@@ -666,7 +479,7 @@ TVM_DLL Expr RewriteAnnotatedOps(const Expr& expr, int fallback_device);
  *
  * \return the converted Function.
  */
-TVM_DLL Function ToCPS(const Function& f, const Module& mod);
+TVM_DLL Function ToCPS(const Function& f, const IRModule& mod);
 
 /*!
  * \brief Remove the continuation argument of a CPS function.
