@@ -24,10 +24,11 @@
 #define TVM_RUNTIME_OBJECT_H_
 
 #include <dmlc/logging.h>
-#include <type_traits>
+#include <tvm/runtime/c_runtime_api.h>
+
 #include <string>
+#include <type_traits>
 #include <utility>
-#include "c_runtime_api.h"
 
 /*!
  * \brief Whether or not use atomic reference counter.
@@ -46,18 +47,35 @@
 namespace tvm {
 namespace runtime {
 
-/*! \brief list of the type index. */
-enum TypeIndex  {
-  /*! \brief Root object type. */
-  kRoot = 0,
-  kVMTensor = 1,
-  kVMClosure = 2,
-  kVMADT = 3,
-  kRuntimeModule = 4,
-  kStaticIndexEnd,
-  /*! \brief Type index is allocated during runtime. */
-  kDynamic = kStaticIndexEnd
-};
+/*!
+ * \brief Namespace for the list of type index.
+ * \note Use struct so that we have to use TypeIndex::ENumName to refer to
+ *       the constant, but still able to use enum.
+ */
+struct TypeIndex {
+  enum {
+    /*! \brief Root object type. */
+    kRoot = 0,
+    // Standard static index assignments,
+    // Frontends can take benefit of these constants.
+    /*! \brief runtime::Module. */
+    kRuntimeModule = 1,
+    /*! \brief runtime::NDArray. */
+    kRuntimeNDArray = 2,
+    /*! \brief runtime::String. */
+    kRuntimeString = 3,
+    /*! \brief runtime::Array. */
+    kRuntimeArray = 4,
+    /*! \brief runtime::Map. */
+    kRuntimeMap = 5,
+    // static assignments that may subject to change.
+    kRuntimeClosure,
+    kRuntimeADT,
+    kStaticIndexEnd,
+    /*! \brief Type index is allocated during runtime. */
+    kDynamic = kStaticIndexEnd
+  };
+};  // namespace TypeIndex
 
 /*!
  * \brief base class of all object containers.
@@ -69,10 +87,10 @@ enum TypeIndex  {
  *      the type index will be assigned during runtime.
  *      Runtime type index can be accessed by ObjectType::TypeIndex();
  * - _type_key:
- *       The unique string identifier of tyep type.
+ *       The unique string identifier of the type.
  * - _type_final:
  *       Whether the type is terminal type(there is no subclass of the type in the object system).
- *       This field is automatically set by marco TVM_DECLARE_FINAL_OBJECT_INFO
+ *       This field is automatically set by macro TVM_DECLARE_FINAL_OBJECT_INFO
  *       It is still OK to sub-class a terminal object type T and construct it using make_object.
  *       But IsInstance check will only show that the object type is T(instead of the sub-class).
  *
@@ -87,8 +105,8 @@ enum TypeIndex  {
  *       Recommendation: set to estimate number of children needed.
  * - _type_child_slots_can_overflow:
  *       Whether we can add additional child classes even if the number of child classes
- *       exceeds the _type_child_slots. A fallback mechanism to check global type table will be used.
- *       Recommendation: set to false for optimal runtime speed if we know exact number of children.
+ *       exceeds the _type_child_slots. A fallback mechanism to check global type table will be
+ * used. Recommendation: set to false for optimal runtime speed if we know exact number of children.
  *
  * Two macros are used to declare helper functions in the object:
  * - Use TVM_DECLARE_BASE_OBJECT_INFO for object classes that can be sub-classed.
@@ -122,7 +140,7 @@ enum TypeIndex  {
  *    // object properties
  *    static constexpr const uint32_t _type_index = TypeIndex::kDynamic;
  *    static constexpr const char* _type_key = "test.LeafObj";
- *    TVM_DECLARE_BASE_OBJECT_INFO(LeaffObj, Object);
+ *    TVM_DECLARE_BASE_OBJECT_INFO(LeafObj, Object);
  *  };
  *
  *  // The following code should be put into a cc file.
@@ -142,7 +160,7 @@ enum TypeIndex  {
  *
  * \endcode
  */
-class Object {
+class TVM_DLL Object {
  public:
   /*!
    * \brief Object deleter
@@ -150,28 +168,22 @@ class Object {
    */
   typedef void (*FDeleter)(Object* self);
   /*! \return The internal runtime type index of the object. */
-  uint32_t type_index() const {
-    return type_index_;
-  }
+  uint32_t type_index() const { return type_index_; }
   /*!
    * \return the type key of the object.
    * \note this operation is expensive, can be used for error reporting.
    */
-  std::string GetTypeKey() const {
-    return TypeIndex2Key(type_index_);
-  }
+  std::string GetTypeKey() const { return TypeIndex2Key(type_index_); }
   /*!
    * \return A hash value of the return of GetTypeKey.
    */
-  size_t GetTypeKeyHash() const {
-    return TypeIndex2KeyHash(type_index_);
-  }
+  size_t GetTypeKeyHash() const { return TypeIndex2KeyHash(type_index_); }
   /*!
    * Check if the object is an instance of TargetType.
    * \tparam TargetType The target type to be checked.
    * \return Whether the target type is true.
    */
-  template<typename TargetType>
+  template <typename TargetType>
   inline bool IsInstance() const;
 
   /*!
@@ -179,19 +191,19 @@ class Object {
    * \param tindex The type index.
    * \return the result.
    */
-  TVM_DLL static std::string TypeIndex2Key(uint32_t tindex);
+  static std::string TypeIndex2Key(uint32_t tindex);
   /*!
    * \brief Get the type key hash of the corresponding index from runtime.
    * \param tindex The type index.
    * \return the related key-hash.
    */
-  TVM_DLL static size_t TypeIndex2KeyHash(uint32_t tindex);
+  static size_t TypeIndex2KeyHash(uint32_t tindex);
   /*!
    * \brief Get the type index of the corresponding key from runtime.
    * \param key The type key.
    * \return the result.
    */
-  TVM_DLL static uint32_t TypeKey2Index(const std::string& key);
+  static uint32_t TypeKey2Index(const std::string& key);
 
 #if TVM_OBJECT_ATOMIC_REF_COUNTER
   using RefCounterType = std::atomic<int32_t>;
@@ -199,19 +211,19 @@ class Object {
   using RefCounterType = int32_t;
 #endif
 
-  static constexpr const char* _type_key = "Object";
+  static constexpr const char* _type_key = "runtime.Object";
 
-  static uint32_t _GetOrAllocRuntimeTypeIndex() {
-    return TypeIndex::kRoot;
-  }
-  static uint32_t RuntimeTypeIndex() {
-    return TypeIndex::kRoot;
-  }
+  static uint32_t _GetOrAllocRuntimeTypeIndex() { return TypeIndex::kRoot; }
+  static uint32_t RuntimeTypeIndex() { return TypeIndex::kRoot; }
 
   // Default object type properties for sub-classes
   static constexpr bool _type_final = false;
   static constexpr uint32_t _type_child_slots = 0;
   static constexpr bool _type_child_slots_can_overflow = true;
+  // member information
+  static constexpr bool _type_has_method_visit_attrs = true;
+  static constexpr bool _type_has_method_sequal_reduce = false;
+  static constexpr bool _type_has_method_shash_reduce = false;
   // NOTE: the following field is not type index of Object
   // but was intended to be used by sub-classes as default value.
   // The type index of Object is TypeIndex::kRoot
@@ -228,10 +240,10 @@ class Object {
   }
   Object(Object&& other) {  // NOLINT(*)
   }
-  Object& operator=(const Object& other) {  //NOLINT(*)
+  Object& operator=(const Object& other) {  // NOLINT(*)
     return *this;
   }
-  Object& operator=(Object&& other) {  //NOLINT(*)
+  Object& operator=(Object&& other) {  // NOLINT(*)
     return *this;
   }
 
@@ -249,7 +261,7 @@ class Object {
   FDeleter deleter_ = nullptr;
   // Invariant checks.
   static_assert(sizeof(int32_t) == sizeof(RefCounterType) &&
-                alignof(int32_t) == sizeof(RefCounterType),
+                    alignof(int32_t) == sizeof(RefCounterType),
                 "RefCounter ABI check.");
 
   /*!
@@ -269,12 +281,9 @@ class Object {
    * \param type_child_slots_can_overflow Whether to allow child to overflow the slots.
    * \return The allocated type index.
    */
-  TVM_DLL static uint32_t GetOrAllocRuntimeTypeIndex(
-      const std::string& key,
-      uint32_t static_tindex,
-      uint32_t parent_tindex,
-      uint32_t type_child_slots,
-      bool type_child_slots_can_overflow);
+  static uint32_t GetOrAllocRuntimeTypeIndex(const std::string& key, uint32_t static_tindex,
+                                             uint32_t parent_tindex, uint32_t type_child_slots,
+                                             bool type_child_slots_can_overflow);
 
   // reference counter related operations
   /*! \brief developer function, increases reference counter. */
@@ -296,11 +305,11 @@ class Object {
    * \param parent_tindex The parent type index.
    * \return The derivation results.
    */
-  TVM_DLL bool DerivedFrom(uint32_t parent_tindex) const;
+  bool DerivedFrom(uint32_t parent_tindex) const;
   // friend classes
-  template<typename>
+  template <typename>
   friend class ObjAllocatorBase;
-  template<typename>
+  template <typename>
   friend class ObjectPtr;
   friend class TVMRetValue;
   friend class ObjectInternal;
@@ -318,8 +327,8 @@ class Object {
  * \tparam ObjectType The object type
  * \return The corresponding RefType
  */
-template <typename RefType, typename ObjectType>
-inline RefType GetRef(const ObjectType* ptr);
+template <typename RelayRefType, typename ObjectType>
+inline RelayRefType GetRef(const ObjectType* ptr);
 
 /*!
  * \brief Downcast a base reference type to a more specific type.
@@ -380,9 +389,7 @@ class ObjectPtr {
     other.data_ = nullptr;
   }
   /*! \brief destructor */
-  ~ObjectPtr() {
-    this->reset();
-  }
+  ~ObjectPtr() { this->reset(); }
   /*!
    * \brief Swap this array with another Object
    * \param other The other Object
@@ -393,15 +400,11 @@ class ObjectPtr {
   /*!
    * \return Get the content of the pointer
    */
-  T* get() const {
-    return static_cast<T*>(data_);
-  }
+  T* get() const { return static_cast<T*>(data_); }
   /*!
    * \return The pointer
    */
-  T* operator->() const {
-    return get();
-  }
+  T* operator->() const { return get(); }
   /*!
    * \return The reference
    */
@@ -437,29 +440,17 @@ class ObjectPtr {
     }
   }
   /*! \return The use count of the ptr, for debug purposes */
-  int use_count() const {
-    return data_ != nullptr ? data_->use_count() : 0;
-  }
+  int use_count() const { return data_ != nullptr ? data_->use_count() : 0; }
   /*! \return whether the reference is unique */
-  bool unique() const {
-    return data_ != nullptr && data_->use_count() == 1;
-  }
+  bool unique() const { return data_ != nullptr && data_->use_count() == 1; }
   /*! \return Whether two ObjectPtr do not equal each other */
-  bool operator==(const ObjectPtr<T>& other) const {
-    return data_ == other.data_;
-  }
+  bool operator==(const ObjectPtr<T>& other) const { return data_ == other.data_; }
   /*! \return Whether two ObjectPtr equals each other */
-  bool operator!=(const ObjectPtr<T>& other) const {
-    return data_ != other.data_;
-  }
+  bool operator!=(const ObjectPtr<T>& other) const { return data_ != other.data_; }
   /*! \return Whether the pointer is nullptr */
-  bool operator==(std::nullptr_t null) const {
-    return data_ == nullptr;
-  }
+  bool operator==(std::nullptr_t null) const { return data_ == nullptr; }
   /*! \return Whether the pointer is not nullptr */
-  bool operator!=(std::nullptr_t null) const {
-    return data_ != nullptr;
-  }
+  bool operator!=(std::nullptr_t null) const { return data_ != nullptr; }
 
  private:
   /*! \brief internal pointer field */
@@ -473,20 +464,32 @@ class ObjectPtr {
       data_->IncRef();
     }
   }
+  /*!
+   * \brief Move an ObjectPtr from an RValueRef argument.
+   * \param ref The rvalue reference.
+   * \return the moved result.
+   */
+  static ObjectPtr<T> MoveFromRValueRefArg(Object** ref) {
+    ObjectPtr<T> ptr;
+    ptr.data_ = *ref;
+    *ref = nullptr;
+    return ptr;
+  }
   // friend classes
   friend class Object;
   friend class ObjectRef;
-  friend struct ObjectHash;
-  template<typename>
+  friend struct ObjectPtrHash;
+  template <typename>
   friend class ObjectPtr;
-  template<typename>
+  template <typename>
   friend class ObjAllocatorBase;
   friend class TVMPODValue_;
   friend class TVMArgsSetter;
   friend class TVMRetValue;
   friend class TVMArgValue;
-  template <typename RefType, typename ObjType>
-  friend RefType GetRef(const ObjType* ptr);
+  friend class TVMMovableArgValue_;
+  template <typename RelayRefType, typename ObjType>
+  friend RelayRefType GetRef(const ObjType* ptr);
   template <typename BaseType, typename ObjType>
   friend ObjectPtr<BaseType> GetObjectPtr(ObjType* ptr);
 };
@@ -503,49 +506,37 @@ class ObjectRef {
    * \param other Another object ref.
    * \return the compare result.
    */
-  bool same_as(const ObjectRef& other) const {
-    return data_ == other.data_;
-  }
+  bool same_as(const ObjectRef& other) const { return data_ == other.data_; }
   /*!
    * \brief Comparator
    * \param other Another object ref.
    * \return the compare result.
    */
-  bool operator==(const ObjectRef& other) const {
-    return data_ == other.data_;
-  }
+  bool operator==(const ObjectRef& other) const { return data_ == other.data_; }
   /*!
    * \brief Comparator
    * \param other Another object ref.
    * \return the compare result.
    */
-  bool operator!=(const ObjectRef& other) const {
-    return data_ != other.data_;
-  }
+  bool operator!=(const ObjectRef& other) const { return data_ != other.data_; }
   /*!
    * \brief Comparator
    * \param other Another object ref by address.
    * \return the compare result.
    */
-  bool operator<(const ObjectRef& other) const {
-    return data_.get() < other.data_.get();
-  }
-  /*! \return whether the expression is null */
-  bool defined() const {
-    return data_ != nullptr;
-  }
+  bool operator<(const ObjectRef& other) const { return data_.get() < other.data_.get(); }
+  /*!
+   * \return whether the object is defined(not null).
+   */
+  bool defined() const { return data_ != nullptr; }
   /*! \return the internal object pointer */
-  const Object* get() const {
-    return data_.get();
-  }
+  const Object* get() const { return data_.get(); }
   /*! \return the internal object pointer */
-  const Object* operator->() const {
-    return get();
-  }
+  const Object* operator->() const { return get(); }
   /*! \return whether the reference is unique */
-  bool unique() const {
-    return data_.unique();
-  }
+  bool unique() const { return data_.unique(); }
+  /*! \return The use count of the ptr, for debug purposes */
+  int use_count() const { return data_.use_count(); }
   /*!
    * \brief Try to downcast the internal Object to a
    *  raw pointer of a corresponding type.
@@ -562,38 +553,45 @@ class ObjectRef {
 
   /*! \brief type indicate the container type. */
   using ContainerType = Object;
+  // Default type properties for the reference class.
+  static constexpr bool _type_is_nullable = true;
 
  protected:
   /*! \brief Internal pointer that backs the reference. */
   ObjectPtr<Object> data_;
   /*! \return return a mutable internal ptr, can be used by sub-classes. */
-  Object* get_mutable() const {
-    return data_.get();
-  }
+  Object* get_mutable() const { return data_.get(); }
   /*!
    * \brief Internal helper function downcast a ref without check.
    * \note Only used for internal dev purposes.
    * \tparam T The target reference type.
    * \return The casted result.
    */
-  template<typename T>
+  template <typename T>
   static T DowncastNoCheck(ObjectRef ref) {
     return T(std::move(ref.data_));
   }
+  /*!
+   * \brief Clear the object ref data field without DecRef
+   *        after we successfully moved the field.
+   * \param ref The reference data.
+   */
+  static void FFIClearAfterMove(ObjectRef* ref) { ref->data_.data_ = nullptr; }
   /*!
    * \brief Internal helper function get data_ as ObjectPtr of ObjectType.
    * \note only used for internal dev purpose.
    * \tparam ObjectType The corresponding object type.
    * \return the corresponding type.
    */
-  template<typename ObjectType>
+  template <typename ObjectType>
   static ObjectPtr<ObjectType> GetDataPtr(const ObjectRef& ref) {
     return ObjectPtr<ObjectType>(ref.data_.data_);
   }
   // friend classes.
-  friend struct ObjectHash;
+  friend struct ObjectPtrHash;
   friend class TVMRetValue;
   friend class TVMArgsSetter;
+  friend class ObjectInternal;
   template <typename SubRef, typename BaseRef>
   friend SubRef Downcast(BaseRef ref);
 };
@@ -610,63 +608,69 @@ template <typename BaseType, typename ObjectType>
 inline ObjectPtr<BaseType> GetObjectPtr(ObjectType* ptr);
 
 /*! \brief ObjectRef hash functor */
-struct ObjectHash {
-  size_t operator()(const ObjectRef& a) const {
-    return operator()(a.data_);
-  }
+struct ObjectPtrHash {
+  size_t operator()(const ObjectRef& a) const { return operator()(a.data_); }
 
-  template<typename T>
+  template <typename T>
   size_t operator()(const ObjectPtr<T>& a) const {
     return std::hash<Object*>()(a.get());
   }
 };
 
-
 /*! \brief ObjectRef equal functor */
-struct ObjectEqual {
-  bool operator()(const ObjectRef& a, const ObjectRef& b) const {
-    return a.same_as(b);
-  }
+struct ObjectPtrEqual {
+  bool operator()(const ObjectRef& a, const ObjectRef& b) const { return a.same_as(b); }
 
-  template<typename T>
+  template <typename T>
   size_t operator()(const ObjectPtr<T>& a, const ObjectPtr<T>& b) const {
     return a == b;
   }
 };
 
-
 /*!
  * \brief helper macro to declare a base object type that can be inheritated.
  * \param TypeName The name of the current type.
  * \param ParentType The name of the ParentType
  */
-#define TVM_DECLARE_BASE_OBJECT_INFO(TypeName, ParentType)              \
-  static const uint32_t RuntimeTypeIndex()  {                           \
-    if (TypeName::_type_index != ::tvm::runtime::TypeIndex::kDynamic) { \
-      return TypeName::_type_index;                                     \
-    }                                                                   \
-    return _GetOrAllocRuntimeTypeIndex();                               \
-  }                                                                     \
-  static const uint32_t _GetOrAllocRuntimeTypeIndex()  {                \
-    static uint32_t tidx = GetOrAllocRuntimeTypeIndex(                  \
-        TypeName::_type_key,                                            \
-        TypeName::_type_index,                                          \
-        ParentType::_GetOrAllocRuntimeTypeIndex(),                      \
-        TypeName::_type_child_slots,                                    \
-        TypeName::_type_child_slots_can_overflow);                      \
-    return tidx;                                                        \
-  }                                                                     \
+#define TVM_DECLARE_BASE_OBJECT_INFO(TypeName, ParentType)                                     \
+  static_assert(!ParentType::_type_final, "ParentObj maked as final");                         \
+  static uint32_t RuntimeTypeIndex() {                                                         \
+    static_assert(TypeName::_type_child_slots == 0 || ParentType::_type_child_slots == 0 ||    \
+                      TypeName::_type_child_slots < ParentType::_type_child_slots,             \
+                  "Need to set _type_child_slots when parent specifies it.");                  \
+    if (TypeName::_type_index != ::tvm::runtime::TypeIndex::kDynamic) {                        \
+      return TypeName::_type_index;                                                            \
+    }                                                                                          \
+    return _GetOrAllocRuntimeTypeIndex();                                                      \
+  }                                                                                            \
+  static uint32_t _GetOrAllocRuntimeTypeIndex() {                                              \
+    static uint32_t tidx = Object::GetOrAllocRuntimeTypeIndex(                                 \
+        TypeName::_type_key, TypeName::_type_index, ParentType::_GetOrAllocRuntimeTypeIndex(), \
+        TypeName::_type_child_slots, TypeName::_type_child_slots_can_overflow);                \
+    return tidx;                                                                               \
+  }
 
 /*!
  * \brief helper macro to declare type information in a final class.
-  * \param TypeName The name of the current type.
-  * \param ParentType The name of the ParentType
-  */
-#define TVM_DECLARE_FINAL_OBJECT_INFO(TypeName, ParentType)             \
-  static const constexpr bool _type_final = true;                       \
-  static const constexpr int _type_child_slots = 0;                     \
-  TVM_DECLARE_BASE_OBJECT_INFO(TypeName, ParentType)                    \
+ * \param TypeName The name of the current type.
+ * \param ParentType The name of the ParentType
+ */
+#define TVM_DECLARE_FINAL_OBJECT_INFO(TypeName, ParentType) \
+  static const constexpr bool _type_final = true;           \
+  static const constexpr int _type_child_slots = 0;         \
+  TVM_DECLARE_BASE_OBJECT_INFO(TypeName, ParentType)
 
+/*! \brief helper macro to supress unused warning */
+#if defined(__GNUC__)
+#define TVM_ATTRIBUTE_UNUSED __attribute__((unused))
+#else
+#define TVM_ATTRIBUTE_UNUSED
+#endif
+
+#define TVM_STR_CONCAT_(__x, __y) __x##__y
+#define TVM_STR_CONCAT(__x, __y) TVM_STR_CONCAT_(__x, __y)
+
+#define TVM_OBJECT_REG_VAR_DEF static TVM_ATTRIBUTE_UNUSED uint32_t __make_Object_tid
 
 /*!
  * \brief Helper macro to register the object type to runtime.
@@ -674,40 +678,97 @@ struct ObjectEqual {
  *
  *  Use this macro in the cc file for each terminal class.
  */
-#define TVM_REGISTER_OBJECT_TYPE(TypeName)                              \
-  static DMLC_ATTRIBUTE_UNUSED uint32_t __make_Object_tidx ## _ ## TypeName ## __ = \
-      TypeName::_GetOrAllocRuntimeTypeIndex()
+#define TVM_REGISTER_OBJECT_TYPE(TypeName) \
+  TVM_STR_CONCAT(TVM_OBJECT_REG_VAR_DEF, __COUNTER__) = TypeName::_GetOrAllocRuntimeTypeIndex()
 
+/*
+ * \brief Define the default copy/move constructor and assign opeator
+ * \param TypeName The class typename.
+ */
+#define TVM_DEFINE_DEFAULT_COPY_MOVE_AND_ASSIGN(TypeName) \
+  TypeName(const TypeName& other) = default;              \
+  TypeName(TypeName&& other) = default;                   \
+  TypeName& operator=(const TypeName& other) = default;   \
+  TypeName& operator=(TypeName&& other) = default;
 
-#define TVM_DEFINE_OBJECT_REF_METHODS(TypeName, ParentType, ObjectName) \
-  TypeName() {}                                                         \
-  explicit TypeName(                                                    \
-      ::tvm::runtime::ObjectPtr<::tvm::runtime::Object> n)              \
-      : ParentType(n) {}                                                \
-  const ObjectName* operator->() const {                                \
-    return static_cast<const ObjectName*>(data_.get());                 \
-  }                                                                     \
-  operator bool() const { return data_ != nullptr; }                    \
+/*
+ * \brief Define object reference methods.
+ * \param TypeName The object type name
+ * \param ParentType The parent type of the objectref
+ * \param ObjectName The type name of the object.
+ */
+#define TVM_DEFINE_OBJECT_REF_METHODS(TypeName, ParentType, ObjectName)                        \
+  TypeName() = default;                                                                        \
+  explicit TypeName(::tvm::runtime::ObjectPtr<::tvm::runtime::Object> n) : ParentType(n) {}    \
+  TVM_DEFINE_DEFAULT_COPY_MOVE_AND_ASSIGN(TypeName);                                           \
+  const ObjectName* operator->() const { return static_cast<const ObjectName*>(data_.get()); } \
+  const ObjectName* get() const { return operator->(); }                                       \
   using ContainerType = ObjectName;
 
-#define TVM_DEFINE_OBJECT_REF_METHODS_MUT(TypeName, ParentType, ObjectName) \
-  TypeName() {}                                                             \
-  explicit TypeName(                                                        \
-      ::tvm::runtime::ObjectPtr<::tvm::runtime::Object> n)                  \
-      : ParentType(n) {}                                                    \
-  ObjectName* operator->() {                                    \
-    return static_cast<ObjectName*>(data_.get());                     \
-  }                                                                         \
-  operator bool() const { return data_ != nullptr; }                        \
+/*
+ * \brief Define object reference methods that is not nullable.
+ *
+ * \param TypeName The object type name
+ * \param ParentType The parent type of the objectref
+ * \param ObjectName The type name of the object.
+ */
+#define TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(TypeName, ParentType, ObjectName)            \
+  explicit TypeName(::tvm::runtime::ObjectPtr<::tvm::runtime::Object> n) : ParentType(n) {}    \
+  TVM_DEFINE_DEFAULT_COPY_MOVE_AND_ASSIGN(TypeName);                                           \
+  const ObjectName* operator->() const { return static_cast<const ObjectName*>(data_.get()); } \
+  const ObjectName* get() const { return operator->(); }                                       \
+  static constexpr bool _type_is_nullable = false;                                             \
   using ContainerType = ObjectName;
 
+/*
+ * \brief Define object reference methods of whose content is mutable.
+ * \param TypeName The object type name
+ * \param ParentType The parent type of the objectref
+ * \param ObjectName The type name of the object.
+ * \note We recommend making objects immutable when possible.
+ *       This macro is only reserved for objects that stores runtime states.
+ */
+#define TVM_DEFINE_MUTABLE_OBJECT_REF_METHODS(TypeName, ParentType, ObjectName)             \
+  TypeName() = default;                                                                     \
+  TVM_DEFINE_DEFAULT_COPY_MOVE_AND_ASSIGN(TypeName);                                        \
+  explicit TypeName(::tvm::runtime::ObjectPtr<::tvm::runtime::Object> n) : ParentType(n) {} \
+  ObjectName* operator->() const { return static_cast<ObjectName*>(data_.get()); }          \
+  using ContainerType = ObjectName;
+
+/*!
+ * \brief Define CopyOnWrite function in an ObjectRef.
+ * \param ObjectName The Type of the Node.
+ *
+ *  CopyOnWrite will generate a unique copy of the internal node.
+ *  The node will be copied if it is referenced by multiple places.
+ *  The function returns the raw pointer to the node to allow modification
+ *  of the content.
+ *
+ * \code
+ *
+ *  MyCOWObjectRef ref, ref2;
+ *  ref2 = ref;
+ *  ref.CopyOnWrite()->value = new_value;
+ *  assert(ref2->value == old_value);
+ *  assert(ref->value == new_value);
+ *
+ * \endcode
+ */
+#define TVM_DEFINE_OBJECT_REF_COW_METHOD(ObjectName)     \
+  ObjectName* CopyOnWrite() {                            \
+    CHECK(data_ != nullptr);                             \
+    if (!data_.unique()) {                               \
+      auto n = make_object<ObjectName>(*(operator->())); \
+      ObjectPtr<Object>(std::move(n)).swap(data_);       \
+    }                                                    \
+    return static_cast<ObjectName*>(data_.get());        \
+  }
+
 // Implementations details below
 // Object reference counting.
 #if TVM_OBJECT_ATOMIC_REF_COUNTER
 
-inline void Object::IncRef() {
-  ref_counter_.fetch_add(1, std::memory_order_relaxed);
-}
+inline void Object::IncRef() { ref_counter_.fetch_add(1, std::memory_order_relaxed); }
 
 inline void Object::DecRef() {
   if (ref_counter_.fetch_sub(1, std::memory_order_release) == 1) {
@@ -718,31 +779,25 @@ inline void Object::DecRef() {
   }
 }
 
-inline int Object::use_count() const {
-  return ref_counter_.load(std::memory_order_relaxed);
-}
+inline int Object::use_count() const { return ref_counter_.load(std::memory_order_relaxed); }
 
 #else
 
-inline void Object::IncRef() {
-  ++ref_counter_;
-}
+inline void Object::IncRef() { ++ref_counter_; }
 
 inline void Object::DecRef() {
-  if (--ref_counter == 0) {
+  if (--ref_counter_ == 0) {
     if (this->deleter_ != nullptr) {
       (*this->deleter_)(this);
     }
   }
 }
 
-inline int Object::use_count() const {
-  return ref_counter_;
-}
+inline int Object::use_count() const { return ref_counter_; }
 
 #endif  // TVM_OBJECT_ATOMIC_REF_COUNTER
 
-template<typename TargetType>
+template <typename TargetType>
 inline bool Object::IsInstance() const {
   const Object* self = this;
   // NOTE: the following code can be optimized by
@@ -776,11 +831,9 @@ inline bool Object::IsInstance() const {
   }
 }
 
-
 template <typename ObjectType>
 inline const ObjectType* ObjectRef::as() const {
-  if (data_ != nullptr &&
-      data_->IsInstance<ObjectType>()) {
+  if (data_ != nullptr && data_->IsInstance<ObjectType>()) {
     return static_cast<ObjectType*>(data_.get());
   } else {
     return nullptr;
@@ -791,6 +844,9 @@ template <typename RefType, typename ObjType>
 inline RefType GetRef(const ObjType* ptr) {
   static_assert(std::is_base_of<typename RefType::ContainerType, ObjType>::value,
                 "Can only cast to the ref of same container type");
+  if (!RefType::_type_is_nullable) {
+    CHECK(ptr != nullptr);
+  }
   return RefType(ObjectPtr<Object>(const_cast<Object*>(static_cast<const Object*>(ptr))));
 }
 
@@ -803,17 +859,18 @@ inline ObjectPtr<BaseType> GetObjectPtr(ObjType* ptr) {
 
 template <typename SubRef, typename BaseRef>
 inline SubRef Downcast(BaseRef ref) {
-  CHECK(ref->template IsInstance<typename SubRef::ContainerType>())
-      << "Downcast from " << ref->GetTypeKey() << " to "
-      << SubRef::ContainerType::_type_key << " failed.";
+  if (ref.defined()) {
+    CHECK(ref->template IsInstance<typename SubRef::ContainerType>())
+        << "Downcast from " << ref->GetTypeKey() << " to " << SubRef::ContainerType::_type_key
+        << " failed.";
+  } else {
+    CHECK(SubRef::_type_is_nullable) << "Downcast from nullptr to not nullable reference of "
+                                     << SubRef::ContainerType::_type_key;
+  }
   return SubRef(std::move(ref.data_));
 }
 
 }  // namespace runtime
-
-template<typename T>
-using NodePtr = runtime::ObjectPtr<T>;
-
 }  // namespace tvm
 
 #endif  // TVM_RUNTIME_OBJECT_H_
