@@ -15,10 +15,10 @@
 # specific language governing permissions and limitations
 # under the License.
 """Library information."""
-from __future__ import absolute_import
 import sys
 import os
 
+
 def split_env_var(env_var, split):
     """Splits environment variable string.
 
@@ -39,6 +39,7 @@ def split_env_var(env_var, split):
         return [p.strip() for p in os.environ[env_var].split(split)]
     return []
 
+
 def find_lib_path(name=None, search_path=None, optional=False):
     """Find dynamic library files.
 
@@ -66,17 +67,17 @@ def find_lib_path(name=None, search_path=None, optional=False):
 
     dll_path = []
 
-    if os.environ.get('TVM_LIBRARY_PATH', None):
-        dll_path.append(os.environ['TVM_LIBRARY_PATH'])
+    if os.environ.get("TVM_LIBRARY_PATH", None):
+        dll_path.append(os.environ["TVM_LIBRARY_PATH"])
 
-    if sys.platform.startswith('linux'):
-        dll_path.extend(split_env_var('LD_LIBRARY_PATH', ':'))
-        dll_path.extend(split_env_var('PATH', ':'))
-    elif sys.platform.startswith('darwin'):
-        dll_path.extend(split_env_var('DYLD_LIBRARY_PATH', ':'))
-        dll_path.extend(split_env_var('PATH', ':'))
-    elif sys.platform.startswith('win32'):
-        dll_path.extend(split_env_var('PATH', ';'))
+    if sys.platform.startswith("linux") or sys.platform.startswith("freebsd"):
+        dll_path.extend(split_env_var("LD_LIBRARY_PATH", ":"))
+        dll_path.extend(split_env_var("PATH", ":"))
+    elif sys.platform.startswith("darwin"):
+        dll_path.extend(split_env_var("DYLD_LIBRARY_PATH", ":"))
+        dll_path.extend(split_env_var("PATH", ":"))
+    elif sys.platform.startswith("win32"):
+        dll_path.extend(split_env_var("PATH", ";"))
 
     # Pip lib directory
     dll_path.append(os.path.join(ffi_dir, ".."))
@@ -88,6 +89,10 @@ def find_lib_path(name=None, search_path=None, optional=False):
 
     dll_path.append(install_lib_dir)
 
+    if os.path.isdir(source_dir):
+        dll_path.append(os.path.join(source_dir, "web", "dist", "wasm"))
+        dll_path.append(os.path.join(source_dir, "web", "dist"))
+
     dll_path = [os.path.realpath(x) for x in dll_path]
     if search_path is not None:
         if isinstance(search_path, list):
@@ -103,17 +108,19 @@ def find_lib_path(name=None, search_path=None, optional=False):
             lib_dll_path = [os.path.join(p, name) for p in dll_path]
         runtime_dll_path = []
     else:
-        if sys.platform.startswith('win32'):
-            lib_dll_path = [os.path.join(p, 'libtvm.dll') for p in dll_path] +\
-                           [os.path.join(p, 'tvm.dll') for p in dll_path]
-            runtime_dll_path = [os.path.join(p, 'libtvm_runtime.dll') for p in dll_path] +\
-                               [os.path.join(p, 'tvm_runtime.dll') for p in dll_path]
-        elif sys.platform.startswith('darwin'):
-            lib_dll_path = [os.path.join(p, 'libtvm.dylib') for p in dll_path]
-            runtime_dll_path = [os.path.join(p, 'libtvm_runtime.dylib') for p in dll_path]
+        if sys.platform.startswith("win32"):
+            lib_dll_path = [os.path.join(p, "libtvm.dll") for p in dll_path] + [
+                os.path.join(p, "tvm.dll") for p in dll_path
+            ]
+            runtime_dll_path = [os.path.join(p, "libtvm_runtime.dll") for p in dll_path] + [
+                os.path.join(p, "tvm_runtime.dll") for p in dll_path
+            ]
+        elif sys.platform.startswith("darwin"):
+            lib_dll_path = [os.path.join(p, "libtvm.dylib") for p in dll_path]
+            runtime_dll_path = [os.path.join(p, "libtvm_runtime.dylib") for p in dll_path]
         else:
-            lib_dll_path = [os.path.join(p, 'libtvm.so') for p in dll_path]
-            runtime_dll_path = [os.path.join(p, 'libtvm_runtime.so') for p in dll_path]
+            lib_dll_path = [os.path.join(p, "libtvm.so") for p in dll_path]
+            runtime_dll_path = [os.path.join(p, "libtvm_runtime.so") for p in dll_path]
 
     if not use_runtime:
         # try to find lib_dll_path
@@ -125,9 +132,11 @@ def find_lib_path(name=None, search_path=None, optional=False):
         lib_found = [p for p in runtime_dll_path if os.path.exists(p) and os.path.isfile(p)]
 
     if not lib_found:
-        message = ('Cannot find the files.\n' +
-                   'List of candidates:\n' +
-                   str('\n'.join(lib_dll_path + runtime_dll_path)))
+        message = (
+            "Cannot find the files.\n"
+            + "List of candidates:\n"
+            + str("\n".join(lib_dll_path + runtime_dll_path))
+        )
         if not optional:
             raise RuntimeError(message)
         return None
@@ -154,12 +163,13 @@ def find_include_path(name=None, search_path=None, optional=False):
     ffi_dir = os.path.dirname(os.path.abspath(os.path.expanduser(__file__)))
     source_dir = os.path.join(ffi_dir, "..", "..", "..")
     install_include_dir = os.path.join(ffi_dir, "..", "..", "..", "..")
+
     third_party_dir = os.path.join(source_dir, "3rdparty")
 
     header_path = []
 
-    if os.environ.get('TVM_INCLUDE_PATH', None):
-        header_path.append(os.environ['TVM_INCLUDE_PATH'])
+    if os.environ.get("TVM_INCLUDE_PATH", None):
+        header_path.append(os.environ["TVM_INCLUDE_PATH"])
 
     header_path.append(install_include_dir)
     header_path.append(source_dir)
@@ -179,18 +189,23 @@ def find_include_path(name=None, search_path=None, optional=False):
         else:
             tvm_include_path = [os.path.join(p, name) for p in header_path]
         dlpack_include_path = []
+        dmlc_include_path = []
     else:
-        tvm_include_path = [os.path.join(p, 'include') for p in header_path]
-        dlpack_include_path = [os.path.join(p, 'dlpack/include') for p in header_path]
+        tvm_include_path = [os.path.join(p, "include") for p in header_path]
+        dlpack_include_path = [os.path.join(p, "dlpack/include") for p in header_path]
+        dmlc_include_path = [os.path.join(p, "dmlc-core/include") for p in header_path]
 
         # try to find include path
         include_found = [p for p in tvm_include_path if os.path.exists(p) and os.path.isdir(p)]
         include_found += [p for p in dlpack_include_path if os.path.exists(p) and os.path.isdir(p)]
+        include_found += [p for p in dmlc_include_path if os.path.exists(p) and os.path.isdir(p)]
 
     if not include_found:
-        message = ('Cannot find the files.\n' +
-                   'List of candidates:\n' +
-                   str('\n'.join(tvm_include_path + dlpack_include_path)))
+        message = (
+            "Cannot find the files.\n"
+            + "List of candidates:\n"
+            + str("\n".join(tvm_include_path + dlpack_include_path))
+        )
         if not optional:
             raise RuntimeError(message)
         return None
@@ -202,4 +217,4 @@ def find_include_path(name=None, search_path=None, optional=False):
 # We use the version of the incoming release for code
 # that is under development.
 # The following line is set by tvm/python/update_version.py
-__version__ = "0.6.0"
+__version__ = "0.7.0"
