@@ -16,8 +16,13 @@
 # under the License.
 """Distributed executor infrastructure to scale up the tuning"""
 
-from .measure import MeasureInput, MeasureResult, MeasureErrorNo, measure_option, \
-    create_measure_batch
+from .measure import (
+    MeasureInput,
+    MeasureResult,
+    MeasureErrorNo,
+    measure_option,
+    create_measure_batch,
+)
 from .measure_methods import LocalBuilder, LocalRunner, RPCRunner, request_remote
 from .executor import Executor
 from .local_executor import LocalExecutor
