@@ -14,15 +14,18 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+# pylint: disable=broad-except
 """Common utilities"""
 from __future__ import absolute_import as _abs
 import logging
+import numpy as np
 
 import tvm
-import numpy as np
-from topi.util import get_const_tuple
+from tvm.ir import IRModule
+from tvm.topi.util import get_const_tuple
+
 from .. import expr as _expr
-from .. import module as _module
+from .. import function as _function
 from .. import transform as _transform
 from .. import op as _op
 from .. import analysis
@@ -40,6 +43,7 @@ class StrAttrsDict(object):
     attrs : Dict[str, str]
         The attributes to be used.
     """
+
     def __init__(self, attrs):
         self.attrs = attrs
 
@@ -140,8 +144,11 @@ class StrAttrsDict(object):
         """
         if key in self.attrs:
             tshape = self.attrs[key]
-            return tuple(int(x) if x.strip("- ").isdigit() else None
-                         for x in tshape.strip('()[]').split(',') if x)
+            return tuple(
+                int(x) if x.strip("- ").isdigit() else None
+                for x in tshape.strip("()[]").split(",")
+                if x
+            )
         if isinstance(default, RequiredAttr):
             raise AttributeError("Required attribute {} not found.".format(key))
         return default
@@ -164,8 +171,7 @@ class StrAttrsDict(object):
 
         if key in self.attrs:
             tshape = self.attrs[key]
-            return tuple(float(x.strip()) for x in
-                         tshape.strip('()[]').split(','))
+            return tuple(float(x.strip()) for x in tshape.strip("()[]").split(","))
         if isinstance(default, RequiredAttr):
             raise AttributeError("Required attribute {} not found.".format(key))
         return default
@@ -188,9 +194,9 @@ class StrAttrsDict(object):
         if key in self.attrs:
             value = self.attrs[key]
             seq = []
-            for tup in value.strip('()').split('),'):
-                tup = tup.strip('[]()')
-                els = [int(x.strip('( ')) for x in tup.split(',')]
+            for tup in value.strip("()").split("),"):
+                tup = tup.strip("[]()")
+                els = [int(x.strip("( ")) for x in tup.split(",")]
                 seq.append(tuple(els))
 
             return tuple(seq)
@@ -216,7 +222,7 @@ class StrAttrsDict(object):
         """
         if key in self.attrs:
             tshape = self.attrs[key]
-            return tuple(int(x.strip()) for x in tshape.strip('[]()').split(','))
+            return tuple(int(x.strip()) for x in tshape.strip("[]()").split(","))
         if isinstance(default, RequiredAttr):
             raise AttributeError("Required attribute {} not found.".format(key))
         return default
@@ -238,7 +244,7 @@ class StrAttrsDict(object):
         """
         if key in self.attrs:
             val = self.attrs[key]
-            return val.strip().lower() in ['true', '1', 't', 'y', 'yes']
+            return val.strip().lower() in ["true", "1", "t", "y", "yes"]
         if isinstance(default, RequiredAttr):
             raise AttributeError("Required attribute {} not found.".format(key))
         return default
@@ -251,11 +257,11 @@ def get_relay_op(op_name):
     op_name : str
         The Relay operator name.
     """
-    if '.' in op_name:
+    if "." in op_name:
         # explicit hierachical modules
         op = _op
         try:
-            for opn in op_name.split('.'):
+            for opn in op_name.split("."):
                 op = getattr(op, opn)
         except AttributeError:
             op = None
@@ -272,6 +278,7 @@ def get_relay_op(op_name):
 
 class ExprTable(object):
     """Table storing Relay expressions by names."""
+
     def __init__(self):
         self.exprs = {}
         self.params = {}
@@ -302,7 +309,7 @@ class ExprTable(object):
             self.exprs[name] = expr
 
     def has_expr(self, name):
-        return True if name in self.exprs else False
+        return name in self.exprs
 
     def set_padding(self, paddings):
         self.paddings = paddings
@@ -350,9 +357,17 @@ class AttrCvt(object):
         A custom function takes attribute, and return True/False.
         Raise RuntimeError if not bool(True) returned.
     """
-    def __init__(self, op_name, transforms=None,
-                 excludes=None, disables=None, ignores=None,
-                 extras=None, custom_check=None):
+
+    def __init__(
+        self,
+        op_name,
+        transforms=None,
+        excludes=None,
+        disables=None,
+        ignores=None,
+        extras=None,
+        custom_check=None,
+    ):
         self._op_name = op_name
         self._transforms = transforms if transforms else {}
         self._excludes = excludes if excludes else []
@@ -362,13 +377,13 @@ class AttrCvt(object):
         self._custom_check = custom_check
 
     def __call__(self, inputs, attrs, *args):
-        self._ignores.append('_output_shapes')
-        self._ignores.append('_input_shapes')
-        self._ignores.append('T')
-        self._ignores.append('use_cudnn_on_gpu')
-        self._ignores.append('_node_name')
-        self._ignores.append('is_training')
-        self._ignores.append('_target_layout')
+        self._ignores.append("_output_shapes")
+        self._ignores.append("_input_shapes")
+        self._ignores.append("T")
+        self._ignores.append("use_cudnn_on_gpu")
+        self._ignores.append("_node_name")
+        self._ignores.append("is_training")
+        self._ignores.append("_target_layout")
 
         # apply custom check
         if self._custom_check:
@@ -383,18 +398,19 @@ class AttrCvt(object):
             op_name = self._op_name(attrs)
 
         # ignore 'tvm_custom' always
-        self._ignores.append('tvm_custom')
+        self._ignores.append("tvm_custom")
 
         # convert attributes
         new_attrs = {}
         for k in attrs.keys():
             if k in self._excludes:
-                raise NotImplementedError('Attribute %s in operator %s is not' +
-                                          ' supported.', k, op_name)
-            elif k in self._disables:
+                raise NotImplementedError(
+                    "Attribute %s in operator %s is not" + " supported.", k, op_name
+                )
+            if k in self._disables:
                 logging.warning("Attribute %s is disabled in relay.sym.%s", k, op_name)
             elif k in self._ignores:
-                if k != 'tvm_custom':
+                if k != "tvm_custom":
                     logging.warning("Attribute %s is ignored in relay.sym.%s", k, op_name)
             elif k in self._transforms:
                 new_name, defaults, transform = self._parse_default(self._transforms[k])
@@ -433,7 +449,7 @@ class AttrCvt(object):
     def _parse_bool(self, value):
         """Helper function to parse default boolean values."""
         if isinstance(value, str):
-            return value.strip().lower() in ['true', '1', 't', 'y', 'yes']
+            return value.strip().lower() in ["true", "1", "t", "y", "yes"]
         return bool(value)
 
     def _required_attr(self, attr, key):
@@ -445,7 +461,7 @@ class AttrCvt(object):
 
 
 def get_name(node):
-    name = ''
+    name = ""
     if hasattr(node, "name_hint"):
         name = node.name_hint
     return name
@@ -453,22 +469,21 @@ def get_name(node):
 
 def infer_type(node, mod=None):
     """A method to infer the type of an intermediate node in the relay graph."""
-    new_mod = _module.Module.from_expr(node)
-    if mod is not None:
-        new_mod.update(mod)
-    new_mod = _transform.InferType()(new_mod)
-    entry = new_mod["main"]
-    return entry if isinstance(node, _expr.Function) else entry.body
+    if isinstance(mod, IRModule):
+        mod["main"] = _function.Function(tvm.relay.analysis.free_vars(node), node)
+        mod = _transform.InferType()(mod)
+        entry = mod["main"]
+        ret = entry.body
+    else:
+        new_mod = IRModule.from_expr(node)
+        if mod is not None:
+            new_mod.update(mod)
+            new_mod = _transform.InferType()(new_mod)
+        entry = new_mod["main"]
+        ret = entry if isinstance(node, _function.Function) else entry.body
+
+    return ret
 
-def infer_shape(inputs, mod=None):
-    """A method to get the output type of an intermediate node in the graph."""
-    out_type = infer_type(inputs, mod=mod)
-    checked_type = out_type.checked_type
-    if hasattr(checked_type, 'shape'):
-        # Regular operator that outputs tensors
-        return get_const_tuple(out_type.checked_type.shape)
-    # The return type is not a tensor, for example List
-    return checked_type
 
 def infer_channels(inputs, transpose=False):
     """A hack for getting 'channels' or 'units' since caffe2 does not provide
@@ -480,23 +495,49 @@ def infer_channels(inputs, transpose=False):
     return channels
 
 
-def infer_value(input_val, params):
+def infer_shape(inputs, mod=None):
+    """A method to get the output type of an intermediate node in the graph."""
+    out_type = infer_type(inputs, mod=mod)
+    checked_type = out_type.checked_type
+    if hasattr(checked_type, "shape"):
+        # Regular operator that outputs tensors
+        return get_const_tuple(checked_type.shape)
+    # The return type is not a tensor, for example List
+    return checked_type
+
+
+def infer_value(input_val, params, mod=None):
     """A hack for getting the value of an expression by evaluating a
     portion of the relay graph. This is often needed for functions that
     whose output shape depends on the value of a tensor.
     """
-    from tvm.contrib import graph_runtime
     # Check that all free variables have associated parameters.
-    assert all(var.name_hint in params.keys() for var in analysis.free_vars(
-        input_val)), "All inputs to infer must be available in params."
-    func = _expr.Function(analysis.free_vars(input_val), input_val)
-    with tvm.relay.build_config(opt_level=0):
-        graph, lib, params = tvm.relay.build(func, target="llvm", params=params)
-    ctx = tvm.cpu(0)
-    m = graph_runtime.create(graph, lib, ctx)
-    m.set_input(**params)
-    m.run()
-    return m.get_output(0)
+    assert all(
+        var.name_hint in params.keys() for var in analysis.free_vars(input_val)
+    ), "All inputs to infer must be available in params."
+    try:
+        # TODO(kevinthesun): Use VM for all cases.
+        # pylint: disable=import-outside-toplevel
+        from tvm.contrib import graph_runtime
+
+        func = _function.Function(analysis.free_vars(input_val), input_val)
+        with tvm.transform.PassContext(opt_level=0):
+            lib = tvm.relay.build(func, target="llvm", params=params)
+        ctx = tvm.cpu(0)
+        m = graph_runtime.GraphModule(lib["default"](ctx))
+        m.run()
+        return m.get_output(0)
+    except Exception:
+        if isinstance(mod, IRModule):
+            mod["main"] = _function.Function(analysis.free_vars(input_val), input_val)
+        else:
+            mod = IRModule.from_expr(input_val)
+        exc = tvm.relay.create_executor("debug", mod=mod, ctx=tvm.cpu(), target="llvm")
+        inputs = []
+        for param in mod["main"].params:
+            inputs.append(params[param.name_hint])
+        result = exc.evaluate()(*inputs)
+        return result
 
 
 def infer_value_simulated(input_val, params):
@@ -513,9 +554,7 @@ def infer_value_simulated(input_val, params):
             fp_dtype = free_param.type_annotation.dtype
             fp_shape = [s.value for s in free_param.type_annotation.shape]
             fake_params.append(free_param)
-            params[free_param.name_hint] = tvm.nd.array(
-                np.random.rand(*fp_shape).astype(fp_dtype)
-            )
+            params[free_param.name_hint] = tvm.nd.array(np.random.rand(*fp_shape).astype(fp_dtype))
     # Now infer the value.
     output_value = infer_value(input_val, params)
     # Clean fake params out of param dictionary.
@@ -524,10 +563,24 @@ def infer_value_simulated(input_val, params):
     return output_value
 
 
-def new_var(name_hint,
-            type_annotation=None,
-            shape=None,
-            dtype="float32"):
+def try_infer_value(val, on_success=None, on_failure=None):
+    """Try running infer_value on the input val, and if successful, return the inferred value or
+    pass it to on_success callback if provided. Otherwise, run on_failure callback if it is
+    provided, or return the input val as output. In each case, the second return value
+    indicates whether infer_value has succeeded or not.
+    """
+    try:
+        ret = infer_value(val, {}).asnumpy()
+        if on_success:
+            return on_success(ret), True
+        return ret, True
+    except Exception:
+        if on_failure:
+            return on_failure(), False
+        return val, False
+
+
+def new_var(name_hint, type_annotation=None, shape=None, dtype="float32"):
     return _expr.var(name_hint, type_annotation, shape, dtype)
 
 
@@ -539,10 +592,11 @@ class Renamer(object):
     new_name : str
         The new name for the operator
     """
+
     def __init__(self, new_name):
         self._new_name = new_name
 
     def __call__(self, inputs, attrs, *args):
-        if 'tvm_custom' in attrs:
-            attrs.pop('tvm_custom')
+        if "tvm_custom" in attrs:
+            attrs.pop("tvm_custom")
         return get_relay_op(self._new_name)(*inputs, **attrs)
