@@ -15,7 +15,10 @@
 # specific language governing permissions and limitations
 # under the License.
 # pylint: disable=wildcard-import
-"""Neural network related operators."""
-from __future__ import absolute_import as _abs
-from .contrib import *
-from . import _contrib
+"""Contrib modules."""
+from .register import get_pattern_table, register_pattern_table
+
+from .arm_compute_lib import *
+from .dnnl import *
+from .coreml import *
+from .ethosn import *
