@@ -16,11 +16,13 @@
 # under the License.
 """Basic tensor operations."""
 # pylint: disable=redefined-builtin
-from __future__ import absolute_import as _abs
+from tvm.runtime import ndarray as _nd
+from tvm.runtime import TVMContext as _TVMContext
+
 from . import _make
-from ..expr import Tuple
-from ... import nd as _nd
-from ... import TVMContext as _TVMContext
+from .dyn import _make as _dyn_make
+from ..expr import Tuple, Expr
+
 
 # We create a wrapper function for each operator in the
 # python side to call into the positional _make.OpName function.
@@ -31,6 +33,7 @@ from ... import TVMContext as _TVMContext
 # - Not put too much burden on FFI to support complicated features
 #   like default value and keyword arguments
 
+
 def log(data):
     """Compute elementwise log of data.
 
@@ -46,6 +49,55 @@ def log(data):
     """
     return _make.log(data)
 
+
+def log2(data):
+    """Compute elementwise log to the base 2 of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.log2(data)
+
+
+def log10(data):
+    """Compute elementwise log to the base 10 of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.log10(data)
+
+
+def tan(data):
+    """Compute elementwise tan of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.tan(data)
+
+
 def cos(data):
     """Compute elementwise cos of data.
 
@@ -61,6 +113,23 @@ def cos(data):
     """
     return _make.cos(data)
 
+
+def cosh(data):
+    """Compute elementwise cosh of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.cosh(data)
+
+
 def sin(data):
     """Compute elementwise sin of data.
 
@@ -76,6 +145,87 @@ def sin(data):
     """
     return _make.sin(data)
 
+
+def sinh(data):
+    """Compute elementwise sinh of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.sinh(data)
+
+
+def acos(data):
+    """Compute elementwise acos of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.acos(data)
+
+
+def acosh(data):
+    """Compute elementwise acosh of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.acosh(data)
+
+
+def asin(data):
+    """Compute elementwise asin of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.asin(data)
+
+
+def asinh(data):
+    """Compute elementwise asinh of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.asinh(data)
+
+
 def atan(data):
     """Compute elementwise atan of data.
 
@@ -91,6 +241,23 @@ def atan(data):
     """
     return _make.atan(data)
 
+
+def atanh(data):
+    """Compute elementwise atanh of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.atanh(data)
+
+
 def exp(data):
     """Compute elementwise exp of data.
 
@@ -254,6 +421,7 @@ def abs(data):
     """
     return _make.abs(data)
 
+
 def sign(data):
     """Compute element-wise absolute of data.
 
@@ -269,6 +437,7 @@ def sign(data):
     """
     return _make.sign(data)
 
+
 def tanh(data):
     """Compute element-wise tanh of data.
 
@@ -317,6 +486,22 @@ def logical_not(data):
     return _make.logical_not(data)
 
 
+def bitwise_not(data):
+    """Compute element-wise bitwise not of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.bitwise_not(data)
+
+
 def add(lhs, rhs):
     """Addition with numpy-style broadcasting.
 
@@ -505,6 +690,78 @@ def logical_or(lhs, rhs):
     return _make.logical_or(lhs, rhs)
 
 
+def logical_xor(lhs, rhs):
+    """logical XOR with numpy-style broadcasting.
+
+    Parameters
+    ----------
+    lhs : relay.Expr
+        The left hand side input data
+    rhs : relay.Expr
+        The right hand side input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.logical_xor(lhs, rhs)
+
+
+def bitwise_and(lhs, rhs):
+    """bitwise AND with numpy-style broadcasting.
+
+    Parameters
+    ----------
+    lhs : relay.Expr
+        The left hand side input data
+    rhs : relay.Expr
+        The right hand side input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.bitwise_and(lhs, rhs)
+
+
+def bitwise_or(lhs, rhs):
+    """bitwise OR with numpy-style broadcasting.
+
+    Parameters
+    ----------
+    lhs : relay.Expr
+        The left hand side input data
+    rhs : relay.Expr
+        The right hand side input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.bitwise_or(lhs, rhs)
+
+
+def bitwise_xor(lhs, rhs):
+    """bitwise XOR with numpy-style broadcasting.
+
+    Parameters
+    ----------
+    lhs : relay.Expr
+        The left hand side input data
+    rhs : relay.Expr
+        The right hand side input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.bitwise_xor(lhs, rhs)
+
+
 def equal(lhs, rhs):
     """Broadcasted elementwise test for (lhs == rhs).
 
@@ -690,7 +947,7 @@ def zeros(shape, dtype):
 
     Parameters
     ----------
-    shape : tuple of int
+    shape : tuple of int or relay.Expr
         The shape of the target.
 
     dtype : data type
@@ -701,6 +958,12 @@ def zeros(shape, dtype):
     result : relay.Expr
         The resulting tensor.
     """
+    if isinstance(shape, Expr):
+        return _dyn_make.zeros(shape, dtype)
+    if isinstance(shape, int):
+        shape = [shape]
+    if isinstance(shape, (list, tuple)):
+        shape = list(shape)
     return _make.zeros(shape, dtype)
 
 
@@ -725,7 +988,7 @@ def ones(shape, dtype):
 
     Parameters
     ----------
-    shape : tuple of int
+    shape : tuple of int or relay.Expr
         The shape of the target.
 
     dtype : data type
@@ -736,6 +999,12 @@ def ones(shape, dtype):
     result : relay.Expr
         The resulting tensor.
     """
+    if isinstance(shape, Expr):
+        return _dyn_make.ones(shape, dtype)
+    if isinstance(shape, int):
+        shape = [shape]
+    if isinstance(shape, (list, tuple)):
+        shape = list(shape)
     return _make.ones(shape, dtype)
 
 
@@ -776,6 +1045,7 @@ def clip(a, a_min, a_max):
     Examples
     --------
     .. code:: python
+
       x = relay.Constant(tvm.nd.array([0, 1, 5, 3, 4, 2]))
       relay.clip(x, 1., 4.)
       # [1, 1, 4, 3, 4, 2]
@@ -783,6 +1053,28 @@ def clip(a, a_min, a_max):
     return _make.clip(a, a_min, a_max)
 
 
+def fixed_point_multiply(data, multiplier, shift):
+    """Fixed point multiplication between data and a fixed point
+    constant expressed as multiplier * 2^(-shift), where multiplier
+    is a Q-number with 31 fractional bits
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input tensor.
+    multiplier : int
+        The integer multiplier of the fixed point constant.
+    a_max : float
+        The integer shift of the fixed point constant.
+
+    Returns
+    -------
+    result : relay.Expr
+        The output of the fixed point multiplication
+    """
+    return _make.fixed_point_multiply(data, multiplier, shift)
+
+
 def concatenate(data, axis):
     """Concatenate the input tensors along the given axis.
 
@@ -872,16 +1164,20 @@ def device_copy(data, src_dev, dst_dev):
     elif isinstance(src_dev, str):
         src_dev = _nd.context(src_dev).device_type
     else:
-        raise ValueError("src_dev is expected to be the type of TVMContext or "
-                         "str, but received %s" % (type(src_dev)))
+        raise ValueError(
+            "src_dev is expected to be the type of TVMContext or "
+            "str, but received %s" % (type(src_dev))
+        )
 
     if isinstance(dst_dev, _TVMContext):
         dst_dev = dst_dev.device_type
     elif isinstance(dst_dev, str):
         dst_dev = _nd.context(dst_dev).device_type
     else:
-        raise ValueError("dst_dev is expected to be the type of TVMContext or "
-                         "str, but received %s" % (type(dst_dev)))
+        raise ValueError(
+            "dst_dev is expected to be the type of TVMContext or "
+            "str, but received %s" % (type(dst_dev))
+        )
     return _make.device_copy(data, src_dev, dst_dev)
 
 
@@ -902,3 +1198,70 @@ def shape_of(data, dtype="int32"):
         The shape tensor.
     """
     return _make.shape_of(data, dtype)
+
+
+def ndarray_size(data, dtype="int32"):
+    """Get number of elements of input tensor.
+
+    Parameters
+    ----------
+    data : tvm.relay.Expr
+        The input tensor.
+
+    dtype : str, optional
+        The target data type.
+
+    Returns
+    -------
+    result : tvm.relay.Expr
+        The number of elements of input tensor.
+    """
+    return _make.ndarray_size(data, dtype)
+
+
+def isnan(data):
+    """Check nan in input data element-wise.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.isnan(data)
+
+
+def isfinite(data):
+    """Compute element-wise finiteness of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.isfinite(data)
+
+
+def isinf(data):
+    """Compute element-wise infiniteness of data.
+
+    Parameters
+    ----------
+    data : relay.Expr
+        The input data
+
+    Returns
+    -------
+    result : relay.Expr
+        The computed result.
+    """
+    return _make.isinf(data)
