@@ -14,9 +14,10 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-#pylint: disable=unused-argument
+# pylint: disable=unused-argument
 """The register functions for the QNN dialect."""
-from tvm.relay.op.op import register as register
+import tvm.ir
+
 
 def register_qnn_legalize(op_name, legal_op=None, level=10):
     """Register legal transformation function for a QNN op
@@ -32,4 +33,4 @@ def register_qnn_legalize(op_name, legal_op=None, level=10):
     level : int
         The priority level
     """
-    return register(op_name, "FTVMQnnLegalize", legal_op, level)
+    return tvm.ir.register_op_attr(op_name, "FTVMQnnLegalize", legal_op, level)
