@@ -25,9 +25,6 @@ Server is TCP based with the following protocol:
    - {server|client}:device-type[:random-key] [-timeout=timeout]
 """
 # pylint: disable=invalid-name
-
-from __future__ import absolute_import
-
 import os
 import ctypes
 import socket
@@ -40,16 +37,18 @@ import time
 import sys
 import signal
 import platform
+import tvm._ffi
 
-from .._ffi.function import register_func
-from .._ffi.base import py_str
-from .._ffi.libinfo import find_lib_path
-from ..module import load as _load_module
-from ..contrib import util
+from tvm._ffi.base import py_str
+from tvm._ffi.libinfo import find_lib_path
+from tvm.runtime.module import load_module as _load_module
+from tvm.contrib import util
+from . import _ffi_api
 from . import base
-from . base import TrackerCode
+from .base import TrackerCode
+
+logger = logging.getLogger("RPCServer")
 
-logger = logging.getLogger('RPCServer')
 
 def _server_env(load_library, work_path=None):
     """Server environment function return temp dir"""
@@ -59,11 +58,11 @@ def _server_env(load_library, work_path=None):
         temp = util.tempdir()
 
     # pylint: disable=unused-variable
-    @register_func("tvm.rpc.server.workpath")
+    @tvm._ffi.register_func("tvm.rpc.server.workpath", override=True)
     def get_workpath(path):
         return temp.relpath(path)
 
-    @register_func("tvm.rpc.server.load_module", override=True)
+    @tvm._ffi.register_func("tvm.rpc.server.load_module", override=True)
     def load_module(file_name):
         """Load module from remote side."""
         path = temp.relpath(file_name)
@@ -71,6 +70,37 @@ def _server_env(load_library, work_path=None):
         logger.info("load_module %s", path)
         return m
 
+    @tvm._ffi.register_func("tvm.rpc.server.download_linked_module", override=True)
+    def download_linked_module(file_name):
+        """Load module from remote side."""
+        # c++ compiler/linker
+        cc = os.environ.get("CXX", "g++")
+
+        # pylint: disable=import-outside-toplevel
+        path = temp.relpath(file_name)
+
+        if path.endswith(".o"):
+            # Extra dependencies during runtime.
+            from tvm.contrib import cc as _cc
+
+            _cc.create_shared(path + ".so", path, cc=cc)
+            path += ".so"
+        elif path.endswith(".tar"):
+            # Extra dependencies during runtime.
+            from tvm.contrib import cc as _cc, tar as _tar
+
+            tar_temp = util.tempdir(custom_path=path.replace(".tar", ""))
+            _tar.untar(path, tar_temp.temp_dir)
+            files = [tar_temp.relpath(x) for x in tar_temp.listdir()]
+            _cc.create_shared(path + ".so", files, cc=cc)
+            path += ".so"
+        elif path.endswith(".dylib") or path.endswith(".so"):
+            pass
+        else:
+            raise RuntimeError("Do not know how to link %s" % file_name)
+        logger.info("Send linked module %s to client", path)
+        return bytearray(open(path, "rb").read())
+
     libs = []
     load_library = load_library.split(":") if load_library else []
     for file_name in load_library:
@@ -80,15 +110,17 @@ def _server_env(load_library, work_path=None):
     temp.libs = libs
     return temp
 
+
 def _serve_loop(sock, addr, load_library, work_path=None):
     """Server loop"""
     sockfd = sock.fileno()
     temp = _server_env(load_library, work_path)
-    base._ServerLoop(sockfd)
+    _ffi_api.ServerLoop(sockfd)
     if not work_path:
         temp.remove()
     logger.info("Finish serving %s", addr)
 
+
 def _parse_server_opt(opts):
     # parse client options
     ret = {}
@@ -97,8 +129,10 @@ def _parse_server_opt(opts):
             ret["timeout"] = float(kv[9:])
     return ret
 
+
 def _listen_loop(sock, port, rpc_key, tracker_addr, load_library, custom_addr):
     """Listening loop of the server master."""
+
     def _accept_conn(listen_sock, tracker_conn, ping_period=2):
         """Accept connection from the other places.
 
@@ -117,8 +151,7 @@ def _listen_loop(sock, port, rpc_key, tracker_addr, load_library, custom_addr):
         # Report resource to tracker
         if tracker_conn:
             matchkey = base.random_key(rpc_key + ":")
-            base.sendjson(tracker_conn,
-                          [TrackerCode.PUT, rpc_key, (port, matchkey), custom_addr])
+            base.sendjson(tracker_conn, [TrackerCode.PUT, rpc_key, (port, matchkey), custom_addr])
             assert base.recvjson(tracker_conn) == TrackerCode.SUCCESS
         else:
             matchkey = rpc_key
@@ -143,9 +176,9 @@ def _listen_loop(sock, port, rpc_key, tracker_addr, load_library, custom_addr):
                     if unmatch_period_count * ping_period > unmatch_timeout + ping_period:
                         logger.info("no incoming connections, regenerate key ...")
                         matchkey = base.random_key(rpc_key + ":", old_keyset)
-                        base.sendjson(tracker_conn,
-                                      [TrackerCode.PUT, rpc_key, (port, matchkey),
-                                       custom_addr])
+                        base.sendjson(
+                            tracker_conn, [TrackerCode.PUT, rpc_key, (port, matchkey), custom_addr]
+                        )
                         assert base.recvjson(tracker_conn) == TrackerCode.SUCCESS
                         unmatch_period_count = 0
                     continue
@@ -164,11 +197,10 @@ def _listen_loop(sock, port, rpc_key, tracker_addr, load_library, custom_addr):
                 conn.close()
                 logger.warning("mismatch key from %s", addr)
                 continue
-            else:
-                conn.sendall(struct.pack("<i", base.RPC_CODE_SUCCESS))
-                conn.sendall(struct.pack("<i", len(server_key)))
-                conn.sendall(server_key.encode("utf-8"))
-                return conn, addr, _parse_server_opt(arr[1:])
+            conn.sendall(struct.pack("<i", base.RPC_CODE_SUCCESS))
+            conn.sendall(struct.pack("<i", len(server_key)))
+            conn.sendall(server_key.encode("utf-8"))
+            return conn, addr, _parse_server_opt(arr[1:])
 
     # Server logic
     tracker_conn = None
@@ -182,9 +214,8 @@ def _listen_loop(sock, port, rpc_key, tracker_addr, load_library, custom_addr):
                 if magic != base.RPC_TRACKER_MAGIC:
                     raise RuntimeError("%s is not RPC Tracker" % str(tracker_addr))
                 # report status of current queue
-                cinfo = {"key" : "server:" + rpc_key}
-                base.sendjson(tracker_conn,
-                              [TrackerCode.UPDATE_INFO, cinfo])
+                cinfo = {"key": "server:" + rpc_key}
+                base.sendjson(tracker_conn, [TrackerCode.UPDATE_INFO, cinfo])
                 assert base.recvjson(tracker_conn) == TrackerCode.SUCCESS
 
             # step 2: wait for in-coming connections
@@ -201,8 +232,9 @@ def _listen_loop(sock, port, rpc_key, tracker_addr, load_library, custom_addr):
         # step 3: serving
         work_path = util.tempdir()
         logger.info("connection from %s", addr)
-        server_proc = multiprocessing.Process(target=_serve_loop,
-                                              args=(conn, addr, load_library, work_path))
+        server_proc = multiprocessing.Process(
+            target=_serve_loop, args=(conn, addr, load_library, work_path)
+        )
         server_proc.deamon = True
         server_proc.start()
         # close from our side.
@@ -211,7 +243,9 @@ def _listen_loop(sock, port, rpc_key, tracker_addr, load_library, custom_addr):
         server_proc.join(opts.get("timeout", None))
         if server_proc.is_alive():
             logger.info("Timeout in RPC session, kill..")
+            # pylint: disable=import-outside-toplevel
             import psutil
+
             parent = psutil.Process(server_proc.pid)
             # terminate worker childs
             for child in parent.children(recursive=True):
@@ -236,7 +270,8 @@ def _connect_proxy_loop(addr, key, load_library):
             magic = struct.unpack("<i", base.recvall(sock, 4))[0]
             if magic == base.RPC_CODE_DUPLICATE:
                 raise RuntimeError("key: %s has already been used in proxy" % key)
-            elif magic == base.RPC_CODE_MISMATCH:
+
+            if magic == base.RPC_CODE_MISMATCH:
                 logger.warning("RPCProxy do not have matching client key %s", key)
             elif magic != base.RPC_CODE_SUCCESS:
                 raise RuntimeError("%s is not RPC Proxy" % str(addr))
@@ -244,8 +279,7 @@ def _connect_proxy_loop(addr, key, load_library):
             remote_key = py_str(base.recvall(sock, keylen))
             opts = _parse_server_opt(remote_key.split()[1:])
             logger.info("connected to %s", str(addr))
-            process = multiprocessing.Process(
-                target=_serve_loop, args=(sock, addr, load_library))
+            process = multiprocessing.Process(target=_serve_loop, args=(sock, addr, load_library))
             process.deamon = True
             process.start()
             sock.close()
@@ -261,11 +295,9 @@ def _connect_proxy_loop(addr, key, load_library):
                 raise RuntimeError("Maximum retry error: last error: %s" % str(err))
             time.sleep(retry_period)
 
+
 def _popen(cmd):
-    proc = subprocess.Popen(cmd,
-                            stdout=subprocess.PIPE,
-                            stderr=subprocess.STDOUT,
-                            env=os.environ)
+    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=os.environ)
     (out, _) = proc.communicate()
     if proc.returncode != 0:
         msg = "Server invoke error:\n"
@@ -317,19 +349,24 @@ class Server(object):
     silent: bool, optional
         Whether run this server in silent mode.
     """
-    def __init__(self,
-                 host,
-                 port=9091,
-                 port_end=9199,
-                 is_proxy=False,
-                 use_popen=False,
-                 tracker_addr=None,
-                 key="",
-                 load_library=None,
-                 custom_addr=None,
-                 silent=False):
+
+    def __init__(
+        self,
+        host,
+        port=9091,
+        port_end=9199,
+        is_proxy=False,
+        use_popen=False,
+        tracker_addr=None,
+        key="",
+        load_library=None,
+        custom_addr=None,
+        silent=False,
+        utvm_dev_id=None,
+        utvm_dev_config_args=None,
+    ):
         try:
-            if base._ServerLoop is None:
+            if _ffi_api.ServerLoop is None:
                 raise RuntimeError("Please compile with USE_RPC=1")
         except NameError:
             raise RuntimeError("Please compile with USE_RPC=1")
@@ -343,20 +380,27 @@ class Server(object):
             logger.setLevel(logging.ERROR)
 
         if use_popen:
-            cmd = [sys.executable,
-                   "-m", "tvm.exec.rpc_server",
-                   "--host=%s" % host,
-                   "--port=%s" % port]
+            cmd = [
+                sys.executable,
+                "-m",
+                "tvm.exec.rpc_server",
+                "--host=%s" % host,
+                "--port=%s" % port,
+                "--port-end=%s" % port_end,
+            ]
             if tracker_addr:
                 assert key
-                cmd += ["--tracker=%s:%d" % tracker_addr,
-                        "--key=%s" % key]
+                cmd += ["--tracker=%s:%d" % tracker_addr, "--key=%s" % key]
             if load_library:
                 cmd += ["--load-library", load_library]
             if custom_addr:
                 cmd += ["--custom-addr", custom_addr]
             if silent:
                 cmd += ["--silent"]
+            if utvm_dev_id is not None:
+                assert utvm_dev_config_args is not None
+                cmd += [f"--utvm-dev-id={utvm_dev_id}"]
+                cmd += [f"--utvm-dev-config-args={utvm_dev_config_args}"]
 
             # prexec_fn is not thread safe and may result in deadlock.
             # python 3.2 introduced the start_new_session parameter as
@@ -367,7 +411,10 @@ class Server(object):
             # interim, stop the pylint diagnostic.
             #
             # pylint: disable=subprocess-popen-preexec-fn
-            self.proc = subprocess.Popen(cmd, preexec_fn=os.setsid)
+            if platform.system() == "Windows":
+                self.proc = subprocess.Popen(cmd, creationflags=subprocess.CREATE_NEW_PROCESS_GROUP)
+            else:
+                self.proc = subprocess.Popen(cmd, preexec_fn=os.setsid)
             time.sleep(0.5)
         elif not is_proxy:
             sock = socket.socket(base.get_addr_family((host, port)), socket.SOCK_STREAM)
@@ -380,22 +427,22 @@ class Server(object):
                 except socket.error as sock_err:
                     if sock_err.errno in [98, 48]:
                         continue
-                    else:
-                        raise sock_err
+                    raise sock_err
             if not self.port:
                 raise ValueError("cannot bind to any port in [%d, %d)" % (port, port_end))
             logger.info("bind to %s:%d", host, self.port)
             sock.listen(1)
             self.sock = sock
             self.proc = multiprocessing.Process(
-                target=_listen_loop, args=(
-                    self.sock, self.port, key, tracker_addr, load_library,
-                    self.custom_addr))
+                target=_listen_loop,
+                args=(self.sock, self.port, key, tracker_addr, load_library, self.custom_addr),
+            )
             self.proc.deamon = True
             self.proc.start()
         else:
             self.proc = multiprocessing.Process(
-                target=_connect_proxy_loop, args=((host, port), key, load_library))
+                target=_connect_proxy_loop, args=((host, port), key, load_library)
+            )
             self.proc.deamon = True
             self.proc.start()
 
