@@ -6,9 +6,9 @@
  * to you under the Apache License, Version 2.0 (the
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
- * 
+ *
  *   http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing,
  * software distributed under the License is distributed on an
  * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
@@ -24,9 +24,9 @@
 
 #include "touch_extractor.h"
 
-#include <set>
 #include <algorithm>
 #include <cmath>
+#include <set>
 #include <unordered_map>
 
 namespace tvm {
@@ -34,9 +34,14 @@ namespace autotvm {
 
 int ParallelLevel(AnnotationType ann) {
   switch (ann) {
-    case kBlockX: case kBlockY: case kBlockZ:
+    case kBlockX:
+    case kBlockY:
+    case kBlockZ:
       return 2;
-    case kThreadX: case kThreadY: case kThreadZ: case kParallel:
+    case kThreadX:
+    case kThreadY:
+    case kThreadZ:
+    case kParallel:
       return 1;
     default:
       return 0;
@@ -44,14 +49,14 @@ int ParallelLevel(AnnotationType ann) {
 }
 
 // get touch pattern from index expression
-class IndexParser: public IRVisitor {
+class IndexParser : public ExprVisitor {
  public:
-  void Parse(Expr expr) {
+  void Parse(PrimExpr expr) {
     pattern_map.clear();
-    this->Visit(expr);
+    this->VisitExpr(expr);
   }
 
-  void Visit_(const Variable *op) {
+  void VisitExpr_(const VarNode* op) final {
     // TODO(lmzheng): handle more index types (multiple occurrence)
     if (pattern_map.count(op) == 0) {
       pattern_map[op] = TouchPattern();
@@ -60,23 +65,23 @@ class IndexParser: public IRVisitor {
     }
   }
 
-  void Visit_(const Mul *op) {
-    if (op->a.as<Variable>()) {
-      if (const auto stride = op->b.as<IntImm>()) {
+  void VisitExpr_(const MulNode* op) final {
+    if (op->a.as<VarNode>()) {
+      if (const auto stride = op->b.as<IntImmNode>()) {
         next_stride_ = stride->value;
       }
     }
-    IRVisitor::Visit_(op);
+    ExprVisitor::VisitExpr_(op);
   }
 
-  std::unordered_map<const Variable*, TouchPattern> pattern_map;
+  std::unordered_map<const VarNode*, TouchPattern> pattern_map;
 
  private:
   int64_t next_stride_ = 1;
 };
 
 // extract iter vars and their touch pattern from ir
-bool TouchExtractor::EnterItervar_(VarExpr var, int64_t length, AnnotationType ann_type) {
+bool TouchExtractor::EnterItervar_(Var var, int64_t length, AnnotationType ann_type) {
   // do not insert duplicated occurrences of virtual thread
   if (ann_type == kVirtualThread && itervar_map.count(var) != 0) {
     skip_stack_size_.push_back(itervar_stack_.size());
@@ -90,16 +95,14 @@ bool TouchExtractor::EnterItervar_(VarExpr var, int64_t length, AnnotationType a
       // these happens when we create tvm.thread_axis("threadIdx.x") once and
       // bind it twice. Here we treat them as two axes
       // so we create a snapshot for the old one and freeze it
-      VarExpr old = VarExpr(var.get()->name_hint);
+      Var old = Var(var.get()->name_hint);
       itervar_map.insert({old, itervar_map[var]});
       itervar_map.erase(var);
     }
 
-    itervar_map.insert({var, ItervarFeature(var, length,
-                                            static_cast<int>(itervar_stack_.size()),
-                                            ann_type,
-                                            topdown_product_,
-                                            static_cast<int>(itervar_counter_++))});
+    itervar_map.insert(
+        {var, ItervarFeature(var, length, static_cast<int>(itervar_stack_.size()), ann_type,
+                             topdown_product_, static_cast<int>(itervar_counter_++))});
   }
 
   return true;
@@ -110,7 +113,7 @@ void TouchExtractor::ExitItervar_() {
     skip_stack_size_.pop_back();
     return;
   }
-  VarExpr var = itervar_stack_.back();
+  Var var = itervar_stack_.back();
 
   // update count and reuse ratio for upper iter vars (includes self)
   for (auto kv : itervar_map[var].touch_feature) {
@@ -120,7 +123,7 @@ void TouchExtractor::ExitItervar_() {
         CHECK(touch_pattern != itervar_map[stack_var].touch_feature.end());
         touch_pattern->second.count *= itervar_map[var].length;
       }
-    } else {                      // multiply reuse ratio
+    } else {  // multiply reuse ratio
       for (auto stack_var : itervar_stack_) {
         auto touch_pattern = itervar_map[stack_var].touch_feature.find(kv.first);
         CHECK(touch_pattern != itervar_map[stack_var].touch_feature.end());
@@ -131,8 +134,7 @@ void TouchExtractor::ExitItervar_() {
   itervar_stack_.pop_back();
 
   int64_t length = itervar_map[var].length;
-  if (length != 0)
-      topdown_product_ /= length;
+  if (length != 0) topdown_product_ /= length;
   int64_t bottomup_product = -1;
   for (auto kv : itervar_map[var].touch_feature) {
     bottomup_product = std::max(bottomup_product, kv.second.count * kv.second.reuse);
@@ -169,7 +171,7 @@ void TouchExtractor::ExitItervar_() {
   }
 }
 
-void TouchExtractor::EnterMem_(VarExpr buffer_var, Expr index) {
+void TouchExtractor::EnterMem_(Var buffer_var, PrimExpr index) {
   std::string name = buffer_var.get()->name_hint;
   TouchedBuffer buf = name + "_" + std::to_string(buffer_counter_[name]++);
 
@@ -188,8 +190,7 @@ void TouchExtractor::EnterMem_(VarExpr buffer_var, Expr index) {
   }
 }
 
-void TouchExtractor::ExitMem_() {
-}
+void TouchExtractor::ExitMem_() {}
 
 /*!
  * \brief Get axis-based feature for all axes
@@ -219,17 +220,17 @@ void TouchExtractor::ExitMem_() {
  * \note If you want to flatten these features as the input of your model,
  * You can use the faster one GetItervarFeatureFlatten below.
  */
-void GetItervarFeature(Stmt stmt, bool take_log, Array<Array<Array<Expr> > > *ret_feature) {
+void GetItervarFeature(Stmt stmt, bool take_log, Array<Array<Array<PrimExpr> > >* ret_feature) {
   // extract
   TouchExtractor touch_analyzer;
   touch_analyzer.Analyze(stmt);
 
   // sort according to order
-  std::vector<VarExpr> vars;
+  std::vector<Var> vars;
   for (auto kv : touch_analyzer.itervar_map) {
     vars.push_back(kv.first);
   }
-  std::sort(vars.begin(), vars.end(), [&](const VarExpr &lhs, const VarExpr &rhs) -> bool {
+  std::sort(vars.begin(), vars.end(), [&](const Var& lhs, const Var& rhs) -> bool {
     return touch_analyzer.itervar_map[lhs].order < touch_analyzer.itervar_map[rhs].order;
   });
 
@@ -237,28 +238,26 @@ void GetItervarFeature(Stmt stmt, bool take_log, Array<Array<Array<Expr> > > *re
   std::function<double(int64_t)> trans;
   if (take_log) {
     trans = [](int64_t x) {
-      if (x < 0)
-        return -std::log(-x+1) / std::log(2);
+      if (x < 0) return -std::log(-x + 1) / std::log(2);
       x = x + 1;
       return std::log(x) / std::log(2);
     };
   } else {
-    trans = [](int64_t x) {
-      return x;
-    };
+    trans = [](int64_t x) { return x; };
   }
 
   // serialize for front end
   for (auto var : vars) {
-    Array<Array<Expr> > feature_row;
-    ItervarFeature &fea = touch_analyzer.itervar_map[var];
-    feature_row.push_back(Array<Expr>{std::string("_itervar_"), var});
+    Array<Array<PrimExpr> > feature_row;
+    ItervarFeature& fea = touch_analyzer.itervar_map[var];
+    feature_row.push_back(Array<PrimExpr>{tvm::tir::StringImm("_itervar_"), var});
 
-    Array<Expr> attr{std::string("_attr_"),
-                     FloatImm::make(Float(32), trans(fea.length)),
-                     IntImm::make(Int(32), fea.nest_level),
-                     FloatImm::make(Float(32), trans(fea.topdown_product)),
-                     FloatImm::make(Float(32), trans(fea.bottomup_product)),
+    Array<PrimExpr> attr{
+        tvm::tir::StringImm("_attr_"),
+        FloatImm(DataType::Float(32), trans(fea.length)),
+        IntImm(DataType::Int(32), fea.nest_level),
+        FloatImm(DataType::Float(32), trans(fea.topdown_product)),
+        FloatImm(DataType::Float(32), trans(fea.bottomup_product)),
     };
     // one hot annotation
     for (int i = 0; i < kNum; i++) {
@@ -267,10 +266,11 @@ void GetItervarFeature(Stmt stmt, bool take_log, Array<Array<Array<Expr> > > *re
     feature_row.push_back(attr);
 
     // arithmetic
-    feature_row.push_back(Array<Expr>{std::string("_arith_"),
-                                      FloatImm::make(Float(32), trans(fea.add_ct)),
-                                      FloatImm::make(Float(32), trans(fea.mul_ct)),
-                                      FloatImm::make(Float(32), trans(fea.div_ct)),
+    feature_row.push_back(Array<PrimExpr>{
+        tvm::tir::StringImm("_arith_"),
+        FloatImm(DataType::Float(32), trans(fea.add_ct)),
+        FloatImm(DataType::Float(32), trans(fea.mul_ct)),
+        FloatImm(DataType::Float(32), trans(fea.div_ct)),
     });
 
     // touch map
@@ -280,14 +280,15 @@ void GetItervarFeature(Stmt stmt, bool take_log, Array<Array<Array<Expr> > > *re
     }
     std::sort(bufs.begin(), bufs.end());
     for (auto k : bufs) {
-      TouchPattern &v = fea.touch_feature[k];
-      feature_row.push_back(Array<Expr>{k,
-                                        FloatImm::make(Float(32), trans(v.stride)),
-                                        FloatImm::make(Float(32), trans(v.mod)),
-                                        FloatImm::make(Float(32), trans(v.count)),
-                                        FloatImm::make(Float(32), trans(v.reuse)),
-                                        FloatImm::make(Float(32), trans(v.thread_count)),
-                                        FloatImm::make(Float(32), trans(v.thread_reuse)),
+      TouchPattern& v = fea.touch_feature[k];
+      feature_row.push_back(Array<PrimExpr>{
+          tvm::tir::StringImm(k),
+          FloatImm(DataType::Float(32), trans(v.stride)),
+          FloatImm(DataType::Float(32), trans(v.mod)),
+          FloatImm(DataType::Float(32), trans(v.count)),
+          FloatImm(DataType::Float(32), trans(v.reuse)),
+          FloatImm(DataType::Float(32), trans(v.thread_count)),
+          FloatImm(DataType::Float(32), trans(v.thread_reuse)),
       });
     }
 
@@ -304,17 +305,17 @@ void GetItervarFeature(Stmt stmt, bool take_log, Array<Array<Array<Expr> > > *re
  * \note See GetItervarFeature for more details about the return value.
  *       This is an optimized version of GetItervarFeature + Flatten. This runs much faster.
  */
-void GetItervarFeatureFlatten(Stmt stmt, bool take_log, std::vector<float> *ret_feature) {
+void GetItervarFeatureFlatten(Stmt stmt, bool take_log, std::vector<float>* ret_feature) {
   // extract touch feature
   TouchExtractor touch_analyzer;
   touch_analyzer.Analyze(stmt);
 
   // sort according to order
-  std::vector<VarExpr> vars;
+  std::vector<Var> vars;
   for (auto kv : touch_analyzer.itervar_map) {
     vars.push_back(kv.first);
   }
-  std::sort(vars.begin(), vars.end(), [&](const VarExpr &lhs, const VarExpr &rhs) -> bool {
+  std::sort(vars.begin(), vars.end(), [&](const Var& lhs, const Var& rhs) -> bool {
     return touch_analyzer.itervar_map[lhs].order < touch_analyzer.itervar_map[rhs].order;
   });
 
@@ -322,20 +323,17 @@ void GetItervarFeatureFlatten(Stmt stmt, bool take_log, std::vector<float> *ret_
   std::function<float(int64_t)> trans;
   if (take_log) {
     trans = [](int64_t x) {
-      if (x < 0)
-        return -std::log(-x+1) / std::log(2);
+      if (x < 0) return -std::log(-x + 1) / std::log(2);
       x = x + 1;
       return std::log(x) / std::log(2);
     };
   } else {
-    trans = [](int64_t x) {
-      return x;
-    };
+    trans = [](int64_t x) { return x; };
   }
 
   // serialize for front end
   for (auto var : vars) {
-    ItervarFeature &fea = touch_analyzer.itervar_map[var];
+    ItervarFeature& fea = touch_analyzer.itervar_map[var];
 
     ret_feature->push_back(trans(fea.length));
     ret_feature->push_back(fea.nest_level);
@@ -359,7 +357,7 @@ void GetItervarFeatureFlatten(Stmt stmt, bool take_log, std::vector<float> *ret_
     }
     std::sort(bufs.begin(), bufs.end());
     for (auto k : bufs) {
-      TouchPattern &v = fea.touch_feature[k];
+      TouchPattern& v = fea.touch_feature[k];
       ret_feature->push_back(trans(v.stride));
       ret_feature->push_back(trans(v.mod));
       ret_feature->push_back(trans(v.count));
@@ -371,22 +369,22 @@ void GetItervarFeatureFlatten(Stmt stmt, bool take_log, std::vector<float> *ret_
 }
 
 /*!
- * \brief Get curve sample feature (relation feature) and flatten them into a one-dimensional vector.
- * \param stmt The statement to be extracted
- * \param sample_n The number of points used for sampling a curve (along one dimension)
- * \param ret_feature The buffer where the return value is stored
+ * \brief Get curve sample feature (relation feature) and flatten them into a one-dimensional
+ * vector. \param stmt The statement to be extracted \param sample_n The number of points used for
+ * sampling a curve (along one dimension) \param ret_feature The buffer where the return value is
+ * stored
  */
-void GetCurveSampleFeatureFlatten(Stmt stmt, int sample_n, std::vector<float> *ret_feature) {
+void GetCurveSampleFeatureFlatten(Stmt stmt, int sample_n, std::vector<float>* ret_feature) {
   // extract touch feature
   TouchExtractor touch_ext;
   touch_ext.Analyze(stmt);
 
   // sort according to order
-  std::vector<VarExpr> vars;
+  std::vector<Var> vars;
   for (auto kv : touch_ext.itervar_map) {
     vars.push_back(kv.first);
   }
-  std::sort(vars.begin(), vars.end(), [&](const VarExpr &lhs, const VarExpr &rhs) -> bool {
+  std::sort(vars.begin(), vars.end(), [&](const Var& lhs, const Var& rhs) -> bool {
     return touch_ext.itervar_map[lhs].order < touch_ext.itervar_map[rhs].order;
   });
 
@@ -400,14 +398,14 @@ void GetCurveSampleFeatureFlatten(Stmt stmt, int sample_n, std::vector<float> *r
 
   // find maximum depth of loop nest
   for (auto var : vars) {
-    ItervarFeature &fea = touch_ext.itervar_map[var];
+    ItervarFeature& fea = touch_ext.itervar_map[var];
     max_depth = std::max(max_depth, fea.nest_level);
   }
 
   // mark inner most buffer
   for (auto iter = vars.rbegin(); iter != vars.rend(); iter++) {
     auto var = *iter;
-    ItervarFeature &fea = touch_ext.itervar_map[var];
+    ItervarFeature& fea = touch_ext.itervar_map[var];
     if (fea.nest_level == max_depth) {
       for (auto kv : fea.touch_feature) {
         // delete buffer no (e.g. 'A_0' -> 'A', 'A_1' -> 'A')
@@ -415,8 +413,7 @@ void GetCurveSampleFeatureFlatten(Stmt stmt, int sample_n, std::vector<float> *r
 
         // delete memory scope (e.g. 'A.local' -> 'A', 'A.shared' -> 'A')
         size_t pos = raw_name.find(".");
-        if (pos < kv.first.size())
-          raw_name = raw_name.substr(0, pos);
+        if (pos < kv.first.size()) raw_name = raw_name.substr(0, pos);
 
         // If there are multiple innermost buffers that are derived from a same raw buffer
         // We only record the last occurrence (note the `iter` is in reverse order)
@@ -440,7 +437,7 @@ void GetCurveSampleFeatureFlatten(Stmt stmt, int sample_n, std::vector<float> *r
 
   // extract curves
   for (auto var : vars) {
-    ItervarFeature &fea = touch_ext.itervar_map[var];
+    ItervarFeature& fea = touch_ext.itervar_map[var];
     for (auto kv : fea.touch_feature) {
       if (innermost_buffers.find(kv.first) != innermost_buffers.end()) {
         reuse_curve[kv.first].emplace_back(std::log(kv.second.reuse) / std::log(2));
@@ -452,7 +449,7 @@ void GetCurveSampleFeatureFlatten(Stmt stmt, int sample_n, std::vector<float> *r
   }
 
   // sample relation in the curve
-  auto sample_curve = [&](const std::vector<double> &x, const std::vector<double> &y,
+  auto sample_curve = [&](const std::vector<double>& x, const std::vector<double>& y,
                           double weight) {
     for (int i = 0; i < sample_n; i++) {
       double xx = i * weight;
@@ -468,9 +465,9 @@ void GetCurveSampleFeatureFlatten(Stmt stmt, int sample_n, std::vector<float> *r
 
   // serialize to frontend
   for (auto k : innermost_buffers) {
-    std::vector<double> &count = count_curve[k];
-    std::vector<double> &reuse = reuse_curve[k];
-    std::vector<double> &top_down = topdown_curve[k];
+    std::vector<double>& count = count_curve[k];
+    std::vector<double>& reuse = reuse_curve[k];
+    std::vector<double>& top_down = topdown_curve[k];
 
     std::sort(count.begin(), count.end());
     std::sort(reuse.begin(), reuse.end());
@@ -483,49 +480,45 @@ void GetCurveSampleFeatureFlatten(Stmt stmt, int sample_n, std::vector<float> *r
   }
 }
 
-
 // register API for front end
-TVM_REGISTER_API("autotvm.feature.GetItervarFeature")
-.set_body([](TVMArgs args, TVMRetValue *ret) {
-  Stmt stmt = args[0];
-  bool take_log = args[1];
-  Array<Array<Array<Expr > > > ret_feature;
+TVM_REGISTER_GLOBAL("autotvm.feature.GetItervarFeature")
+    .set_body([](TVMArgs args, TVMRetValue* ret) {
+      Stmt stmt = args[0];
+      bool take_log = args[1];
+      Array<Array<Array<PrimExpr> > > ret_feature;
 
-  GetItervarFeature(stmt, take_log, &ret_feature);
+      GetItervarFeature(stmt, take_log, &ret_feature);
 
-  *ret = ret_feature;
-});
+      *ret = ret_feature;
+    });
 
+TVM_REGISTER_GLOBAL("autotvm.feature.GetItervarFeatureFlatten")
+    .set_body([](TVMArgs args, TVMRetValue* ret) {
+      Stmt stmt = args[0];
+      bool take_log = args[1];
+      std::vector<float> ret_feature;
 
-TVM_REGISTER_API("autotvm.feature.GetItervarFeatureFlatten")
-.set_body([](TVMArgs args, TVMRetValue *ret) {
-  Stmt stmt = args[0];
-  bool take_log = args[1];
-  std::vector<float> ret_feature;
+      GetItervarFeatureFlatten(stmt, take_log, &ret_feature);
 
-  GetItervarFeatureFlatten(stmt, take_log, &ret_feature);
+      TVMByteArray arr;
+      arr.size = sizeof(float) * ret_feature.size();
+      arr.data = reinterpret_cast<char*>(ret_feature.data());
+      *ret = arr;
+    });
 
-  TVMByteArray arr;
-  arr.size = sizeof(float) * ret_feature.size();
-  arr.data = reinterpret_cast<char *>(ret_feature.data());
-  *ret = arr;
-});
+TVM_REGISTER_GLOBAL("autotvm.feature.GetCurveSampleFeatureFlatten")
+    .set_body([](TVMArgs args, TVMRetValue* ret) {
+      Stmt stmt = args[0];
+      int sample_n = args[1];
+      std::vector<float> ret_feature;
 
+      GetCurveSampleFeatureFlatten(stmt, sample_n, &ret_feature);
 
-TVM_REGISTER_API("autotvm.feature.GetCurveSampleFeatureFlatten")
-.set_body([](TVMArgs args, TVMRetValue *ret) {
-  Stmt stmt = args[0];
-  int sample_n = args[1];
-  std::vector<float> ret_feature;
-
-  GetCurveSampleFeatureFlatten(stmt, sample_n, &ret_feature);
-
-  TVMByteArray arr;
-  arr.size = sizeof(float) * ret_feature.size();
-  arr.data = reinterpret_cast<char *>(ret_feature.data());
-  *ret = arr;
-});
-
+      TVMByteArray arr;
+      arr.size = sizeof(float) * ret_feature.size();
+      arr.data = reinterpret_cast<char*>(ret_feature.data());
+      *ret = arr;
+    });
 
 }  // namespace autotvm
 }  // namespace tvm
