@@ -18,18 +18,20 @@
  */
 
 /*!
- * \file src/tvm/relay/interpreter.cc
+ * \file src/relay/interpreter.cc
  * \brief An interpreter for the Relay IR.
  */
-#include <tvm/packed_func_ext.h>
-#include <tvm/runtime/device_api.h>
-#include <tvm/relay/expr_functor.h>
-#include <tvm/relay/pattern_functor.h>
-#include <tvm/relay/interpreter.h>
-#include <tvm/relay/transform.h>
+#include <tvm/driver/driver_api.h>
 #include <tvm/relay/analysis.h>
 #include <tvm/relay/attrs/debug.h>
+#include <tvm/relay/expr_functor.h>
 #include <tvm/relay/feature.h>
+#include <tvm/relay/interpreter.h>
+#include <tvm/relay/pattern_functor.h>
+#include <tvm/relay/transform.h>
+#include <tvm/runtime/device_api.h>
+#include <tvm/runtime/object.h>
+
 #include "compile_engine.h"
 
 namespace tvm {
@@ -37,129 +39,90 @@ namespace relay {
 
 using namespace runtime;
 
+InterpreterClosure::InterpreterClosure(tvm::Map<Var, ObjectRef> env, Function func) {
+  ObjectPtr<InterpreterClosureObj> n = make_object<InterpreterClosureObj>();
+  n->env = std::move(env);
+  n->func = std::move(func);
+  data_ = std::move(n);
+}
+
+TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
+    .set_dispatch<InterpreterClosureObj>([](const ObjectRef& ref, ReprPrinter* p) {
+      auto* node = static_cast<const InterpreterClosureObj*>(ref.get());
+      p->stream << "InterpreterClosureNode(" << node->func << ", " << node->env << ")";
+    });
+
 inline const PackedFunc& GetPackedFunc(const std::string& name) {
   const PackedFunc* pf = tvm::runtime::Registry::Get(name);
   CHECK(pf != nullptr) << "Cannot find function " << name << " in registry";
   return *pf;
 }
 
-/* Value Implementation */
-Closure ClosureNode::make(tvm::Map<Var, Value> env, Function func) {
-  NodePtr<ClosureNode> n = make_node<ClosureNode>();
-  n->env = std::move(env);
-  n->func = std::move(func);
-  return Closure(n);
-}
-
-TVM_REGISTER_API("relay._make.Closure")
-.set_body_typed(ClosureNode::make);
-
-TVM_STATIC_IR_FUNCTOR(IRPrinter, vtable)
-.set_dispatch<ClosureNode>([](const ObjectRef& ref, IRPrinter* p) {
-    auto* node = static_cast<const ClosureNode*>(ref.get());
-    p->stream << "ClosureNode(" << node->func << ", " << node->env << ")";
-  });
-
-
 // TODO(@jroesch): this doesn't support mutual letrec
-/* Value Implementation */
-RecClosure RecClosureNode::make(Closure clos, Var bind) {
-  NodePtr<RecClosureNode> n = make_node<RecClosureNode>();
+/* Object Implementation */
+RecClosure::RecClosure(InterpreterClosure clos, Var bind) {
+  ObjectPtr<RecClosureObj> n = make_object<RecClosureObj>();
   n->clos = std::move(clos);
   n->bind = std::move(bind);
-  return RecClosure(n);
+  data_ = std::move(n);
 }
 
-TVM_REGISTER_API("relay._make.RecClosure")
-.set_body_typed(RecClosureNode::make);
+TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
+    .set_dispatch<RecClosureObj>([](const ObjectRef& ref, ReprPrinter* p) {
+      auto* node = static_cast<const RecClosureObj*>(ref.get());
+      p->stream << "RecClosureObj(" << node->clos << ")";
+    });
 
-TVM_STATIC_IR_FUNCTOR(IRPrinter, vtable)
-.set_dispatch<RecClosureNode>([](const ObjectRef& ref, IRPrinter* p) {
-    auto* node = static_cast<const RecClosureNode*>(ref.get());
-    p->stream << "RecClosureNode(" << node->clos << ")";
-  });
-
-TupleValue TupleValueNode::make(tvm::Array<Value> value) {
-  NodePtr<TupleValueNode> n = make_node<TupleValueNode>();
-  n->fields = value;
-  return TupleValue(n);
-}
-
-TVM_REGISTER_API("relay._make.TupleValue")
-.set_body_typed(TupleValueNode::make);
-
-TVM_STATIC_IR_FUNCTOR(IRPrinter, vtable)
-.set_dispatch<TupleValueNode>([](const ObjectRef& ref, IRPrinter* p) {
-    auto* node = static_cast<const TupleValueNode*>(ref.get());
-    p->stream << "TupleValueNode(" << node->fields << ")";
-  });
-
-TensorValue TensorValueNode::make(runtime::NDArray data) {
-  NodePtr<TensorValueNode> n = make_node<TensorValueNode>();
-  n->data = std::move(data);
-  return TensorValue(n);
-}
-
-TVM_STATIC_IR_FUNCTOR(IRPrinter, vtable)
-.set_dispatch<TensorValueNode>([](const ObjectRef& ref, IRPrinter* p) {
-    auto* node = static_cast<const TensorValueNode*>(ref.get());
-    auto to_str = GetPackedFunc("relay._tensor_value_repr");
-    std::string data_str = to_str(GetRef<TensorValue>(node));
-    p->stream << "TensorValueNode(" << data_str << ")";
-  });
-
-TVM_REGISTER_API("relay._make.TensorValue")
-.set_body_typed(TensorValueNode::make);
-
-RefValue RefValueNode::make(Value value) {
-  NodePtr<RefValueNode> n = make_node<RefValueNode>();
+RefValue::RefValue(ObjectRef value) {
+  ObjectPtr<RefValueObj> n = make_object<RefValueObj>();
   n->value = value;
-  return RefValue(n);
+  data_ = std::move(n);
 }
 
-TVM_REGISTER_API("relay._make.RefValue")
-.set_body_typed(RefValueNode::make);
+TVM_REGISTER_GLOBAL("relay._make.RefValue").set_body_typed([](ObjectRef value) {
+  return RefValue(value);
+});
 
-TVM_REGISTER_NODE_TYPE(RefValueNode);
+TVM_REGISTER_NODE_TYPE(RefValueObj);
 
-TVM_STATIC_IR_FUNCTOR(IRPrinter, vtable)
-.set_dispatch<RefValueNode>([](const ObjectRef& ref, IRPrinter* p) {
-    auto* node = static_cast<const RefValueNode*>(ref.get());
-    p->stream << "RefValueNode(" << node->value << ")";
-  });
+TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
+    .set_dispatch<RefValueObj>([](const ObjectRef& ref, ReprPrinter* p) {
+      auto* node = static_cast<const RefValueObj*>(ref.get());
+      p->stream << "RefValueObj(" << node->value << ")";
+    });
 
-ConstructorValue ConstructorValueNode::make(int32_t tag,
-                                            tvm::Array<Value> fields,
-                                            Constructor constructor) {
-  NodePtr<ConstructorValueNode> n = make_node<ConstructorValueNode>();
+ConstructorValue::ConstructorValue(int32_t tag, tvm::Array<ObjectRef> fields,
+                                   Constructor constructor) {
+  ObjectPtr<ConstructorValueObj> n = make_object<ConstructorValueObj>();
   n->tag = tag;
   n->fields = fields;
   n->constructor = constructor;
-  return ConstructorValue(n);
+  data_ = std::move(n);
 }
 
-TVM_REGISTER_API("relay._make.ConstructorValue")
-.set_body_typed(ConstructorValueNode::make);
+TVM_REGISTER_GLOBAL("relay._make.ConstructorValue")
+    .set_body_typed([](int32_t tag, tvm::Array<ObjectRef> fields, Constructor constructor) {
+      return ConstructorValue(tag, fields, constructor);
+    });
 
-TVM_REGISTER_NODE_TYPE(ConstructorValueNode);
+TVM_REGISTER_NODE_TYPE(ConstructorValueObj);
 
-TVM_STATIC_IR_FUNCTOR(IRPrinter, vtable)
-.set_dispatch<ConstructorValueNode>([](const ObjectRef& ref, IRPrinter* p) {
-  auto* node = static_cast<const ConstructorValueNode*>(ref.get());
-  p->stream << "ConstructorValueNode(" << node->tag << ","
-            << node->fields << ")";
-});
+TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
+    .set_dispatch<ConstructorValueObj>([](const ObjectRef& ref, ReprPrinter* p) {
+      auto* node = static_cast<const ConstructorValueObj*>(ref.get());
+      p->stream << "ConstructorValueObj(" << node->tag << "," << node->fields << ")";
+    });
 
 /*!
  * \brief A stack frame in the Relay interpreter.
  *
- * Contains a mapping from relay::Var to relay::Value.
+ * Contains a mapping from relay::Var to relay::ObjectRef.
  */
 struct Frame {
   /*! \brief The set of local variables and arguments for the frame. */
-  tvm::Map<Var, Value> locals;
+  tvm::Map<Var, ObjectRef> locals;
 
-  explicit Frame(tvm::Map<Var, Value> locals) : locals(locals) {}
+  explicit Frame(tvm::Map<Var, ObjectRef> locals) : locals(locals) {}
 };
 
 /*!
@@ -175,7 +138,7 @@ struct Stack {
 
   Frame& current_frame() { return frames.back(); }
 
-  Value Lookup(const Var& local) {
+  ObjectRef Lookup(const Var& local) {
     for (auto frame = frames.rbegin(); frame != frames.rend(); frame++) {
       auto elem = frame->locals.find(local);
       if (elem != frame->locals.end()) {
@@ -185,7 +148,7 @@ struct Stack {
 
     LOG(FATAL) << "could not find variable binding for " << local
                << "address= " << local.operator->();
-    return Value();
+    return ObjectRef();
   }
   /*!
    * A wrapper around Frame to add RAII semantics to pushing and popping
@@ -193,9 +156,7 @@ struct Stack {
    */
   struct LocalFrame {
     Stack& st;
-    explicit LocalFrame(Stack& st, const Frame& fr) : st(st) {
-      st.frames.push_back(fr);
-    }
+    explicit LocalFrame(Stack& st, const Frame& fr) : st(st) { st.frames.push_back(fr); }
     ~LocalFrame() { st.frames.pop_back(); }
   };
 };
@@ -204,9 +165,9 @@ struct Stack {
 class InterpreterState;
 
 /*! \brief A container capturing the state of the interpreter. */
-class InterpreterStateNode : public Node {
+class InterpreterStateObj : public Object {
  public:
-  using Frame = tvm::Map<Var, Value>;
+  using Frame = tvm::Map<Var, ObjectRef>;
   using Stack = tvm::Array<Frame>;
 
   /*! \brief The current expression under evaluation. */
@@ -220,19 +181,25 @@ class InterpreterStateNode : public Node {
     v->Visit("stack", &stack);
   }
 
-  static InterpreterState make(Expr current_expr, Stack stack);
-
   static constexpr const char* _type_key = "relay.InterpreterState";
-  TVM_DECLARE_NODE_TYPE_INFO(InterpreterStateNode, Node);
+  TVM_DECLARE_FINAL_OBJECT_INFO(InterpreterStateObj, Object);
 };
 
-RELAY_DEFINE_NODE_REF(InterpreterState, InterpreterStateNode, NodeRef);
+class InterpreterState : public ObjectRef {
+ public:
+  using Frame = tvm::Map<Var, ObjectRef>;
+  using Stack = tvm::Array<Frame>;
 
-InterpreterState InterpreterStateNode::make(Expr current_expr, Stack stack) {
-  NodePtr<InterpreterStateNode> n = make_node<InterpreterStateNode>();
+  InterpreterState(Expr current_expr, Stack stack);
+
+  TVM_DEFINE_OBJECT_REF_METHODS(InterpreterState, ObjectRef, InterpreterStateObj);
+};
+
+InterpreterState::InterpreterState(Expr current_expr, InterpreterState::Stack stack) {
+  ObjectPtr<InterpreterStateObj> n = make_object<InterpreterStateObj>();
   n->current_expr = std::move(current_expr);
   n->stack = std::move(stack);
-  return InterpreterState(n);
+  data_ = std::move(n);
 }
 
 // NOTE: the current interpreter assumes A-normal form.
@@ -242,14 +209,11 @@ InterpreterState InterpreterStateNode::make(Expr current_expr, Stack stack) {
 // contains DAG in dataflow-form.
 //
 // Conversion to ANF is recommended before running the interpretation.
-class Interpreter :
-      public ExprFunctor<Value(const Expr& n)>,
-             PatternFunctor<bool(const Pattern& p, const Value& v)> {
+class Interpreter : public ExprFunctor<ObjectRef(const Expr& n)>,
+                    PatternFunctor<bool(const Pattern& p, const ObjectRef& v)> {
  public:
-  Interpreter(Module mod,
-              DLContext context,
-              Target target)
-      : mod_(mod), context_(context), target_(target) {
+  Interpreter(IRModule mod, DLContext context, Target target)
+      : mod_(mod), context_(context), target_(target), debug_op_(Op::Get("debug")) {
     engine_ = CompileEngine::Global();
   }
 
@@ -259,56 +223,41 @@ class Interpreter :
     return f();
   }
 
-  void extend(const Var& id, Value v) {
-    stack_.current_frame().locals.Set(id, v);
-  }
+  void extend(const Var& id, ObjectRef v) { stack_.current_frame().locals.Set(id, v); }
 
-  inline Value Lookup(const Var& local) {
-    return stack_.Lookup(local);
-  }
+  ObjectRef Lookup(const Var& local) { return stack_.Lookup(local); }
 
-  Value Eval(const Expr& expr) {
-    return VisitExpr(expr);
-  }
+  ObjectRef Eval(const Expr& expr) { return VisitExpr(expr); }
 
-  Value VisitExpr(const Expr& expr) final {
-    auto ret = ExprFunctor<Value(const Expr& n)>::VisitExpr(expr);
-    return ret;
-  }
+  ObjectRef VisitExpr_(const VarNode* var_node) final { return Lookup(GetRef<Var>(var_node)); }
 
-  Value VisitExpr_(const VarNode* var_node) final {
-    return Lookup(GetRef<Var>(var_node));
-  }
-
-  Value VisitExpr_(const GlobalVarNode* op) final {
+  ObjectRef VisitExpr_(const GlobalVarNode* op) final {
     return Eval(mod_->Lookup(GetRef<GlobalVar>(op)));
   }
 
-  Value VisitExpr_(const OpNode* id) override {
+  ObjectRef VisitExpr_(const OpNode* id) override {
     // TODO(@jroesch): Eta-expand and return in this case.
     LOG(FATAL) << "internal error, need to wrap intrinsic into call synthetic call node "
                << "in "
                << "this case, eta expand";
-    return Value();
+    return ObjectRef();
   }
 
-  Value VisitExpr_(const ConstantNode* op) final {
-    return TensorValueNode::make(op->data.CopyTo(context_));
-  }
+  ObjectRef VisitExpr_(const ConstantNode* op) final { return op->data.CopyTo(context_); }
 
-  Value VisitExpr_(const TupleNode* op) final {
-    std::vector<Value> values;
+  ObjectRef VisitExpr_(const TupleNode* op) final {
+    std::vector<ObjectRef> values;
 
     for (const auto& field : op->fields) {
-      Value field_value = Eval(field);
+      ObjectRef field_value = Eval(field);
       values.push_back(field_value);
     }
 
-    return TupleValueNode::make(values);
+    return ADT::Tuple(values);
   }
 
-  inline Value MakeClosure(const Function& func, Var letrec_name = Var()) {
-    tvm::Map<Var, Value> captured_mod;
+  ObjectRef MakeClosure(const Function& func, Var letrec_name = Var()) {
+    tvm::Map<Var, ObjectRef> captured_mod;
     Array<Var> free_vars = FreeVars(func);
 
     for (const auto& var : free_vars) {
@@ -322,21 +271,20 @@ class Interpreter :
     }
 
     // We must use mutation here to build a self referential closure.
-    auto closure = ClosureNode::make(captured_mod, func);
+    InterpreterClosure closure(captured_mod, func);
     if (letrec_name.defined()) {
-      return RecClosureNode::make(closure, letrec_name);
+      return RecClosure(closure, letrec_name);
     }
     return std::move(closure);
   }
 
-  Value VisitExpr_(const FunctionNode* func_node) final {
+  ObjectRef VisitExpr_(const FunctionNode* func_node) final {
     auto func = GetRef<Function>(func_node);
     return MakeClosure(func);
   }
 
-  Array<Shape> ComputeDynamicShape(const Function& func,
-                                   const Array<Value>& args) {
-    auto key = CCacheKeyNode::make(func, Target::Create("llvm"));
+  Array<Shape> ComputeDynamicShape(const Function& func, const Array<ObjectRef>& args) {
+    CCacheKey key(func, Target("llvm"));
     auto cfunc = engine_->LowerShapeFunc(key);
     size_t arity = cfunc->inputs.size() + cfunc->outputs.size();
 
@@ -350,28 +298,27 @@ class Interpreter :
     cpu_ctx.device_type = kDLCPU;
     cpu_ctx.device_id = 0;
 
-    auto fset_input = [&](size_t i, Value val, bool need_shape) {
-        const TensorValueNode* tv = val.as<TensorValueNode>();
-        CHECK(tv != nullptr) << "expect Tensor argument";
-        if (need_shape) {
-          int64_t ndim = tv->data.Shape().size();
-          NDArray shape_arr;
-          if (ndim == 0) {
-            shape_arr = NDArray::Empty({}, Type2TVMType(Int(64)), cpu_ctx);
-          } else {
-            shape_arr = NDArray::Empty({ndim}, Type2TVMType(Int(64)), cpu_ctx);
-            int64_t* data = reinterpret_cast<int64_t*>(shape_arr->data);
-            for (auto j = 0; j < ndim; ++j) {
-              data[j] = tv->data.Shape()[j];
-            }
-          }
-          inputs[i] = shape_arr;
-          setter(i, shape_arr);
+    auto fset_input = [&](size_t i, ObjectRef val, bool need_shape) {
+      auto nd_array = Downcast<NDArray>(val);
+      if (need_shape) {
+        int64_t ndim = nd_array.Shape().size();
+        NDArray shape_arr;
+        if (ndim == 0) {
+          shape_arr = NDArray::Empty({}, DataType::Int(64), cpu_ctx);
         } else {
-          auto arr = tv->data.CopyTo(cpu_ctx);
-          inputs[i] = arr;
-          setter(i, arr);
+          shape_arr = NDArray::Empty({ndim}, DataType::Int(64), cpu_ctx);
+          int64_t* data = reinterpret_cast<int64_t*>(shape_arr->data);
+          for (auto j = 0; j < ndim; ++j) {
+            data[j] = nd_array.Shape()[j];
+          }
         }
+        inputs[i] = shape_arr;
+        setter(i, shape_arr);
+      } else {
+        auto arr = nd_array.CopyTo(cpu_ctx);
+        inputs[i] = arr;
+        setter(i, arr);
+      }
     };
 
     size_t arg_counter = 0;
@@ -379,7 +326,7 @@ class Interpreter :
       auto arg = args[i];
       auto param = func->params[i];
       int state = cfunc->shape_func_param_states[i]->value;
-      if (arg.as<TensorValueNode>()) {
+      if (arg->IsInstance<runtime::NDArray::ContainerType>()) {
         if (state & kNeedInputData) {
           fset_input(arg_counter++, arg, false);
         }
@@ -387,31 +334,29 @@ class Interpreter :
           fset_input(arg_counter++, arg, true);
         }
       } else {
-        const TupleValueNode* tuple = arg.as<TupleValueNode>();
-        CHECK(tuple != nullptr);
+        const ADT adt = Downcast<ADT>(arg);
         if (state & kNeedInputData) {
-          for (size_t i = 0; i < tuple->fields.size(); ++i) {
-            fset_input(arg_counter++, tuple->fields[i], false);
+          for (size_t i = 0; i < adt.size(); ++i) {
+            fset_input(arg_counter++, adt[i], false);
           }
         }
         if (state & kNeedInputShape) {
-          for (size_t i = 0; i < tuple->fields.size(); ++i) {
-            fset_input(arg_counter++, tuple->fields[i], true);
+          for (size_t i = 0; i < adt.size(); ++i) {
+            fset_input(arg_counter++, adt[i], true);
           }
         }
       }
     }
-    CHECK_EQ(arg_counter, cfunc->inputs.size())
-      << "Shape function input sizes mismatch";
+    CHECK_EQ(arg_counter, cfunc->inputs.size()) << "Shape function input sizes mismatch";
 
     auto fset_shape_output = [&](size_t i, Type val_type) {
-        // TODO(@icemelon): allow recursive tuple
-        const TensorTypeNode* rtype = val_type.as<TensorTypeNode>();
-        CHECK(rtype != nullptr);
-        int64_t ndim = rtype->shape.size();
-        auto arr = NDArray::Empty({ndim}, Type2TVMType(Int(64)), cpu_ctx);
-        outputs[i] = arr;
-        setter(arg_counter + i, arr);
+      // TODO(@icemelon): allow recursive tuple
+      const TensorTypeNode* rtype = val_type.as<TensorTypeNode>();
+      CHECK(rtype != nullptr);
+      int64_t ndim = rtype->shape.size();
+      auto arr = NDArray::Empty({ndim}, DataType::Int(64), cpu_ctx);
+      outputs[i] = arr;
+      setter(arg_counter + i, arr);
     };
 
     auto ret_type = func->body->checked_type();
@@ -426,17 +371,17 @@ class Interpreter :
       auto tt = Downcast<TensorType>(ret_type);
       fset_shape_output(0, tt);
     }
-    CHECK_EQ(cfunc->outputs.size(), out_cnt)
-      << "Shape function output sizes mismatch";
+    CHECK_EQ(cfunc->outputs.size(), out_cnt) << "Shape function output sizes mismatch";
 
     PackedFunc shape_func;
+    Module m;
     TVMRetValue rv;
     if (const auto* f = runtime::Registry::Get("relay.backend.build")) {
-      tvm::runtime::Module m = (*f)(cfunc->funcs, cfunc->target);
-      shape_func = m.GetFunction(cfunc->func_name);
+      m = (*f)(cfunc->funcs, cfunc->target);
     } else {
-      LOG(FATAL) << "relay.backend.build is not registered";
+      m = build(cfunc->funcs, cfunc->target, Target(nullptr));
     }
+    shape_func = m.GetFunction(cfunc->func_name);
     shape_func.CallPacked(TVMArgs(values.data(), codes.data(), arity), &rv);
 
     // Get output shapes
@@ -452,11 +397,10 @@ class Interpreter :
     return out_shapes;
   }
 
-  Value InvokePrimitiveOp(const Function& func,
-                          const Array<Value>& args) {
-    auto call_node = func->body.as<CallNode>();
+  ObjectRef InvokePrimitiveOp(const Function& func, const Array<ObjectRef>& args) {
+    const auto* call_node = func->body.as<CallNode>();
 
-    if (call_node && call_node->op == Op::Get("debug")) {
+    if (call_node && call_node->op == debug_op_) {
       auto dattrs = call_node->attrs.as<DebugAttrs>();
       auto interp_state = this->get_state(call_node->args[0]);
 
@@ -470,48 +414,43 @@ class Interpreter :
     }
 
     // Marshal the arguments.
-    // Handle tuple input/output by flattening them.
+    // Handle adt input/output by flattening them.
     size_t arg_len = 0;
     for (size_t i = 0; i < args.size(); ++i) {
-      if (args[i].as<TensorValueNode>()) {
+      if (args[i]->IsInstance<NDArray::ContainerType>()) {
         ++arg_len;
       } else {
-        const auto* tvalue = args[i].as<TupleValueNode>();
-        arg_len += tvalue->fields.size();
+        auto adt = Downcast<ADT>(args[i]);
+        arg_len += adt.size();
       }
     }
     size_t num_inputs = arg_len;
     if (const auto* tuple_type = func->body->checked_type().as<TupleTypeNode>()) {
       arg_len += tuple_type->fields.size();
     } else {
-      CHECK(func->body->checked_type().as<TensorTypeNode>())
-        << func->body->checked_type();
+      CHECK(func->body->checked_type().as<TensorTypeNode>()) << func->body->checked_type();
       arg_len += 1;
     }
     std::vector<TVMValue> values(arg_len);
     std::vector<int> codes(arg_len);
     TVMArgsSetter setter(values.data(), codes.data());
 
-    auto fset_input = [&](size_t i, Value val) {
-      const TensorValueNode* tv = val.as<TensorValueNode>();
-      CHECK(tv != nullptr) << "expect Tensor argument";
-      setter(i, tv->data);
-      DLContext arg_ctx = tv->data->ctx;
-      CHECK(arg_ctx.device_type ==  context_.device_type &&
-            arg_ctx.device_id == context_.device_id)
-        << "Interpreter expect context to be "
-        << context_ << ", but get " << arg_ctx;
+    auto fset_input = [&](size_t i, ObjectRef val) {
+      const auto nd_array = Downcast<NDArray>(val);
+      setter(i, nd_array);
+      DLContext arg_ctx = nd_array->ctx;
+      CHECK(arg_ctx.device_type == context_.device_type && arg_ctx.device_id == context_.device_id)
+          << "Interpreter expect context to be " << context_ << ", but get " << arg_ctx;
     };
 
     int arg_counter = 0;
-    for (Value arg : args) {
-      if (arg.as<TensorValueNode>()) {
-        fset_input(arg_counter++,  arg);
+    for (ObjectRef arg : args) {
+      if (arg->IsInstance<NDArray::ContainerType>()) {
+        fset_input(arg_counter++, arg);
       } else {
-        const TupleValueNode* tuple = arg.as<TupleValueNode>();
-        CHECK(tuple != nullptr);
-        for (size_t i = 0; i < tuple->fields.size(); ++i) {
-          fset_input(arg_counter++, tuple->fields[i]);
+        auto adt = Downcast<ADT>(arg);
+        for (size_t i = 0; i < adt.size(); ++i) {
+          fset_input(arg_counter++, adt[i]);
         }
       }
     }
@@ -526,54 +465,48 @@ class Interpreter :
       // Allocate output tensor.
       std::vector<int64_t> shape;
       for (auto dim : rtype->shape) {
-        const auto* ivalue = as_const_int(dim);
+        const auto* ivalue = tir::as_const_int(dim);
         CHECK(ivalue) << "expected concrete dimensions";
         shape.push_back(ivalue[0]);
       }
-      DLDataType dtype = Type2TVMType(rtype->dtype);
-      auto out_tensor = TensorValueNode::make(
-          NDArray::Empty(shape, dtype, context_));
-      setter(num_inputs + i, out_tensor->data);
-      return out_tensor;
+      DLDataType dtype = rtype->dtype;
+      NDArray nd_array = NDArray::Empty(shape, dtype, context_);
+      setter(num_inputs + i, nd_array);
+      return nd_array;
     };
 
     Array<Shape> out_shapes;
     auto ret_type = func->body->checked_type();
-    bool is_dyn = IsDynamic(func->checked_type());
-    if (call_node->op == Op::Get("shape_of")) {
-      // The output shape of shape_of must be static since Relay doesn't support
-      // dynamic rank tensors.
-      is_dyn = false;
-    }
+    bool is_dyn = IsDynamic(ret_type);
 
     if (is_dyn) {
-      CHECK(func->IsPrimitive());
+      CHECK(func->HasNonzeroAttr(attr::kPrimitive));
       out_shapes = ComputeDynamicShape(func, args);
     }
 
-    PackedFunc packed_func = engine_->JIT(CCacheKeyNode::make(func, target_));
+    PackedFunc packed_func = engine_->JIT(CCacheKey(func, target_));
     TVMRetValue rv;
     if (const TupleTypeNode* rtype = func->body->checked_type().as<TupleTypeNode>()) {
       CHECK(!is_dyn || out_shapes.size() == rtype->fields.size());
-      Array<Value> fields;
+      std::vector<ObjectRef> fields;
       for (size_t i = 0; i < rtype->fields.size(); ++i) {
         if (is_dyn) {
           auto sh = out_shapes[i];
           auto tt = Downcast<TensorType>(rtype->fields[i]);
-          fields.push_back(fset_output(i, TensorTypeNode::make(sh, tt->dtype)));
+          fields.push_back(fset_output(i, TensorType(sh, tt->dtype)));
         } else {
           fields.push_back(fset_output(i, rtype->fields[i]));
         }
       }
       packed_func.CallPacked(TVMArgs(values.data(), codes.data(), arg_len), &rv);
-      return TupleValueNode::make(fields);
+      return ADT::Tuple(fields);
     } else {
-      Value out_tensor;
+      ObjectRef out_tensor;
       if (is_dyn) {
         CHECK_EQ(out_shapes.size(), 1);
         auto sh = out_shapes[0];
         auto tt = Downcast<TensorType>(ret_type);
-        out_tensor = fset_output(0, TensorTypeNode::make(sh, tt->dtype));
+        out_tensor = fset_output(0, TensorType(sh, tt->dtype));
       } else {
         out_tensor = fset_output(0, ret_type);
       }
@@ -583,14 +516,15 @@ class Interpreter :
   }
 
   // Invoke the closure
-  Value Invoke(const Closure& closure, const tvm::Array<Value>& args, const Var& bind = Var()) {
+  ObjectRef Invoke(const InterpreterClosure& closure, const tvm::Array<ObjectRef>& args,
+                   const Var& bind = Var()) {
     // Get a reference to the function inside the closure.
-    if (closure->func->IsPrimitive()) {
+    if (closure->func->HasNonzeroAttr(attr::kPrimitive)) {
       return InvokePrimitiveOp(closure->func, args);
     }
     auto func = closure->func;
     // Allocate a frame with the parameters and free variables.
-    tvm::Map<Var, Value> locals;
+    tvm::Map<Var, ObjectRef> locals;
 
     CHECK_EQ(func->params.size(), args.size());
 
@@ -606,14 +540,14 @@ class Interpreter :
     }
 
     if (bind.defined()) {
-      locals.Set(bind, RecClosureNode::make(closure, bind));
+      locals.Set(bind, RecClosure(closure, bind));
     }
 
-    return WithFrame<Value>(Frame(locals), [&]() { return Eval(func->body); });
+    return WithFrame<ObjectRef>(Frame(locals), [&]() { return Eval(func->body); });
   }
 
-  Value VisitExpr_(const CallNode* call) final {
-    tvm::Array<Value> args;
+  ObjectRef VisitExpr_(const CallNode* call) final {
+    tvm::Array<ObjectRef> args;
     for (auto arg : call->args) {
       args.push_back(Eval(arg));
     }
@@ -628,23 +562,23 @@ class Interpreter :
                     "fusing and lowering";
     }
     if (auto con = call->op.as<ConstructorNode>()) {
-      return ConstructorValueNode::make(con->tag, args, GetRef<Constructor>(con));
+      return ConstructorValue(con->tag, args, GetRef<Constructor>(con));
     }
     // Now we just evaluate and expect to find a closure.
-    Value fn_val = Eval(call->op);
-    if (const ClosureNode* closure_node = fn_val.as<ClosureNode>()) {
-      auto closure = GetRef<Closure>(closure_node);
+    ObjectRef fn_val = Eval(call->op);
+    if (const InterpreterClosureObj* closure_node = fn_val.as<InterpreterClosureObj>()) {
+      auto closure = GetRef<InterpreterClosure>(closure_node);
       return this->Invoke(closure, args);
-    } else if (const RecClosureNode* closure_node = fn_val.as<RecClosureNode>()) {
+    } else if (const RecClosureObj* closure_node = fn_val.as<RecClosureObj>()) {
       return this->Invoke(closure_node->clos, args, closure_node->bind);
     } else {
       LOG(FATAL) << "internal error: type error, expected function value in the call "
                  << "position";
-      return Value();
+      return ObjectRef();
     }
   }
 
-  Value VisitExpr_(const LetNode* let) final {
+  ObjectRef VisitExpr_(const LetNode* let) final {
     if (auto func = let->value.as<FunctionNode>()) {
       auto clo = MakeClosure(GetRef<Function>(func), let->var);
       this->extend(let->var, clo);
@@ -656,24 +590,24 @@ class Interpreter :
     return Eval(let->body);
   }
 
-  Value VisitExpr_(const TupleGetItemNode* op) final {
-    Value val = Eval(op->tuple);
-    auto product_node = val.as<TupleValueNode>();
-    CHECK(product_node)
-      << "interal error: when evaluating TupleGetItem expected a tuple value";
-    CHECK_LT(static_cast<size_t>(op->index), product_node->fields.size())
-        << "internal error: index out of bounds";
-    return product_node->fields[op->index];
+  ObjectRef VisitExpr_(const TupleGetItemNode* op) final {
+    ObjectRef val = Eval(op->tuple);
+    const auto* adt_obj = val.as<ADTObj>();
+    CHECK(adt_obj) << "interal error: when evaluating TupleGetItem expected an ADT value";
+    auto adt = GetRef<ADT>(adt_obj);
+    CHECK_LT(static_cast<size_t>(op->index), adt.size()) << "internal error: index out of bounds";
+    return adt[op->index];
   }
 
-  Value VisitExpr_(const IfNode* op) final {
-    Value v = Eval(op->cond);
-    if (const TensorValueNode* bv = v.as<TensorValueNode>()) {
+  ObjectRef VisitExpr_(const IfNode* op) final {
+    ObjectRef v = Eval(op->cond);
+    if (v->IsInstance<NDArray::ContainerType>()) {
+      auto nd_array = Downcast<NDArray>(v);
       DLContext cpu_ctx;
       cpu_ctx.device_type = kDLCPU;
       cpu_ctx.device_id = 0;
-      NDArray cpu_array = bv->data.CopyTo(cpu_ctx);
-      CHECK_EQ(TVMType2Type(cpu_array->dtype), Bool());
+      NDArray cpu_array = nd_array.CopyTo(cpu_ctx);
+      CHECK_EQ(DataType(cpu_array->dtype), DataType::Bool());
       // TODO(@jroesch, @MK): Refactor code into helper from DCE.
       if (reinterpret_cast<uint8_t*>(cpu_array->data)[0]) {
         return Eval(op->true_branch);
@@ -682,48 +616,46 @@ class Interpreter :
       }
     } else {
       LOG(FATAL) << "type error, type system should have caught this";
-      return Value();
+      return ObjectRef();
     }
   }
 
-  Value VisitExpr_(const RefWriteNode* op) final {
-    Value r = Eval(op->ref);
-    if (const RefValueNode* rv = r.as<RefValueNode>()) {
+  ObjectRef VisitExpr_(const RefWriteNode* op) final {
+    ObjectRef r = Eval(op->ref);
+    if (const RefValueObj* rv = r.as<RefValueObj>()) {
       rv->value = Eval(op->value);
-      return TupleValueNode::make({});
+      return ADT::Tuple(std::vector<ObjectRef>());
     } else {
       LOG(FATAL) << "type error, type system should have caught this";
-      return Value();
+      return ObjectRef();
     }
   }
 
-  Value VisitExpr_(const RefCreateNode* op) final {
-    return RefValueNode::make(Eval(op->value));
-  }
+  ObjectRef VisitExpr_(const RefCreateNode* op) final { return RefValue(Eval(op->value)); }
 
-  Value VisitExpr_(const RefReadNode* op) final {
-    Value r = Eval(op->ref);
-    if (const RefValueNode* rv = r.as<RefValueNode>()) {
+  ObjectRef VisitExpr_(const RefReadNode* op) final {
+    ObjectRef r = Eval(op->ref);
+    if (const RefValueObj* rv = r.as<RefValueObj>()) {
       return rv->value;
     } else {
       LOG(FATAL) << "type error, type system should have caught this";
-      return Value();
+      return ObjectRef();
     }
   }
 
-  Value VisitExpr_(const MatchNode* op) final {
-    Value v = Eval(op->data);
+  ObjectRef VisitExpr_(const MatchNode* op) final {
+    ObjectRef v = Eval(op->data);
     for (const Clause& c : op->clauses) {
       if (VisitPattern(c->lhs, v)) {
         return VisitExpr(c->rhs);
       }
     }
     LOG(FATAL) << "did not find any match";
-    return Value();
+    return ObjectRef();
   }
 
-  bool VisitPattern_(const PatternConstructorNode* op, const Value& v) final {
-    const ConstructorValueNode* cvn = v.as<ConstructorValueNode>();
+  bool VisitPattern_(const PatternConstructorNode* op, const ObjectRef& v) final {
+    const ConstructorValueObj* cvn = v.as<ConstructorValueObj>();
     CHECK(cvn) << "need to be a constructor for match";
     CHECK_NE(op->constructor->tag, -1);
     CHECK_NE(cvn->tag, -1);
@@ -739,62 +671,55 @@ class Interpreter :
     return false;
   }
 
-  bool VisitPattern_(const PatternTupleNode* op, const Value& v) final {
-    const TupleValueNode* tvn = v.as<TupleValueNode>();
-    CHECK(tvn) << "need to be a tuple for match";
-    CHECK_EQ(op->patterns.size(), tvn->fields.size());
+  bool VisitPattern_(const PatternTupleNode* op, const ObjectRef& v) final {
+    auto adt = Downcast<ADT>(v);
+    CHECK_EQ(op->patterns.size(), adt.size());
     for (size_t i = 0; i < op->patterns.size(); ++i) {
-      if (!VisitPattern(op->patterns[i], tvn->fields[i])) {
+      if (!VisitPattern(op->patterns[i], adt[i])) {
         return false;
       }
     }
     return true;
   }
 
-  bool VisitPattern_(const PatternWildcardNode* op, const Value& v) final {
-    return true;
-  }
+  bool VisitPattern_(const PatternWildcardNode* op, const ObjectRef& v) final { return true; }
 
-  bool VisitPattern_(const PatternVarNode* op, const Value& v) final {
+  bool VisitPattern_(const PatternVarNode* op, const ObjectRef& v) final {
     extend(op->var, v);
     return true;
   }
 
   InterpreterState get_state(Expr e = Expr()) const {
-    InterpreterStateNode::Stack stack;
+    InterpreterStateObj::Stack stack;
     for (auto fr : this->stack_.frames) {
-      InterpreterStateNode::Frame frame = fr.locals;
+      InterpreterStateObj::Frame frame = fr.locals;
       stack.push_back(frame);
     }
-    auto state = InterpreterStateNode::make(e, stack);
+    auto state = InterpreterState(e, stack);
     return state;
   }
 
  private:
   // Module
-  Module mod_;
+  IRModule mod_;
   // For simplicity we only run the interpreter on a single context.
   // Context to run the interpreter on.
   DLContext context_;
   // Target parameter being used by the interpreter.
   Target target_;
-  // Value stack.
+  // Object stack.
   Stack stack_;
   // Backend compile engine.
   CompileEngine engine_;
+  // Cache ops that need to be frequently used later to reduce lookup overhead.
+  const Op& debug_op_;
 };
 
-
-TypedPackedFunc<Value(Expr)>
-CreateInterpreter(
-    Module mod,
-    DLContext context,
-    Target target) {
+TypedPackedFunc<ObjectRef(Expr)> CreateInterpreter(IRModule mod, DLContext context, Target target) {
   if (mod.defined()) {
     // eta expand to support constructors in argument position
-    transform::Sequential seq({
-        transform::EtaExpand(
-            /* expand_constructor */ true, /* expand_global_var */ false)});
+    transform::Sequential seq({transform::EtaExpand(
+        /* expand_constructor */ true, /* expand_global_var */ false)});
     transform::PassContext pass_ctx = transform::PassContext::Current();
     tvm::With<transform::PassContext> ctx(pass_ctx);
     mod = seq(mod);
@@ -806,15 +731,10 @@ CreateInterpreter(
     CHECK(f.is_subset_of(FeatureSet::All() - fGraph));
     return intrp->Eval(expr);
   };
-  return TypedPackedFunc<Value(Expr)>(packed);
+  return TypedPackedFunc<ObjectRef(Expr)>(packed);
 }
 
-TVM_REGISTER_API("relay.backend.CreateInterpreter")
-.set_body_typed(CreateInterpreter);
-
-TVM_REGISTER_NODE_TYPE(ClosureNode);
-TVM_REGISTER_NODE_TYPE(TupleValueNode);
-TVM_REGISTER_NODE_TYPE(TensorValueNode);
+TVM_REGISTER_GLOBAL("relay.backend.CreateInterpreter").set_body_typed(CreateInterpreter);
 
 }  // namespace relay
 }  // namespace tvm
