@@ -21,14 +21,17 @@
  * \file src/relay/qnn/op/convolution.cc
  * \brief Property def of qnn convolution operator.
  */
-#include <tvm/data_layout.h>
+#include "../../op/nn/convolution.h"
+
 #include <tvm/relay/analysis.h>
 #include <tvm/relay/base.h>
 #include <tvm/relay/op.h>
 #include <tvm/relay/qnn/attrs.h>
 #include <tvm/relay/transform.h>
-#include "../../op/nn/convolution.h"
-#include "../../pass/pattern_util.h"
+#include <tvm/tir/analysis.h>
+#include <tvm/tir/data_layout.h>
+
+#include "../../transforms/pattern_util.h"
 #include "../util.h"
 
 namespace tvm {
@@ -36,28 +39,67 @@ namespace relay {
 namespace qnn {
 
 // relay.op.qnn.conv2d
-TVM_REGISTER_NODE_TYPE(QnnConv2DAttrs);
 
 bool QnnConv2DRel(const Array<Type>& types, int num_inputs, const Attrs& attrs,
                   const TypeReporter& reporter) {
-  CHECK_EQ(types.size(), 3);
+  CHECK_EQ(types.size(), 7);
   const auto* data = types[0].as<TensorTypeNode>();
   const auto* weight = types[1].as<TensorTypeNode>();
   if (data == nullptr || weight == nullptr) return false;
-  const auto* param = attrs.as<QnnConv2DAttrs>();
-  CHECK(param != nullptr) << "QnnConv2DAttrs cannot be nullptr.";
-  CHECK(data->dtype == Int(8) || data->dtype == UInt(8))
+  const auto* param = attrs.as<Conv2DAttrs>();
+  CHECK(param != nullptr) << "Conv2DAttrs cannot be nullptr.";
+  CHECK(data->dtype == DataType::Int(8) || data->dtype == DataType::UInt(8))
       << "Expected qnn conv2d type(int8, uint8) for input but was " << data->dtype;
-  CHECK(weight->dtype == Int(8) || weight->dtype == UInt(8))
+  CHECK(weight->dtype == DataType::Int(8) || weight->dtype == DataType::UInt(8))
       << "Expected qnn conv2d type(int8, uint8) for weight but was " << weight->dtype;
-  CHECK(param->out_dtype == Int(16) || param->out_dtype == Int(32))
+  CHECK(param->out_dtype == DataType::Int(16) || param->out_dtype == DataType::Int(32))
       << "Expected qnn conv2d type(int32, int16) for output but was " << param->out_dtype;
   CHECK(param->out_dtype.bits() > 0) << "Output dtype bits should be greater than 0.";
-  return Conv2DRel<QnnConv2DAttrs>(types, num_inputs, attrs, reporter);
+
+  // Check the types of scale and zero points.
+  CHECK(IsScalarType(types[2], DataType::Int(32)));    // input_zero_point
+  CHECK(IsScalarType(types[3], DataType::Int(32)));    // kernel_zero_point
+  CHECK(IsScalarType(types[4], DataType::Float(32)));  // input_scale
+  // Kernel scale can be a vector of length output_channels or a scalar.
+  if (param->groups == 1) {
+    size_t axis = param->kernel_layout.find('O');
+    CHECK(axis != std::string::npos) << "Kernel layout attribute is not defined";
+    AssignType(types[5], DataType::Float(32), weight->shape[axis], reporter);  // kernel scale
+  } else {
+    // Here, total number of output channels depend on depth multiplier.
+    size_t o_axis = param->kernel_layout.find('O');
+    size_t i_axis = param->kernel_layout.find('I');
+    CHECK(o_axis != std::string::npos || i_axis != std::string::npos)
+        << "Kernel layout attribute is not defined";
+    AssignType(types[5], DataType::Float(32), weight->shape[i_axis] * weight->shape[o_axis],
+               reporter);  // kernel scale
+  }
+
+  // Collect the input tensor and output tensor devoid of scale and zero points to reuse Relay
+  // Conv2D infer type function.
+  Array<Type> tensor_types = {types[0], types[1], types[6]};
+  return Conv2DRel<Conv2DAttrs>(tensor_types, 3, attrs, reporter);
 }
 
-bool is_depthwise(const QnnConv2DAttrs* param) {
-  return param->channels.defined() && tvm::ir::Equal(param->channels, param->groups) &&
+Array<Array<Layout>> QnnConvInferCorrectLayout(const Attrs& attrs,
+                                               const Array<Layout>& new_in_layouts,
+                                               const Array<Layout>& old_in_layouts,
+                                               const Array<tvm::relay::Type>& old_in_types) {
+  // Use Relay Conv2D Infer correct layout.
+  auto layouts =
+      ConvInferCorrectLayout<Conv2DAttrs>(attrs, new_in_layouts, old_in_layouts, old_in_types);
+
+  // Fill the layouts of remaining input tensors - scales and zero points. The layouts of these
+  // tensors can be treated as channel layout.
+  Layout channel_layout = Layout("C");
+  Array<Layout> input_layouts = {layouts[0][0],  layouts[0][1],  channel_layout,
+                                 channel_layout, channel_layout, channel_layout};
+  Array<Layout> output_layouts = layouts[1];
+  return {input_layouts, output_layouts};
+}
+
+bool is_depthwise(const Conv2DAttrs* param) {
+  return param->channels.defined() && tvm::tir::ExprDeepEqual()(param->channels, param->groups) &&
          param->groups != 1;
 }
 
@@ -70,7 +112,7 @@ using WorkloadType = std::tuple<int, int, int, int, int, int>;
  * \param param The qnn conv2d attributes.
  * \return A tuple of workload.
  */
-WorkloadType GetWorkload(const Array<tvm::relay::Type>& arg_types, const QnnConv2DAttrs* param) {
+WorkloadType GetWorkload(const Array<tvm::relay::Type>& arg_types, const Conv2DAttrs* param) {
   // Get conv parameters.
   const auto in_shape = get_shape(arg_types[0]);
   int batch_size, in_channels;
@@ -118,29 +160,33 @@ WorkloadType GetWorkload(const Array<tvm::relay::Type>& arg_types, const QnnConv
 }
 
 /*
- * \brief Fallback to simpler lowering for dilation or grouped conv.
+ * \brief Fallback to simpler lowering for dilation (when non-zero kernel point) or grouped conv.
  * \param data The input expr.
  * \param weight The weight expr.
+ * \param input_zero_point The input zero point expr.
+ * \param kernel_zero_point The kernel zero point expr.
  * \param param The qnn conv2d attributes.
  * \return The fallback lowered sequence of Relay expr.
- * \note In case of dilation, normal lowering would require a dilated pool.
- *       Since, we don't have dilated pool, we fallback to a simpler sequence of
- *       Relay operations. This will potentially lead to performance degradation
- *       as the convolution is called on int32 tensors instead of int8 tensors.
+ * \note In case of dilation with non-zero kernel zero point, normal lowering would require a
+ * dilated pool. Since, we don't have dilated pool, we fallback to a simpler sequence of Relay
+ * operations. This will potentially lead to performance degradation as the convolution is called on
+ * int32 tensors instead of int8 tensors.
  */
-Expr Conv2DFallBack(const Expr& data, const Expr& weight, const QnnConv2DAttrs* param) {
+Expr Conv2DFallBack(const Expr& data, const Expr& weight, const Expr& input_zero_point,
+                    const Expr& kernel_zero_point, const Conv2DAttrs* param) {
   // Upcast the zero point to Int16.
-  auto zp_data = MakeConstantScalar(Int(16), param->input_zero_point);
-  auto zp_kernel = MakeConstantScalar(Int(16), param->kernel_zero_point);
+  auto zp_data = Cast(input_zero_point, DataType::Int(16));
+  auto zp_kernel = Cast(kernel_zero_point, DataType::Int(16));
 
-  auto shifted_data = Cast(data, Int(16));
-  if (param->input_zero_point != 0) {
-    shifted_data = Subtract(Cast(data, Int(16)), zp_data);
+  auto shifted_data = Cast(data, DataType::Int(16));
+  auto zero_scalar = MakeConstantScalar(DataType::Int(32), 0);
+  if (!IsEqualScalar(input_zero_point, zero_scalar)) {
+    shifted_data = Subtract(Cast(data, DataType::Int(16)), zp_data);
   }
 
-  auto shifted_kernel = Cast(weight, Int(16));
-  if (param->kernel_zero_point != 0) {
-    shifted_kernel = Subtract(Cast(weight, Int(16)), zp_kernel);
+  auto shifted_kernel = Cast(weight, DataType::Int(16));
+  if (!IsEqualScalar(kernel_zero_point, zero_scalar)) {
+    shifted_kernel = Subtract(Cast(weight, DataType::Int(16)), zp_kernel);
   }
 
   return Conv2D(shifted_data, shifted_kernel, param->strides, param->padding, param->dilation,
@@ -151,22 +197,27 @@ Expr Conv2DFallBack(const Expr& data, const Expr& weight, const QnnConv2DAttrs*
 /*
  * \brief Pad the input data.
  * \param data The input expr.
+ * \param input_zero_point The input zero point expr.
  * \return The padded input expr.
  * \note For quantized convolution, the input has to be padded with zero point
  *       instead of zero. This might lead to performance degradation as pad
  *       cannot be fused with conv in Relay. In case we see performance
  *       degradation, we can change the conv2D API to accept a pad_const value.
  */
-Expr Conv2DPadInput(const Expr& data, const QnnConv2DAttrs* param) {
+Expr Conv2DPadInput(const Expr& data, const Expr& input_zero_point, const Conv2DAttrs* param) {
   // 1) Pad the input data
   auto padded_data = data;
-  auto pad_h_value = get_const_int(param->padding[0]);
-  auto pad_w_value = get_const_int(param->padding[1]);
-  if (pad_h_value != 0 || pad_w_value != 0) {
+  auto pad_top_value = get_const_int(param->padding[0]);
+  auto pad_left_value = get_const_int(param->padding[1]);
+  auto pad_bottom_value = get_const_int(param->padding[2]);
+  auto pad_right_value = get_const_int(param->padding[3]);
+  bool do_pad =
+      pad_top_value != 0 || pad_left_value != 0 || pad_bottom_value != 0 || pad_right_value != 0;
+  if (do_pad) {
     Array<IndexExpr> pad_n({0, 0});
     Array<IndexExpr> pad_c({0, 0});
-    Array<IndexExpr> pad_h({param->padding[0], param->padding[0]});
-    Array<IndexExpr> pad_w({param->padding[1], param->padding[1]});
+    Array<IndexExpr> pad_h({param->padding[0], param->padding[2]});
+    Array<IndexExpr> pad_w({param->padding[1], param->padding[3]});
 
     Array<Array<IndexExpr>> pad_width;
     if (param->data_layout == "NCHW") {
@@ -176,7 +227,8 @@ Expr Conv2DPadInput(const Expr& data, const QnnConv2DAttrs* param) {
     } else {
       LOG(FATAL) << "qnn.conv2d does not support " << param->data_layout << " layout";
     }
-    padded_data = Pad(data, pad_width, param->input_zero_point, "constant");
+    auto pad_value = GetScalarFromConstant<int>(input_zero_point);
+    padded_data = Pad(data, pad_width, pad_value, "constant");
   }
   return padded_data;
 }
@@ -184,6 +236,7 @@ Expr Conv2DPadInput(const Expr& data, const QnnConv2DAttrs* param) {
 /*
  * \brief Calculates the second term in the qnn.conv2d depthwise lowering sequence.
  * \param padded_data The padded data expr.
+ * \param kernel_zero_point The kernel zero point expr.
  * \param param The qnn conv2d attributes.
  * \param kernel_h The height of kernel.
  * \param kernel_w The width of kernel.
@@ -197,30 +250,40 @@ Expr Conv2DPadInput(const Expr& data, const QnnConv2DAttrs* param) {
  *       However, deeper analysis shows that we can reduce r,s using avg_pool2d,
  *       followed by repeat on the C axis by cm times.
  */
-Expr DepthwiseConv2DSecondTerm(const Expr& padded_data, const QnnConv2DAttrs* param, int kernel_h,
-                               int kernel_w, int channel_multiplier) {
-  // Constant Expr for the kernel zero point.
-  auto zp_kernel = MakeConstantScalar(Int(32), param->kernel_zero_point);
-
-  auto casted_t2 = Cast(padded_data, Int(32));
+Expr DepthwiseConv2DSecondTerm(const Expr& padded_data, const Expr& kernel_zero_point,
+                               const Conv2DAttrs* param, int kernel_h, int kernel_w,
+                               int channel_multiplier) {
+  auto casted_t2 = Cast(padded_data, DataType::Int(32));
 
   // We can reduce the H and W axis by using avg_pool2d. However, avg_pool2d averages the sum.
   // Since, this is integer division (floor), we can first multiply the data by the pool_size and
   // then perform avg_pool2d. Reversing this causes inaccuracy due to floor division. If the
-  // pool_size is 1x1, we don't need avg_pool2d.
+  // pool_size and strides are 1x1, we don't need avg_pool2d.
   auto reduced_t2 = casted_t2;
   if (kernel_h * kernel_w != 1) {
-    auto scaled_hw_t2 = Multiply(casted_t2, MakeConstantScalar(Int(32), kernel_h * kernel_w));
+    auto scaled_hw_t2 =
+        Multiply(casted_t2, MakeConstantScalar(DataType::Int(32), kernel_h * kernel_w));
     Array<IndexExpr> padding({0, 0});
     reduced_t2 =
         AvgPool2D(scaled_hw_t2, param->kernel_size, param->strides, padding, param->data_layout,
                   false,   // ceil_mode
                   false);  // count_include_pad
+  } else {
+    int stride1 = get_const_int(param->strides[0]);
+    int stride2 = get_const_int(param->strides[1]);
+    if (stride1 * stride2 != 1) {
+      Array<IndexExpr> padding({0, 0});
+      reduced_t2 =
+          AvgPool2D(reduced_t2, param->kernel_size, param->strides, padding, param->data_layout,
+                    false,   // ceil_mode
+                    false);  // count_include_pad
+    }
   }
 
   auto multiplied_t2 = reduced_t2;
-  if (param->kernel_zero_point != 1) {
-    multiplied_t2 = Multiply(zp_kernel, reduced_t2);
+  auto one_scalar = MakeConstantScalar(DataType::Int(32), 1);
+  if (!IsEqualScalar(kernel_zero_point, one_scalar)) {
+    multiplied_t2 = Multiply(kernel_zero_point, reduced_t2);
   }
 
   // Reduce the C dimension. Find the dimension.
@@ -242,6 +305,7 @@ Expr DepthwiseConv2DSecondTerm(const Expr& padded_data, const QnnConv2DAttrs* pa
 /*
  * \brief Calculates the third term in the qnn.conv2d depthwise lowering sequence.
  * \param weight The weight expr.
+ * \param input_zero_point The input zero point expr.
  * \param param The qnn conv2d attributes.
  * \param out_channels The number of output channels.
  * \param channel_multiplier The channel/depth multiplier.
@@ -253,11 +317,8 @@ Expr DepthwiseConv2DSecondTerm(const Expr& padded_data, const QnnConv2DAttrs* pa
  *       This can be achieved by calling reduce on r and s axis. The tensor can be then reshaped to
  *       (1, oc, 1, 1) as (oc/m, oc%m) are just contiguous memory locations.
  */
-Expr DepthwiseConv2DThirdTerm(const Expr& weight, const QnnConv2DAttrs* param, int out_channels,
-                              int channel_multiplier) {
-  // Constant expr for input zero point.
-  auto zp_data = MakeConstantScalar(Int(32), param->input_zero_point);
-
+Expr DepthwiseConv2DThirdTerm(const Expr& weight, const Expr& input_zero_point,
+                              const Conv2DAttrs* param, int out_channels, int channel_multiplier) {
   // Find which dimensions are R, S.
   Array<Integer> axes_t3;
   if (param->kernel_layout == "OIHW") {
@@ -270,7 +331,7 @@ Expr DepthwiseConv2DThirdTerm(const Expr& weight, const QnnConv2DAttrs* param, i
   } else {
     LOG(FATAL) << "qnn.conv2d does not support " << param->kernel_layout << " layout";
   }
-  auto reduced_t3 = Sum(Cast(weight, Int(32)), axes_t3, false, false);
+  auto reduced_t3 = Sum(Cast(weight, DataType::Int(32)), axes_t3, false, false);
 
   // Find the newshape depending on NCHW/NHWC layout.
   Array<Integer> newshape;
@@ -283,15 +344,17 @@ Expr DepthwiseConv2DThirdTerm(const Expr& weight, const QnnConv2DAttrs* param, i
   }
   auto reshaped_t3 = Reshape(reduced_t3, newshape);
 
-  if (param->input_zero_point == 1) {
+  auto one_scalar = MakeConstantScalar(DataType::Int(32), 1);
+  if (IsEqualScalar(input_zero_point, one_scalar)) {
     return reshaped_t3;
   }
-  return Multiply(zp_data, reshaped_t3);
+  return Multiply(input_zero_point, reshaped_t3);
 }
 
 /*
  * \brief Calculates the fourth term in the qnn.conv2d depthwise lowering sequence.
- * \param param The qnn conv2d attributes.
+ * \param input_zero_point_int The int value of input zero point.
+ * \param kernel_zero_point_int The int value of kernel zero point.
  * \param kernel_h The height of kernel.
  * \param kernel_w The width of kernel.
  * \return The sequence of Relay operators for term4.
@@ -299,9 +362,10 @@ Expr DepthwiseConv2DThirdTerm(const Expr& weight, const QnnConv2DAttrs* param, i
  *
  *       Sigma(r, s) zp_a * zp_w
  */
-Expr DepthwiseConv2DFourthTerm(const QnnConv2DAttrs* param, int kernel_h, int kernel_w) {
-  int scalar_term4 = param->input_zero_point * param->kernel_zero_point * kernel_h * kernel_w;
-  return MakeConstantScalar(Int(32), scalar_term4);
+Expr DepthwiseConv2DFourthTerm(int input_zero_point_int, int kernel_zero_point_int, int kernel_h,
+                               int kernel_w) {
+  int scalar_term4 = input_zero_point_int * kernel_zero_point_int * kernel_h * kernel_w;
+  return MakeConstantScalar(DataType::Int(32), scalar_term4);
 }
 
 /*
@@ -314,9 +378,9 @@ Expr DepthwiseConv2DFourthTerm(const QnnConv2DAttrs* param, int kernel_h, int ke
  *       Sigma(c,r,s) QW(k, c, r, s) * QA(n, c, h + r, w + s)
  *       This is just conv2d on int tensors.
  */
-Expr Conv2DFirstTerm(const Expr& padded_data, const Expr& weight, const QnnConv2DAttrs* param) {
+Expr Conv2DFirstTerm(const Expr& padded_data, const Expr& weight, const Conv2DAttrs* param) {
   // Lowering for Term 1
-  Array<IndexExpr> padding({0, 0});
+  Array<IndexExpr> padding({0, 0, 0, 0});
   return Conv2D(padded_data, weight, param->strides, padding, param->dilation, param->groups,
                 param->channels, param->kernel_size, param->data_layout, param->kernel_layout,
                 param->out_layout, param->out_dtype);
@@ -325,6 +389,7 @@ Expr Conv2DFirstTerm(const Expr& padded_data, const Expr& weight, const QnnConv2
 /*
  * \brief Calculates the second term in the qnn.conv2d lowering sequence.
  * \param padded_data The padded data expr.
+ * \param kernel_zero_point The kernel zero point expr.
  * \param param The qnn conv2d attributes.
  * \param kernel_h The height of kernel.
  * \param kernel_w The width of kernel.
@@ -338,12 +403,9 @@ Expr Conv2DFirstTerm(const Expr& padded_data, const Expr& weight, const QnnConv2
  *       followed by a reduce on the C axis. Using avg_pool2d also gives an
  *       opportunity to reuse alter_op_layout infrastructure.
  */
-Expr Conv2DSecondTerm(const Expr& padded_data, const QnnConv2DAttrs* param, int kernel_h,
-                      int kernel_w, int out_channels) {
-  // Constant Expr for the kernel zero point.
-  auto zp_kernel = MakeConstantScalar(Int(32), param->kernel_zero_point);
-
-  auto casted_t2 = Cast(padded_data, Int(32));
+Expr Conv2DSecondTerm(const Expr& padded_data, const Expr& kernel_zero_point,
+                      const Conv2DAttrs* param, int kernel_h, int kernel_w, int out_channels) {
+  auto casted_t2 = Cast(padded_data, DataType::Int(32));
 
   // We can reduce the H and W axis by using avg_pool2d. However, avg_pool2d averages the sum.
   // Since, this is integer division (floor), we can first multiply the data by the pool_size and
@@ -362,19 +424,30 @@ Expr Conv2DSecondTerm(const Expr& padded_data, const QnnConv2DAttrs* param, int
   // Keep dims true to retain 4D tensor
   auto reduced_c_t2 = Sum(casted_t2, axes_t2, true, false);
 
-  // If the pool_size is 1x1, we don't need avg_pool2d.
+  // If the pool_size and strides are 1x1, we don't need avg_pool2d.
   auto reduced_t2 = reduced_c_t2;
   if (kernel_h * kernel_w != 1) {
-    reduced_c_t2 = Multiply(reduced_c_t2, MakeConstantScalar(Int(32), kernel_h * kernel_w));
+    reduced_c_t2 =
+        Multiply(reduced_c_t2, MakeConstantScalar(DataType::Int(32), kernel_h * kernel_w));
     reduced_t2 =
         AvgPool2D(reduced_c_t2, param->kernel_size, param->strides, padding, param->data_layout,
                   false,   // ceil_mode
                   false);  // count_include_pad
+  } else {
+    int stride1 = get_const_int(param->strides[0]);
+    int stride2 = get_const_int(param->strides[1]);
+    if (stride1 * stride2 != 1) {
+      reduced_t2 =
+          AvgPool2D(reduced_c_t2, param->kernel_size, param->strides, padding, param->data_layout,
+                    false,   // ceil_mode
+                    false);  // count_include_pad
+    }
   }
 
   auto multiplied_t2 = reduced_t2;
-  if (param->kernel_zero_point != 1) {
-    multiplied_t2 = Multiply(zp_kernel, reduced_t2);
+  auto one_scalar = MakeConstantScalar(DataType::Int(32), 1);
+  if (!IsEqualScalar(kernel_zero_point, one_scalar)) {
+    multiplied_t2 = Multiply(kernel_zero_point, reduced_t2);
   }
   return multiplied_t2;
 }
@@ -382,6 +455,7 @@ Expr Conv2DSecondTerm(const Expr& padded_data, const QnnConv2DAttrs* param, int
 /*
  * \brief Calculates the third term in the qnn.conv2d lowering sequence.
  * \param weight The weight expr.
+ * \param input_zero_point The input zero point expr.
  * \param param The qnn conv2d attributes.
  * \param out_channels The number of output channels.
  * \return The sequence of Relay operatos for term3.
@@ -393,10 +467,8 @@ Expr Conv2DSecondTerm(const Expr& padded_data, const QnnConv2DAttrs* param, int
  *       a 1D tensor. The tensor is then reshaped to conform to NHWC/NCHW
  *       format.
  */
-Expr Conv2DThirdTerm(const Expr& weight, const QnnConv2DAttrs* param, int out_channels) {
-  // Constant expr for input zero point.
-  auto zp_data = MakeConstantScalar(Int(32), param->input_zero_point);
-
+Expr Conv2DThirdTerm(const Expr& weight, const Expr& input_zero_point, const Conv2DAttrs* param,
+                     int out_channels) {
   // Find which dimensions are C, R, S.
   Array<Integer> axes_t3;
   if (param->kernel_layout == "OIHW") {
@@ -409,7 +481,7 @@ Expr Conv2DThirdTerm(const Expr& weight, const QnnConv2DAttrs* param, int out_ch
   } else {
     LOG(FATAL) << "qnn.conv2d does not support " << param->kernel_layout << " layout";
   }
-  auto reduced_t3 = Sum(Cast(weight, Int(32)), axes_t3, false, false);
+  auto reduced_t3 = Sum(Cast(weight, DataType::Int(32)), axes_t3, false, false);
 
   // Find the newshape depending on NCHW/NHWC layout.
   Array<Integer> newshape;
@@ -422,15 +494,17 @@ Expr Conv2DThirdTerm(const Expr& weight, const QnnConv2DAttrs* param, int out_ch
   }
   auto reshaped_t3 = Reshape(reduced_t3, newshape);
 
-  if (param->input_zero_point == 1) {
+  auto one_scalar = MakeConstantScalar(DataType::Int(32), 1);
+  if (IsEqualScalar(input_zero_point, one_scalar)) {
     return reshaped_t3;
   }
-  return Multiply(zp_data, reshaped_t3);
+  return Multiply(input_zero_point, reshaped_t3);
 }
 
 /*
  * \brief Calculates the fourth term in the qnn.conv2d lowering sequence.
- * \param param The qnn conv2d attributes.
+ * \param input_zero_point_int The int value of input zero point.
+ * \param kernel_zero_point_int The int value of kernel zero point.
  * \param in_channels The number of input channels.
  * \param kernel_h The height of kernel.
  * \param kernel_w The width of kernel.
@@ -440,10 +514,11 @@ Expr Conv2DThirdTerm(const Expr& weight, const QnnConv2DAttrs* param, int out_ch
  *       Sigma(c,r,s) zp_a * zp_w
  *
  */
-Expr Conv2DFourthTerm(const QnnConv2DAttrs* param, int in_channels, int kernel_h, int kernel_w) {
+Expr Conv2DFourthTerm(int input_zero_point_int, int kernel_zero_point_int, int in_channels,
+                      int kernel_h, int kernel_w) {
   int scalar_term4 =
-      param->input_zero_point * param->kernel_zero_point * in_channels * kernel_h * kernel_w;
-  return MakeConstantScalar(Int(32), scalar_term4);
+      input_zero_point_int * kernel_zero_point_int * in_channels * kernel_h * kernel_w;
+  return MakeConstantScalar(DataType::Int(32), scalar_term4);
 }
 
 /*
@@ -452,6 +527,8 @@ Expr Conv2DFourthTerm(const QnnConv2DAttrs* param, int in_channels, int kernel_h
  * \param term2 The term2 of qnn conv2d lowering.
  * \param term3 The term3 of qnn conv2d lowering.
  * \param term4 The term4 of qnn conv2d lowering.
+ * \param input_zero_point_int The int value of input zero point.
+ * \param kernel_zero_point_int The int value of kernel zero point.
  * \param param The qnn conv2d attributes.
  * \return The combined sequence of relay operations.
  * \note The combined operation looks like this
@@ -463,14 +540,14 @@ Expr Conv2DFourthTerm(const QnnConv2DAttrs* param, int in_channels, int kernel_h
  *
  */
 Expr Conv2DCombineTerms(const Expr& term1, const Expr& term2, const Expr& term3, const Expr& term4,
-                        const QnnConv2DAttrs* param) {
-  if (param->input_zero_point == 0 && param->kernel_zero_point == 0) {
+                        int input_zero_point_int, int kernel_zero_point_int) {
+  if (input_zero_point_int == 0 && kernel_zero_point_int == 0) {
     // term 2, 3 and 4 become zero.
     return term1;
-  } else if (param->input_zero_point == 0 && param->kernel_zero_point != 0) {
+  } else if (input_zero_point_int == 0 && kernel_zero_point_int != 0) {
     // term 3 and term 4 become zero.
     return Subtract(term1, term2);
-  } else if (param->input_zero_point != 0 && param->kernel_zero_point == 0) {
+  } else if (input_zero_point_int != 0 && kernel_zero_point_int == 0) {
     // term 2 and term 4 become zero.
     return Subtract(term1, term3);
   } else {
@@ -551,13 +628,14 @@ Expr Conv2DCombineTerms(const Expr& term1, const Expr& term2, const Expr& term3,
  */
 Expr QnnConv2DCanonicalize(const Attrs& attrs, const Array<Expr>& new_args,
                            const Array<tvm::relay::Type>& arg_types) {
-  CHECK_EQ(new_args.size(), 2);
+  CHECK_EQ(new_args.size(), 6);
   Expr data = new_args[0];
   Expr weight = new_args[1];
-  const auto* param = attrs.as<QnnConv2DAttrs>();
+  Expr input_zero_point = new_args[2];
+  Expr kernel_zero_point = new_args[3];
+  const auto* param = attrs.as<Conv2DAttrs>();
   CHECK(param != nullptr);
   // Assertion checks for exisiing support.
-  CHECK_EQ(param->padding.size(), 2) << "qnn.conv2d only supports 2D padding";
   CHECK(param->data_layout == "NCHW" || param->data_layout == "NHWC")
       << "qnn.conv2d supports only NCHW/NHWC input data layout.";
   CHECK(param->kernel_layout == "OIHW" || param->kernel_layout == "HWIO" ||
@@ -568,41 +646,54 @@ Expr QnnConv2DCanonicalize(const Attrs& attrs, const Array<Expr>& new_args,
   std::tie(batch_size, in_channels, out_channels, kernel_h, kernel_w, channel_multiplier) =
       GetWorkload(arg_types, param);
 
-  // Fallback to int32 conv if there is dilation or grouped conv2d
+  // Extract the integer zero points.
+  auto input_zero_point_int = GetScalarFromConstant<int>(input_zero_point);
+  auto kernel_zero_point_int = GetScalarFromConstant<int>(kernel_zero_point);
+
+  // Fallback to int32 conv if there is dilation with non-zero kernel point or grouped conv2d
+  // For dilated conv, if the kernel zero point is non-zero, the pooling operator also has to
+  // traverse the elements in dilated manner. Currently, we do not have strided pool. So, in case of
+  // dilated conv with non-zero kernel point, we fall back to simpler but slow lowering.
 
   CHECK_EQ(param->dilation.size(), 2) << "qnn.conv2d only supports 2D dilation";
   auto dilation_h = get_const_int(param->dilation[0]);
   auto dilation_w = get_const_int(param->dilation[1]);
-  if (dilation_h != 1 || dilation_w != 1 || (param->groups != 1 && !is_depthwise(param))) {
-    return Conv2DFallBack(data, weight, param);
+  if ((kernel_zero_point_int != 0 && (dilation_h != 1 || dilation_w != 1)) ||
+      (param->groups != 1 && !is_depthwise(param))) {
+    return Conv2DFallBack(data, weight, input_zero_point, kernel_zero_point, param);
   } else if (is_depthwise(param)) {
     CHECK_NE(channel_multiplier, -1);
-    auto padded_data = Conv2DPadInput(data, param);
+    auto padded_data = Conv2DPadInput(data, input_zero_point, param);
     auto term1 = Conv2DFirstTerm(padded_data, weight, param);
-    auto term2 =
-        DepthwiseConv2DSecondTerm(padded_data, param, kernel_h, kernel_w, channel_multiplier);
-    auto term3 = DepthwiseConv2DThirdTerm(weight, param, out_channels, channel_multiplier);
-    auto term4 = DepthwiseConv2DFourthTerm(param, kernel_h, kernel_w);
-    return Conv2DCombineTerms(term1, term2, term3, term4, param);
+    auto term2 = DepthwiseConv2DSecondTerm(padded_data, kernel_zero_point, param, kernel_h,
+                                           kernel_w, channel_multiplier);
+    auto term3 =
+        DepthwiseConv2DThirdTerm(weight, input_zero_point, param, out_channels, channel_multiplier);
+    auto term4 =
+        DepthwiseConv2DFourthTerm(input_zero_point_int, kernel_zero_point_int, kernel_h, kernel_w);
+    return Conv2DCombineTerms(term1, term2, term3, term4, input_zero_point_int,
+                              kernel_zero_point_int);
   }
 
-  auto padded_data = Conv2DPadInput(data, param);
+  auto padded_data = Conv2DPadInput(data, input_zero_point, param);
   auto term1 = Conv2DFirstTerm(padded_data, weight, param);
-  auto term2 = Conv2DSecondTerm(padded_data, param, kernel_h, kernel_w, out_channels);
-  auto term3 = Conv2DThirdTerm(weight, param, out_channels);
-  auto term4 = Conv2DFourthTerm(param, in_channels, kernel_h, kernel_w);
-  return Conv2DCombineTerms(term1, term2, term3, term4, param);
+  auto term2 =
+      Conv2DSecondTerm(padded_data, kernel_zero_point, param, kernel_h, kernel_w, out_channels);
+  auto term3 = Conv2DThirdTerm(weight, input_zero_point, param, out_channels);
+  auto term4 = Conv2DFourthTerm(input_zero_point_int, kernel_zero_point_int, in_channels, kernel_h,
+                                kernel_w);
+  return Conv2DCombineTerms(term1, term2, term3, term4, input_zero_point_int,
+                            kernel_zero_point_int);
 }
 
 // Positional relay function to create quantized conv2d operator
 // used by frontend FFI.
-Expr MakeQnnConv2D(Expr data, Expr weight, int32_t input_zero_point, int32_t kernel_zero_point,
-                   double input_scale, double kernel_scale, Array<IndexExpr> strides,
-                   Array<IndexExpr> padding, Array<IndexExpr> dilation,
-                   int groups, IndexExpr channels, Array<IndexExpr> kernel_size,
-                   std::string data_layout, std::string kernel_layout, std::string out_layout,
-                   DataType out_dtype) {
-  auto attrs = make_node<QnnConv2DAttrs>();
+Expr MakeQnnConv2D(Expr data, Expr weight, Expr input_zero_point, Expr kernel_zero_point,
+                   Expr input_scale, Expr kernel_scale, Array<IndexExpr> strides,
+                   Array<IndexExpr> padding, Array<IndexExpr> dilation, int groups,
+                   IndexExpr channels, Array<IndexExpr> kernel_size, String data_layout,
+                   String kernel_layout, String out_layout, DataType out_dtype) {
+  auto attrs = make_object<Conv2DAttrs>();
   attrs->strides = std::move(strides);
   attrs->padding = std::move(padding);
   attrs->dilation = std::move(dilation);
@@ -613,16 +704,13 @@ Expr MakeQnnConv2D(Expr data, Expr weight, int32_t input_zero_point, int32_t ker
   attrs->kernel_layout = std::move(kernel_layout);
   attrs->out_layout = std::move(out_layout);
   attrs->out_dtype = std::move(out_dtype);
-  attrs->input_zero_point = std::move(input_zero_point);
-  attrs->kernel_zero_point = std::move(kernel_zero_point);
-  attrs->input_scale = std::move(input_scale);
-  attrs->kernel_scale = std::move(kernel_scale);
   static const Op& op = Op::Get("qnn.conv2d");
-  return CallNode::make(op, {data, weight}, Attrs(attrs), {});
+  return Call(op, {data, weight, input_zero_point, kernel_zero_point, input_scale, kernel_scale},
+              Attrs(attrs), {});
 }
 
 RELAY_REGISTER_OP("qnn.conv2d")
-.describe(R"code(2D quantized convolution layer.
+    .describe(R"code(2D quantized convolution layer.
 This operator convolves quantized weight with quantized data. The scale of the
 output quantized tensor is the product of the weight_scale and input_scale of
 the input quantized tensors. The zero point of the output quantized tensor is
@@ -634,15 +722,21 @@ operator to understand how to scale back the int32 output to (u)int8.
 - **out**:  This depends on the `layout` parameter. Output is 4D array of shape
             (batch_size, channels, out_height, out_width) if `layout` is `NCHW`.
 )code" TVM_ADD_FILELINE)
-.set_attrs_type<QnnConv2DAttrs>()
-.set_num_inputs(2)
-.add_argument("data", "Tensor", "The quantized input data tensor.")
-.add_argument("weight", "Tensor", "The quantized weight tensor.")
-.set_support_level(11)
-.add_type_rel("QnnConv2D", QnnConv2DRel)
-.set_attr<FTVMLegalize>("FTVMQnnCanonicalize", QnnConv2DCanonicalize);
+    .set_attrs_type<Conv2DAttrs>()
+    .set_num_inputs(6)
+    .add_argument("data", "Tensor", "The quantized input data tensor.")
+    .add_argument("weight", "Tensor", "The quantized weight tensor.")
+    .add_argument("input_scale", "Tensor", "The quantization scale of the input tensor.")
+    .add_argument("input_zero_point", "Tensor", "The quantization zero_point of the input tensor.")
+    .add_argument("weight_scale", "Tensor", "The quantization scale of the weight tensor.")
+    .add_argument("weight_zero_point", "Tensor",
+                  "The quantization zero_point of the weight tensor.")
+    .set_support_level(11)
+    .add_type_rel("QnnConv2D", QnnConv2DRel)
+    .set_attr<FTVMLegalize>("FTVMQnnCanonicalize", QnnConv2DCanonicalize)
+    .set_attr<FInferCorrectLayout>("FInferCorrectLayout", QnnConvInferCorrectLayout);
 
-TVM_REGISTER_API("relay.qnn.op._make.conv2d").set_body_typed(MakeQnnConv2D);
+TVM_REGISTER_GLOBAL("relay.qnn.op._make.conv2d").set_body_typed(MakeQnnConv2D);
 
 }  // namespace qnn
 }  // namespace relay
