@@ -22,11 +22,11 @@
  */
 #ifndef TVM_RUNTIME_CONTRIB_NNPACK_NNPACK_UTILS_H_
 #define TVM_RUNTIME_CONTRIB_NNPACK_NNPACK_UTILS_H_
-#include <tvm/runtime/registry.h>
-#include <tvm/runtime/util.h>
-#include <dmlc/thread_local.h>
 #include <dmlc/logging.h>
+#include <dmlc/thread_local.h>
 #include <nnpack.h>
+#include <tvm/runtime/data_type.h>
+#include <tvm/runtime/registry.h>
 
 namespace tvm {
 namespace contrib {
