@@ -17,20 +17,22 @@
  * under the License.
  */
 
-#include <tvm/operation.h>
-#include <topi/elemwise.h>
 #include <gtest/gtest.h>
+#include <tvm/te/operation.h>
+#include <tvm/topi/elemwise.h>
 
+namespace tvm {
 namespace topi {
 TEST(Tensor, Basic) {
   using namespace tvm;
   Var m("m"), n("n"), l("l");
-  Tensor A = placeholder({m, l}, Float(32), "A");
+  Tensor A = placeholder({m, l}, DataType::Float(32), "A");
   auto C = topi::exp(A);
 }
-}
+}  // namespace topi
+}  // namespace tvm
 
-int main(int argc, char ** argv) {
+int main(int argc, char** argv) {
   testing::InitGoogleTest(&argc, argv);
   testing::FLAGS_gtest_death_test_style = "threadsafe";
   return RUN_ALL_TESTS();
