@@ -16,6 +16,7 @@
 # under the License.
 """ test bind function."""
 import tvm
+from tvm import te
 from tvm import relay
 
 
@@ -24,15 +25,13 @@ def test_bind_params():
     y = relay.var("y")
     z = relay.add(x, y)
     f = relay.Function([x, y], z)
-    fbinded = relay.bind(f, {x : relay.const(1, "float32")})
-    fexpected =relay.Function(
-        [y],
-        relay.add(relay.const(1, "float32"),  y))
-    assert relay.analysis.alpha_equal(fbinded, fexpected)
+    fbinded = relay.bind(f, {x: relay.const(1, "float32")})
+    fexpected = relay.Function([y], relay.add(relay.const(1, "float32"), y))
+    assert tvm.ir.structural_equal(fbinded, fexpected)
 
     zbinded = relay.bind(z, {y: x})
     zexpected = relay.add(x, x)
-    assert relay.analysis.alpha_equal(zbinded, zexpected)
+    assert tvm.ir.structural_equal(zbinded, zexpected)
 
 
 if __name__ == "__main__":
