@@ -15,16 +15,16 @@
 # specific language governing permissions and limitations
 # under the License.
 import tvm
+from tvm import te
 from tvm import relay
-from tvm.relay.analysis import graph_equal, assert_graph_equal
-from tvm.relay.analysis import alpha_equal, assert_alpha_equal
+import tvm.relay.testing
 import pytest
 from numpy import isclose
 from typing import Union
 from functools import wraps
-raises_parse_error = pytest.mark.xfail(raises=tvm._ffi.base.TVMError)
 
-SEMVER = "v0.0.4"
+
+SEMVER = '#[version = "0.0.5"]\n'
 
 BINARY_OPS = {
     "*": relay.multiply,
@@ -44,18 +44,14 @@ TYPES = {
     "int16",
     "int32",
     "int64",
-
     "uint8",
     "uint16",
     "uint32",
     "uint64",
-
     "float16",
     "float32",
     "float64",
-
     "bool",
-
     "int8x4",
     "uint1x4",
     "float16x4",
@@ -68,14 +64,30 @@ type List[A] {
 }
 """
 
+
+def assert_graph_equal(lhs, rhs):
+    tvm.ir.assert_structural_equal(lhs, rhs, map_free_vars=True)
+
+
+def graph_equal(lhs, rhs):
+    return tvm.ir.structural_equal(lhs, rhs, map_free_vars=True)
+
+
+def roundtrip_expr(expr):
+    text = tvm.relay.Expr.astext(expr, show_meta_data=False)
+    x = tvm.parser.parse_expr(text)
+    assert_graph_equal(x, expr)
+
+
+# Testing Utilities for expressions.
 def roundtrip(expr):
-    x = relay.fromtext(str(expr))
+    x = tvm.parser.fromtext(expr.astext())
     assert_graph_equal(x, expr)
 
 
 def parse_text(code):
-    expr = relay.fromtext(SEMVER + "\n" + code)
-    roundtrip(expr)
+    expr = tvm.parser.parse_expr(code)
+    roundtrip_expr(expr)
     return expr
 
 
@@ -85,10 +97,29 @@ def parses_as(code, expr):
     result = graph_equal(parsed, expr)
     return result
 
+
+# Testing Utilities for full modules.
+def parse_module(code):
+    mod = tvm.parser.parse(SEMVER + code)
+    roundtrip(mod)
+    return mod
+
+
+def assert_parses_as(code, expr):
+    parsed = parse_text(code)
+    assert_graph_equal(parsed, expr)
+
+
+def assert_parse_module_as(code, mod):
+    parsed = parse_module(code)
+    assert_graph_equal(parsed, mod)
+
+
 def get_scalar(x):
     # type: (relay.Constant) -> (Union[float, int, bool])
     return x.data.asnumpy().item()
 
+
 int32 = relay.scalar_type("int32")
 
 _ = relay.Var("_")
@@ -101,38 +132,38 @@ UNIT = relay.Tuple([])
 
 
 def test_comments():
-    assert parses_as(
+    assert_parses_as(
         """
         // This is a line comment!
         ()
         """,
-        UNIT
+        UNIT,
     )
 
-    assert parses_as(
+    assert_parses_as(
         """
         /* This is a block comment!
             This is still a block comment!
         */
         ()
         """,
-        UNIT
+        UNIT,
     )
 
-    assert parses_as(
+    assert_parses_as(
         """
         /* This is a block comment!
            /*Block comment is recursive!*/
         */
         ()
         """,
-        UNIT
+        UNIT,
     )
 
 
 def test_int_literal():
     assert isinstance(parse_text("1"), relay.Constant)
-    assert isinstance(parse_text("1").data, tvm.ndarray.NDArray)
+    assert isinstance(parse_text("1").data, tvm.nd.NDArray)
 
     assert get_scalar(parse_text("1")) == 1
     assert get_scalar(parse_text("10")) == 10
@@ -149,13 +180,13 @@ def test_float_literal():
 
     # scientific notation
     assert isclose(get_scalar(parse_text("1e-1f")), 1e-1)
-    assert get_scalar(parse_text("1e+1f")) == 1e+1
-    assert isclose(get_scalar(parse_text("1E-1f")), 1E-1)
-    assert get_scalar(parse_text("1E+1f")) == 1E+1
+    assert get_scalar(parse_text("1e+1f")) == 1e1
+    assert isclose(get_scalar(parse_text("1E-1f")), 1e-1)
+    assert get_scalar(parse_text("1E+1f")) == 1e1
     assert isclose(get_scalar(parse_text("1.0e-1f")), 1.0e-1)
-    assert get_scalar(parse_text("1.0e+1f")) == 1.0e+1
-    assert isclose(get_scalar(parse_text("1.0E-1f")), 1.0E-1)
-    assert get_scalar(parse_text("1.0E+1f")) == 1.0E+1
+    assert get_scalar(parse_text("1.0e+1f")) == 1.0e1
+    assert isclose(get_scalar(parse_text("1.0E-1f")), 1.0e-1)
+    assert get_scalar(parse_text("1.0E+1f")) == 1.0e1
 
 
 def test_bool_literal():
@@ -164,16 +195,16 @@ def test_bool_literal():
 
 
 def test_negative():
-    assert isinstance(parse_text("let %x = 1; -%x").body, relay.Call)
+    # need to handle parsing non-literal operations
+    # assert isinstance(parse_text("let %x = 1; -%x").body, relay.Call)
     assert get_scalar(parse_text("--10")) == 10
     assert get_scalar(parse_text("---10")) == -10
 
 
 def test_bin_op():
     for bin_op in BINARY_OPS.keys():
-        assert parses_as(
-            "1 {} 1".format(bin_op),
-            BINARY_OPS.get(bin_op)(relay.const(1), relay.const(1))
+        assert_parses_as(
+            "1 {} 1".format(bin_op), BINARY_OPS.get(bin_op)(relay.const(1), relay.const(1))
         )
 
 
@@ -187,14 +218,7 @@ def test_op_assoc():
     assert graph_equal(parse_text("1 == 1 < 1 + 1 * 1"), parse_text("1 == (1 < (1 + (1 * 1)))"))
 
 
-@pytest.mark.skip
 def test_vars():
-    # temp vars won't work b/c they start with a digit
-    # # temp var
-    # temp_var = parse_text("%1")
-    # assert isinstance(temp_var, relay.Var)
-    # assert temp_var.name == "1"
-
     # var
     var = parse_text("let %foo = (); %foo")
     assert isinstance(var.body, relay.Var)
@@ -206,170 +230,118 @@ def test_vars():
     assert global_var.name_hint == "foo"
 
     # operator id
-    op = parse_text("foo")
-    assert isinstance(op, relay.Op)
-    assert op.name == "foo"
+    op = parse_text("add")
+    assert isinstance(op, tvm.ir.Op)
+    assert op.name == "add"
+
+    # operator id with prefix
+    op = parse_text("nn.global_avg_pool2d")
+    assert isinstance(op, tvm.ir.Op)
+    assert op.name == "nn.global_avg_pool2d"
+
+
+def test_meta_ref():
+    with pytest.raises(tvm.error.DiagnosticError):
+        meta_op = parse_text("meta[type_key][1337]")
+        assert meta_op.attrs.node_type_key == "type_key"
+        assert meta_op.attrs.node_index == 1337
 
 
 def test_let():
-    assert parses_as(
-        "let %x = 1; ()",
-        relay.Let(
-            X,
-            relay.const(1),
-            UNIT
-        )
-    )
+    assert_parses_as("let %x = 1; ()", relay.Let(X, relay.const(1), UNIT))
 
-    assert parses_as(
+    assert_parses_as(
         """
         let %x = 1;
         let %y = 2;
         ()
         """,
-        relay.Let(
-            X,
-            relay.const(1),
-            relay.Let(
-                Y,
-                relay.const(2),
-                UNIT
-            )
-        )
+        relay.Let(X, relay.const(1), relay.Let(Y, relay.const(2), UNIT)),
     )
 
 
 def test_seq():
-    assert parses_as(
-        "();; ()",
-        relay.Let(
-            _,
-            UNIT,
-            UNIT)
-    )
+    assert_parses_as("(); ()", relay.Let(_, UNIT, UNIT))
 
-    assert parses_as(
-        "let %_ = 1; ()",
-        relay.Let(
-            X,
-            relay.const(1),
-            UNIT
-        )
-    )
+    assert_parses_as("let %_ = 1; ()", relay.Let(X, relay.const(1), UNIT))
 
 
 def test_graph():
     code = "%0 = (); %1 = 1; (%0, %0, %1)"
-    assert parses_as(
-        code,
-        relay.Tuple([UNIT, UNIT, relay.const(1)])
-    )
-    assert not parses_as(
-        code,
-        relay.Tuple([relay.Tuple([]), relay.Tuple([]), relay.const(1)])
-    )
+    assert_parses_as(code, relay.Tuple([UNIT, UNIT, relay.const(1)]))
 
 
-@raises_parse_error
-def test_graph_wrong_order():
-    parse_text("%1 = (); %1")
+def test_graph_single():
+    assert_parses_as("%1 = (); %1", relay.Tuple([]))
 
 
-@raises_parse_error
 def test_let_global_var():
-    parse_text("let @x = 1; ()")
+    with pytest.raises(tvm.error.DiagnosticError):
+        parse_text("let @x = 1; ()")
 
 
-@raises_parse_error
 def test_let_op():
-    parse_text("let x = 1; ()")
+    with pytest.raises(tvm.error.DiagnosticError):
+        parse_text("let x = 1; ()")
 
 
 def test_tuple():
-    assert parses_as("()", relay.Tuple([]))
+    assert_parses_as("()", relay.Tuple([]))
 
-    assert parses_as("(0,)", relay.Tuple([relay.const(0)]))
+    assert_parses_as("(0,)", relay.Tuple([relay.const(0)]))
 
-    assert parses_as("(0, 1)", relay.Tuple([relay.const(0), relay.const(1)]))
+    assert_parses_as("(0, 1)", relay.Tuple([relay.const(0), relay.const(1)]))
 
-    assert parses_as("(0, 1, 2)", relay.Tuple([relay.const(0), relay.const(1), relay.const(2)]))
+    assert_parses_as("(0, 1, 2)", relay.Tuple([relay.const(0), relay.const(1), relay.const(2)]))
 
 
 def test_func():
     # 0 args
-    assert parses_as(
-        "fn () { 0 }",
-        relay.Function(
-            [],
-            relay.const(0),
-            None,
-            []
-        )
-    )
+    assert_parses_as("fn () { 0 }", relay.Function([], relay.const(0), None, []))
 
     # 1 arg
-    assert parses_as(
-        "fn (%x) { %x }",
-        relay.Function(
-            [X],
-            X,
-            None,
-            []
-        )
-    )
+    assert_parses_as("fn (%x) { %x }", relay.Function([X], X, None, []))
 
     # 2 args
-    assert parses_as(
-        "fn (%x, %y) { %x + %y }",
-        relay.Function(
-            [X, Y],
-            relay.add(X, Y),
-            None,
-            []
-        )
-    )
+    assert_parses_as("fn (%x, %y) { %x + %y }", relay.Function([X, Y], relay.add(X, Y), None, []))
 
     # annotations
-    assert parses_as(
-        "fn (%x: int32) -> int32 { %x }",
-        relay.Function(
-            [X_ANNO],
-            X_ANNO,
-            int32,
-            []
-        )
-    )
+    assert_parses_as("fn (%x: int32) -> int32 { %x }", relay.Function([X_ANNO], X_ANNO, int32, []))
 
-    # attributes
-    assert parses_as(
-        "fn (n=5) { () }",
-        relay.Function([], UNIT, None, None, tvm.make.node("DictAttrs", n=relay.const(5)))
-    )
+    # Refactor the attribute syntax and printing.
+    #
+    # # attributes
+    # assert_parses_as(
+    #     "fn (n=5) { () }",
+    #     relay.Function([], UNIT, None, None, tvm.ir.make_node("DictAttrs", n=relay.const(5)))
+    # )
 
 
 # TODO(@jmp): Crashes if %x isn't annnotated.
 def test_defn():
-    id_defn = parse_text(
+    id_defn = parse_module(
         """
         def @id(%x: int32) -> int32 {
             %x
         }
-        """)
-    assert isinstance(id_defn, relay.Module)
+        """
+    )
+    assert isinstance(id_defn, tvm.IRModule)
 
 
 def test_recursive_call():
-    id_defn = parse_text(
+    id_defn = parse_module(
         """
         def @id(%x: int32) -> int32 {
             @id(%x)
         }
-        """)
-    assert isinstance(id_defn, relay.Module)
+        """
+    )
+    assert isinstance(id_defn, tvm.IRModule)
 
 
 def test_ifelse():
-    assert parses_as(
+    assert_parses_as(
         """
         if (True) {
             0
@@ -377,32 +349,28 @@ def test_ifelse():
             1
         }
         """,
-        relay.If(
-            relay.const(True),
-            relay.const(0),
-            relay.const(1)
-        )
+        relay.If(relay.const(True), relay.const(0), relay.const(1)),
     )
 
 
-@raises_parse_error
 def test_ifelse_scope():
-    parse_text(
-        """
-        if (True) {
-            let %x = ();
-            ()
-        } else {
-            %x
-        }
-        """
-    )
+    with pytest.raises(tvm.error.DiagnosticError):
+        parse_text(
+            """
+            if (True) {
+                let %x = ();
+                ()
+            } else {
+                %x
+            }
+            """
+        )
 
 
 def test_call():
     # select right function to call: simple ident case
     id_func = relay.Var("id")
-    assert parses_as(
+    assert_parses_as(
         """
         let %id = fn (%x) { %x };
         10 * %id(10)
@@ -410,13 +378,13 @@ def test_call():
         relay.Let(
             id_func,
             relay.Function([X], X, None, []),
-            relay.multiply(relay.const(10), relay.Call(id_func, [relay.const(10)]))
-        )
+            relay.multiply(relay.const(10), relay.Call(id_func, [relay.const(10)])),
+        ),
     )
 
     # 0 args
     constant = relay.Var("constant")
-    assert parses_as(
+    assert_parses_as(
         """
         let %constant = fn () { 0 };
         %constant()
@@ -424,13 +392,13 @@ def test_call():
         relay.Let(
             constant,
             relay.Function([], relay.const(0), None, []),
-            relay.Call(constant, [], None, None)
-        )
+            relay.Call(constant, [], None, None),
+        ),
     )
 
     # 1 arg
     id_var = relay.Var("id")
-    assert parses_as(
+    assert_parses_as(
         """
         let %id = fn (%x) { %x };
         %id(1)
@@ -438,100 +406,70 @@ def test_call():
         relay.Let(
             id_var,
             relay.Function([X], X, None, []),
-            relay.Call(id_var, [relay.const(1)], None, None)
-        )
+            relay.Call(id_var, [relay.const(1)], None, None),
+        ),
     )
 
     # 2 args
     multiply = relay.Var("multiply")
-    assert parses_as(
+    assert_parses_as(
         """
         let %multiply = fn (%x, %y) { %x * %y };
         %multiply(0, 0)
         """,
         relay.Let(
             multiply,
-            relay.Function(
-                [X, Y],
-                relay.multiply(X, Y),
-                None,
-                []
-            ),
-            relay.Call(multiply, [relay.const(0), relay.const(0)], None, None)
-        )
+            relay.Function([X, Y], relay.multiply(X, Y), None, []),
+            relay.Call(multiply, [relay.const(0), relay.const(0)], None, None),
+        ),
     )
 
     # anonymous function
-    assert parses_as(
+    assert_parses_as(
         """
         (fn (%x) { %x })(0)
         """,
-        relay.Call(
-            relay.Function(
-                [X],
-                X,
-                None,
-                []
-            ),
-            [relay.const(0)],
-            None,
-            None
-        )
+        relay.Call(relay.Function([X], X, None, []), [relay.const(0)], None, None),
     )
 
-    # TODO(@jmp): re-enable after sequence parsing improvements
     # curried function
-    # curried_mult = relay.Var("curried_mult")
-    # assert parses_as(
-    #     """
-    #     let %curried_mult =
-    #         fn (%x) {
-    #         fn (%y) {
-    #             %x * %y
-    #         }
-    #         };
-    #     %curried_mult(0);
-    #     %curried_mult(0)(0)
-    #     """,
-    #     relay.Let(
-    #         curried_mult,
-    #         relay.Function(
-    #             [X],
-    #             relay.Function(
-    #                 [Y],
-    #                 relay.multiply(X, Y),
-    #                 None,
-    #                 []
-    #             ),
-    #             None,
-    #             []
-    #         ),
-    #         relay.Let(
-    #             _,
-    #             relay.Call(curried_mult, [relay.const(0)], None, None),
-    #             relay.Call(relay.Call(curried_mult, [relay.const(0)], None, None), [relay.const(0)], None, None)
-    #         )
-    #     )
-    # )
+    curried_mult = relay.Var("curried_mult")
+    assert_parses_as(
+        """
+        let %curried_mult =
+            fn (%x) {
+            fn (%y) {
+                %x * %y
+            }
+            };
+            %curried_mult(0);
+            %curried_mult(0)(0)
+        """,
+        relay.Let(
+            curried_mult,
+            relay.Function([X], relay.Function([Y], relay.multiply(X, Y), None, []), None, []),
+            relay.Let(
+                _,
+                relay.Call(curried_mult, [relay.const(0)], None, None),
+                relay.Call(
+                    relay.Call(curried_mult, [relay.const(0)], None, None),
+                    [relay.const(0)],
+                    None,
+                    None,
+                ),
+            ),
+        ),
+    )
 
     # op
-    assert parses_as(
-        "abs(1)",
-        relay.Call(relay.op.get("abs"), [relay.const(1)], None, None)
-    )
+    assert_parses_as("abs(1)", relay.Call(relay.op.get("abs"), [relay.const(1)], None, None))
+
 
 # Types
 
 
 def test_incomplete_type():
-    assert parses_as(
-        "let %_ : _ = (); ()",
-        relay.Let(
-            _,
-            UNIT,
-            UNIT
-        )
-    )
+    assert_parses_as("let %_ : _ = (); ()", relay.Let(_, UNIT, UNIT))
 
 
 def test_builtin_types():
@@ -540,149 +478,153 @@ def test_builtin_types():
 
 
 def test_tensor_type():
-    assert parses_as(
+    assert_parses_as(
         "let %_ : Tensor[(), float32] = (); ()",
-        relay.Let(
-            relay.Var("_", relay.TensorType((), "float32")),
-            UNIT,
-            UNIT
-        )
+        relay.Let(relay.Var("_", relay.TensorType((), "float32")), UNIT, UNIT),
     )
 
-    assert parses_as(
+    assert_parses_as(
         "let %_ : Tensor[(1), float32] = (); ()",
-        relay.Let(
-            relay.Var("_", relay.TensorType((1,), "float32")),
-            UNIT,
-            UNIT
-        )
+        relay.Let(relay.Var("_", relay.TensorType((1,), "float32")), UNIT, UNIT),
     )
 
-    assert parses_as(
+    assert_parses_as(
         "let %_ : Tensor[(1, 1), float32] = (); ()",
-        relay.Let(
-            relay.Var("_", relay.TensorType((1, 1), "float32")),
-            UNIT,
-            UNIT
-        )
+        relay.Let(relay.Var("_", relay.TensorType((1, 1), "float32")), UNIT, UNIT),
+    )
+
+    assert_parses_as(
+        "let %_ : Tensor[(?, 1), float32] = (); ()",
+        relay.Let(relay.Var("_", relay.TensorType((tvm.tir.Any(), 1), "float32")), UNIT, UNIT),
     )
 
 
 def test_function_type():
-    assert parses_as(
+    assert_parses_as(
         """
         let %_: fn () -> int32 = fn () -> int32 { 0 }; ()
         """,
         relay.Let(
             relay.Var("_", relay.FuncType([], int32, [], [])),
             relay.Function([], relay.const(0), int32, []),
-            UNIT
-        )
+            UNIT,
+        ),
     )
 
-    assert parses_as(
+    assert_parses_as(
         """
         let %_: fn (int32) -> int32 = fn (%x: int32) -> int32 { 0 }; ()
         """,
         relay.Let(
             relay.Var("_", relay.FuncType([int32], int32, [], [])),
             relay.Function([relay.Var("x", int32)], relay.const(0), int32, []),
-            UNIT
-        )
+            UNIT,
+        ),
     )
 
-    assert parses_as(
+    assert_parses_as(
         """
         let %_: fn (int32, int32) -> int32 = fn (%x: int32, %y: int32) -> int32 { 0 }; ()
         """,
         relay.Let(
             relay.Var("_", relay.FuncType([int32, int32], int32, [], [])),
-            relay.Function([relay.Var("x", int32), relay.Var("y", int32)], relay.const(0), int32, []),
-            UNIT
-        )
+            relay.Function(
+                [relay.Var("x", int32), relay.Var("y", int32)], relay.const(0), int32, []
+            ),
+            UNIT,
+        ),
     )
 
 
 def test_tuple_type():
-    assert parses_as(
+    assert_parses_as(
         """
         let %_: () = (); ()
         """,
-        relay.Let(
-            relay.Var("_", relay.TupleType([])),
-            UNIT,
-            UNIT
-        )
+        relay.Let(relay.Var("_", relay.TupleType([])), UNIT, UNIT),
     )
 
-    assert parses_as(
+    assert_parses_as(
         """
         let %_: (int32,) = (0,); ()
         """,
-        relay.Let(
-            relay.Var("_", relay.TupleType([int32])),
-            relay.Tuple([relay.const(0)]),
-            UNIT
-        )
+        relay.Let(relay.Var("_", relay.TupleType([int32])), relay.Tuple([relay.const(0)]), UNIT),
     )
 
-    assert parses_as(
+    assert_parses_as(
         """
         let %_: (int32, int32) = (0, 1); ()
         """,
         relay.Let(
             relay.Var("_", relay.TupleType([int32, int32])),
             relay.Tuple([relay.const(0), relay.const(1)]),
-            UNIT
-        )
+            UNIT,
+        ),
     )
 
 
 def test_adt_defn():
-    mod = relay.Module()
+    mod = tvm.IRModule()
 
     glob_typ_var = relay.GlobalTypeVar("Ayy")
-    prog = relay.TypeData(
-            glob_typ_var,
-            [],
-            [relay.Constructor("Nil", [], glob_typ_var)])
+    prog = relay.TypeData(glob_typ_var, [], [relay.Constructor("Nil", [], glob_typ_var)])
     mod[glob_typ_var] = prog
-    assert parses_as(
+    assert_parse_module_as(
         """
         type Ayy { Nil }
         """,
-        mod
+        mod,
     )
 
 
+def test_adt_any():
+    code = """
+    type my_dtype {
+        my_cons(Tensor[(?, 1), uint16]),
+    }
+    """
+    mod = parse_module(code)
+    items = mod.type_definitions.items()
+    global_type_var, type_data = items[0]
+    assert global_type_var.name_hint == "my_dtype"
+    ctors = type_data.constructors
+    assert len(ctors) == 1
+    my_cons = ctors[0]
+    assert my_cons.name_hint == "my_cons"
+    ty_shape = my_cons.inputs[0].shape
+    assert isinstance(ty_shape[0], tvm.tir.Any)
+    assert ty_shape[1] == 1
+
+
 def test_empty_adt_defn():
-    mod = relay.Module()
+    mod = tvm.IRModule()
 
     glob_typ_var = relay.GlobalTypeVar("Ayy")
     prog = relay.TypeData(glob_typ_var, [], [])
     mod[glob_typ_var] = prog
-    assert parses_as(
+    assert_parse_module_as(
         """
         type Ayy { }
         """,
-        mod
+        mod,
     )
 
 
 def test_multiple_cons_defn():
-    mod = relay.Module()
+    mod = tvm.IRModule()
 
     list_var = relay.GlobalTypeVar("List")
     typ_var = relay.TypeVar("A")
     prog = relay.TypeData(
-            list_var,
-            [typ_var],
-            [
-                relay.Constructor("Cons", [typ_var, list_var(typ_var)], list_var),
-                relay.Constructor("Nil", [], list_var),
-            ])
+        list_var,
+        [typ_var],
+        [
+            relay.Constructor("Cons", [typ_var, list_var(typ_var)], list_var),
+            relay.Constructor("Nil", [], list_var),
+        ],
+    )
     mod[list_var] = prog
-    assert parses_as(LIST_DEFN, mod)
+    assert_parse_module_as(LIST_DEFN, mod)
 
 
 def test_multiple_type_param_defn():
@@ -690,22 +632,23 @@ def test_multiple_type_param_defn():
     typ_var_a = relay.TypeVar("A")
     typ_var_b = relay.TypeVar("B")
     prog = relay.TypeData(
-            glob_typ_var,
-            [typ_var_a, typ_var_b],
-            [
-                relay.Constructor("Left", [typ_var_a], glob_typ_var),
-                relay.Constructor("Right", [typ_var_b], glob_typ_var),
-            ])
-    mod = relay.Module()
+        glob_typ_var,
+        [typ_var_a, typ_var_b],
+        [
+            relay.Constructor("Left", [typ_var_a], glob_typ_var),
+            relay.Constructor("Right", [typ_var_b], glob_typ_var),
+        ],
+    )
+    mod = tvm.IRModule()
     mod[glob_typ_var] = prog
-    assert parses_as(
+    assert_parse_module_as(
         """
         type Either[A, B] {
           Left(A),
           Right(B),
         }
         """,
-        mod
+        mod,
     )
 
 
@@ -713,17 +656,13 @@ def test_match():
     # pair each match keyword with whether it specifies a complete match or not
     match_keywords = [("match", True), ("match?", False)]
     for (match_keyword, is_complete) in match_keywords:
-        mod = relay.Module()
+        mod = tvm.IRModule()
 
         list_var = relay.GlobalTypeVar("List")
         typ_var = relay.TypeVar("A")
-        cons_constructor = relay.Constructor(
-            "Cons", [typ_var, list_var(typ_var)], list_var)
+        cons_constructor = relay.Constructor("Cons", [typ_var, list_var(typ_var)], list_var)
         nil_constructor = relay.Constructor("Nil", [], list_var)
-        list_def = relay.TypeData(
-            list_var,
-            [typ_var],
-            [cons_constructor, nil_constructor])
+        list_def = relay.TypeData(list_var, [typ_var], [cons_constructor, nil_constructor])
         mod[list_var] = list_def
 
         length_var = relay.GlobalVar("length")
@@ -732,173 +671,187 @@ def test_match():
         input_var = relay.Var("xs", input_type)
         rest_var = relay.Var("rest")
         cons_case = relay.Let(
-            _,
+            relay.var("", type_annotation=None),
             UNIT,
-            relay.add(relay.const(1), relay.Call(length_var, [rest_var])))
-        body = relay.Match(input_var,
-            [relay.Clause(
-                relay.PatternConstructor(
-                    cons_constructor,
-                    [relay.PatternWildcard(), relay.PatternVar(rest_var)]),
-                cons_case),
-            relay.Clause(
-                relay.PatternConstructor(nil_constructor, []),
-                relay.const(0))],
-            complete=is_complete
+            relay.add(relay.const(1), relay.Call(length_var, [rest_var])),
         )
-        length_func = relay.Function(
-            [input_var],
-            body,
-            int32,
-            [typ_var]
+        body = relay.Match(
+            input_var,
+            [
+                relay.Clause(
+                    relay.PatternConstructor(
+                        cons_constructor, [relay.PatternWildcard(), relay.PatternVar(rest_var)]
+                    ),
+                    cons_case,
+                ),
+                relay.Clause(relay.PatternConstructor(nil_constructor, []), relay.const(0)),
+            ],
+            complete=is_complete,
         )
+        length_func = relay.Function([input_var], body, int32, [typ_var])
         mod[length_var] = length_func
 
-        assert parses_as(
+        assert_parse_module_as(
             """
             %s
 
             def @length[A](%%xs: List[A]) -> int32 {
               %s (%%xs) {
-                Cons(_, %%rest) => {
-                  ();;
+                Cons(_, %%rest : List[A]) => {
+                  ();
                   1 + @length(%%rest)
                 },
                 Nil => 0,
               }
             }
-            """ % (LIST_DEFN, match_keyword),
-            mod
+            """
+            % (LIST_DEFN, match_keyword),
+            mod,
         )
 
 
 def test_adt_cons_expr():
-    mod = relay.Module()
+    mod = tvm.IRModule()
 
     list_var = relay.GlobalTypeVar("List")
     typ_var = relay.TypeVar("A")
-    cons_constructor = relay.Constructor(
-        "Cons", [typ_var, list_var(typ_var)], list_var)
+    cons_constructor = relay.Constructor("Cons", [typ_var, list_var(typ_var)], list_var)
     nil_constructor = relay.Constructor("Nil", [], list_var)
-    list_def = relay.TypeData(
-        list_var,
-        [typ_var],
-        [cons_constructor, nil_constructor])
+    list_def = relay.TypeData(list_var, [typ_var], [cons_constructor, nil_constructor])
     mod[list_var] = list_def
 
     make_singleton_var = relay.GlobalVar("make_singleton")
     input_var = relay.Var("x", int32)
     make_singleton_func = relay.Function(
-        [input_var],
-        cons_constructor(input_var, nil_constructor()),
-        list_var(int32)
+        [input_var], cons_constructor(input_var, nil_constructor()), list_var(int32)
     )
     mod[make_singleton_var] = make_singleton_func
 
-    assert parses_as(
+    assert_parse_module_as(
         """
         %s
 
         def @make_singleton(%%x: int32) -> List[int32] {
           Cons(%%x, Nil)
         }
-        """ % LIST_DEFN,
-        mod
+        """
+        % LIST_DEFN,
+        mod,
     )
 
 
-@raises_parse_error
 def test_duplicate_adt_defn():
-    parse_text(
-        """
-        %s
+    with pytest.raises(tvm.error.DiagnosticError):
+        parse_module(
+            """
+            %s
 
-        type List[A] {
-          Cons(A, List[A]),
-          Nil,
-        }
-        """ % LIST_DEFN
-    )
+            type List[A] {
+            Cons(A, List[A]),
+            Nil,
+            }
+            """
+            % LIST_DEFN
+        )
 
 
-@raises_parse_error
 def test_duplicate_adt_cons():
-    parse_text(
-        """
-        type Ayy { Lmao }
-        type Haha { Lmao }
-        """
-    )
+    with pytest.raises(tvm.error.DiagnosticError):
+        parse_text(
+            """
+            type Ayy { Lmao }
+            type Haha { Lmao }
+            """
+        )
 
 
-@raises_parse_error
 def test_duplicate_adt_cons_defn():
-    parse_text(
-        """
-        type Ayy { Lmao }
-        type Lmao { Ayy }
-        """
-    )
+    with pytest.raises(tvm.error.DiagnosticError):
+        parse_text(
+            """
+            type Ayy { Lmao }
+            type Lmao { Ayy }
+            """
+        )
 
 
-@raises_parse_error
 def test_duplicate_global_var():
-    parse_text(
-        """
-        def @id[A](%x: A) -> A { x }
-        def @id[A](%x: A) -> A { x }
-        """
-    )
+    with pytest.raises(tvm.error.DiagnosticError):
+        parse_text(
+            """
+            def @id[A](%x: A) -> A { x }
+            def @id[A](%x: A) -> A { x }
+            """
+        )
 
 
 def test_extern_adt_defn():
-    # TODO(weberlo): update this test once extern is implemented
-    mod = relay.Module()
+    mod = tvm.IRModule()
 
     extern_var = relay.GlobalTypeVar("T")
     typ_var = relay.TypeVar("A")
     extern_def = relay.TypeData(extern_var, [typ_var], [])
     mod[extern_var] = extern_def
 
-    assert parses_as(
+    assert_parse_module_as(
         """
         extern type T[A]
         """,
-        mod
+        mod,
     )
 
 
+def test_import_grad():
+    mod = tvm.IRModule()
+    mod.import_from_std("gradient.rly")
+
+
+def test_resnet():
+    mod, _ = relay.testing.resnet.get_workload()
+    text = mod.astext()
+    parsed_mod = tvm.parser.parse(text)
+    tvm.ir.assert_structural_equal(mod, parsed_mod)
+
+
+def inline_params(mod, params):
+    main_fn = mod["main"]
+    str_to_var = {}
+    for param in main_fn.params:
+        str_to_var[param.name_hint] = param
+
+    bind_map = {}
+    for param in params:
+        bind_map[str_to_var[param]] = relay.const(params[param])
+
+    body = relay.bind(main_fn.body, bind_map)
+    main_fn = relay.Function(relay.analysis.free_vars(body), body)
+    mod["main_fn"] = main_fn
+    return mod
+
+
+def test_resnet_inlined_params():
+    mod, params = relay.testing.resnet.get_workload()
+    mod = inline_params(mod, params)
+    text = mod.astext()
+    parsed_mod = tvm.parser.parse(text)
+    tvm.ir.assert_structural_equal(mod, parsed_mod)
+
+
+def test_op_string_attr():
+    call = parse_text(
+        """
+        free_var %x: Tensor[(1, 32, 32, 3), float32];
+        free_var %y: Tensor[(1, 1, 3, 3), float32];
+        nn.conv2d(%x, %y, data_layout="NHWC", kernel_layout="HWIO")
+        """
+    )
+    assert isinstance(call.op, tvm.ir.Op)
+    assert call.op.name == "nn.conv2d"
+    assert call.attrs.data_layout == "NHWC"
+    assert call.attrs.kernel_layout == "HWIO"
+
+
 if __name__ == "__main__":
-    test_comments()
-    test_int_literal()
-    test_float_literal()
-    test_bool_literal()
-    test_negative()
-    test_bin_op()
-    test_parens()
-    test_op_assoc()
-    test_let()
-    test_seq()
-    test_graph()
-    test_tuple()
-    test_func()
-    test_defn()
-    test_recursive_call()
-    test_ifelse()
-    test_call()
-    test_incomplete_type()
-    test_builtin_types()
-    test_tensor_type()
-    test_function_type()
-    test_tuple_type()
-    test_adt_defn()
-    test_empty_adt_defn()
-    test_multiple_cons_defn()
-    test_multiple_type_param_defn()
-    test_match()
-    test_adt_cons_expr()
-    test_duplicate_adt_defn()
-    test_duplicate_adt_cons()
-    test_duplicate_adt_cons_defn()
-    test_duplicate_global_var()
-    test_extern_adt_defn()
+    import sys
+
+    pytest.main(sys.argv)
