@@ -23,13 +23,47 @@ from tvm import relay
 from tvm.contrib import graph_runtime
 from tvm.relay.expr_functor import ExprMutator
 from tvm.relay import transform
+import tvm.testing
+
+
+def _trace(module, metadata, _):
+    if metadata.name == "ManifestAlloc":
+        pass  # import pdb; pdb.set_trace()
+
+
+def check_graph_runtime(
+    target, ref_res, device, func, params, config, opt_level, expected_index=None
+):
+    with tvm.transform.PassContext(opt_level=opt_level, config=config):
+        graph, lib, new_params = relay.build(func, target, params=params)
+        contexts = [tvm.cpu(0), tvm.context(device)]
+        graph_json = json.loads(graph)
+        if "device_index" in graph_json["attrs"]:
+            device_index = graph_json["attrs"]["device_index"][1]
+            assert device_index == expected_index
+        mod = graph_runtime.create(graph, lib, contexts)
+        mod.set_input(**new_params)
+        mod.run()
+        res = mod.get_output(0).asnumpy()
+        tvm.testing.assert_allclose(res, ref_res, rtol=1e-5, atol=1e-5)
+
+
+def check_vm_runtime(target, ref_res, device, func, params, config, opt_level, expected_index=None):
+    with tvm.transform.PassContext(opt_level=opt_level, trace=_trace, config=config):
+        mod = tvm.IRModule()
+        mod["main"] = func
+        exe = relay.vm.compile(mod, target)
+        ctx = [tvm.cpu(0), tvm.context(device)]
+        vm = tvm.runtime.vm.VirtualMachine(exe, ctx)
+        res = vm.invoke("main", **params)
+        tvm.testing.assert_allclose(res.asnumpy(), ref_res, rtol=1e-5, atol=1e-5)
 
 
 def run_opt_pass(expr, passes):
     passes = passes if isinstance(passes, list) else [passes]
-    mod = relay.Module.from_expr(expr)
-    seq = transform.Sequential(passes)
-    with transform.PassContext(opt_level=3):
+    mod = tvm.IRModule.from_expr(expr)
+    seq = tvm.transform.Sequential(passes)
+    with tvm.transform.PassContext(opt_level=3):
         mod = seq(mod)
     return mod["main"]
 
@@ -49,8 +83,7 @@ def test_redundant_annotation():
         sub2 = relay.subtract(_add2, z)
 
         func = relay.Function([x, y, z], relay.Tuple([sub1, sub2]))
-        func = run_opt_pass(func,
-                            transform.RewriteAnnotatedOps(ctx1.device_type))
+        func = run_opt_pass(func, transform.RewriteAnnotatedOps(ctx1.device_type))
         return func
 
     def expected():
@@ -64,7 +97,7 @@ def test_redundant_annotation():
 
     annotated_func = annotated()
     expected_func = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.alpha_equal(annotated_func, expected_func)
+    assert tvm.ir.structural_equal(annotated_func, expected_func)
 
 
 def test_annotate_expr():
@@ -79,8 +112,7 @@ def test_annotate_expr():
         _add = relay.annotation.on_device(add, ctx1)
         sub = relay.subtract(_add, z)
         _sub = relay.annotation.on_device(sub, ctx2)
-        expr = run_opt_pass(_sub,
-                            transform.RewriteAnnotatedOps(ctx1.device_type))
+        expr = run_opt_pass(_sub, transform.RewriteAnnotatedOps(ctx1.device_type))
         return expr
 
     def expected():
@@ -91,7 +123,7 @@ def test_annotate_expr():
 
     annotated_expr = annotated()
     expected_expr = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.graph_equal(annotated_expr, expected_expr)
+    assert tvm.ir.structural_equal(annotated_expr, expected_expr)
 
 
 def test_annotate_all():
@@ -108,8 +140,7 @@ def test_annotate_all():
         _sub = relay.annotation.on_device(sub, ctx2)
 
         func = relay.Function([x, y, z], _sub)
-        func = run_opt_pass(func,
-                            transform.RewriteAnnotatedOps(ctx1.device_type))
+        func = run_opt_pass(func, transform.RewriteAnnotatedOps(ctx1.device_type))
         return func
 
     def expected():
@@ -120,7 +151,7 @@ def test_annotate_all():
 
     annotated_func = annotated()
     expected_func = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.graph_equal(annotated_func, expected_func)
+    assert tvm.ir.structural_equal(annotated_func, expected_func)
 
 
 def test_annotate_none():
@@ -134,8 +165,7 @@ def test_annotate_none():
         add = relay.add(x, y)
         sub = relay.subtract(add, z)
         func = relay.Function([x, y, z], sub)
-        func = run_opt_pass(func,
-                            transform.RewriteAnnotatedOps(ctx1.device_type))
+        func = run_opt_pass(func, transform.RewriteAnnotatedOps(ctx1.device_type))
         return func
 
     def expected():
@@ -146,24 +176,24 @@ def test_annotate_none():
 
     annotated_func = annotated()
     expected_func = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.graph_equal(annotated_func, expected_func)
+    assert tvm.ir.structural_equal(annotated_func, expected_func)
 
 
 def check_annotated_graph(annotated_func, expected_func):
     annotated_func = run_opt_pass(annotated_func, transform.InferType())
     expected_func = run_opt_pass(expected_func, transform.InferType())
-    assert relay.analysis.alpha_equal(annotated_func, expected_func)
+    assert tvm.ir.structural_equal(annotated_func, expected_func)
 
 
 def test_conv_network():
-    R""" The network is as following:
-             data1     data2
-               |         |
-             conv2d    conv2d
-                \       /
-                   add
-                    |
-                  conv2d
+    R"""The network is as following:
+    data1     data2
+      |         |
+    conv2d    conv2d
+       \       /
+          add
+           |
+         conv2d
     """
     batch_size = 1
     dshape = (batch_size, 64, 56, 56)
@@ -174,60 +204,27 @@ def test_conv_network():
     dev2 = tvm.context(2)
 
     def original():
-        conv2d_1 = relay.nn.conv2d(
-            data1,
-            weight,
-            channels=64,
-            kernel_size=(3, 3),
-            padding=(1, 1))
-        conv2d_2 = relay.nn.conv2d(
-            data2,
-            weight,
-            channels=64,
-            kernel_size=(3, 3),
-            padding=(1, 1))
+        conv2d_1 = relay.nn.conv2d(data1, weight, channels=64, kernel_size=(3, 3), padding=(1, 1))
+        conv2d_2 = relay.nn.conv2d(data2, weight, channels=64, kernel_size=(3, 3), padding=(1, 1))
         add = relay.add(conv2d_1, conv2d_2)
-        conv2d_3 = relay.nn.conv2d(
-            add,
-            weight,
-            channels=64,
-            kernel_size=(3, 3),
-            padding=(1, 1))
+        conv2d_3 = relay.nn.conv2d(add, weight, channels=64, kernel_size=(3, 3), padding=(1, 1))
 
         func = relay.Function([data1, data2, weight], conv2d_3)
-        func = run_opt_pass(
-            func, transform.RewriteAnnotatedOps(tvm.context(3).device_type))
+        func = run_opt_pass(func, transform.RewriteAnnotatedOps(tvm.context(3).device_type))
         return func
 
-
     def annotated():
-        conv2d_1 = relay.nn.conv2d(
-            data1,
-            weight,
-            channels=64,
-            kernel_size=(3, 3),
-            padding=(1, 1))
+        conv2d_1 = relay.nn.conv2d(data1, weight, channels=64, kernel_size=(3, 3), padding=(1, 1))
         _conv2d_1 = relay.annotation.on_device(conv2d_1, dev2)
-        conv2d_2 = relay.nn.conv2d(
-            data2,
-            weight,
-            channels=64,
-            kernel_size=(3, 3),
-            padding=(1, 1))
+        conv2d_2 = relay.nn.conv2d(data2, weight, channels=64, kernel_size=(3, 3), padding=(1, 1))
         _conv2d_2 = relay.annotation.on_device(conv2d_2, dev2)
         add = relay.add(_conv2d_1, _conv2d_2)
         _add = relay.annotation.on_device(add, dev1)
-        conv2d_3 = relay.nn.conv2d(
-            _add,
-            weight,
-            channels=64,
-            kernel_size=(3, 3),
-            padding=(1, 1))
+        conv2d_3 = relay.nn.conv2d(_add, weight, channels=64, kernel_size=(3, 3), padding=(1, 1))
         _conv2d_3 = relay.annotation.on_device(conv2d_3, dev2)
 
         func = relay.Function([data1, data2, weight], _conv2d_3)
-        func = run_opt_pass(
-            func, transform.RewriteAnnotatedOps(tvm.context(3).device_type))
+        func = run_opt_pass(func, transform.RewriteAnnotatedOps(tvm.context(3).device_type))
         return func
 
     class ScheduleConv2d(ExprMutator):
@@ -245,41 +242,26 @@ def test_conv_network():
     def annotate_with_visitor(func):
         sched = ScheduleConv2d(dev2)
         func = sched.visit(func)
-        func = run_opt_pass(
-            func, transform.RewriteAnnotatedOps(dev1.device_type))
+        func = run_opt_pass(func, transform.RewriteAnnotatedOps(dev1.device_type))
         return func
 
     def expected():
-        conv2d_1 = relay.nn.conv2d(
-            data1,
-            weight,
-            channels=64,
-            kernel_size=(3, 3),
-            padding=(1, 1))
+        conv2d_1 = relay.nn.conv2d(data1, weight, channels=64, kernel_size=(3, 3), padding=(1, 1))
         device_copy1 = relay.device_copy(conv2d_1, dev2, dev1)
-        conv2d_2 = relay.nn.conv2d(
-            data2,
-            weight,
-            channels=64,
-            kernel_size=(3, 3),
-            padding=(1, 1))
+        conv2d_2 = relay.nn.conv2d(data2, weight, channels=64, kernel_size=(3, 3), padding=(1, 1))
         device_copy2 = relay.device_copy(conv2d_2, dev2, dev1)
         add = relay.add(device_copy1, device_copy2)
         device_copy3 = relay.device_copy(add, dev1, dev2)
         conv2d_3 = relay.nn.conv2d(
-            device_copy3,
-            weight,
-            channels=64,
-            kernel_size=(3, 3),
-            padding=(1, 1))
+            device_copy3, weight, channels=64, kernel_size=(3, 3), padding=(1, 1)
+        )
 
         func = relay.Function([data1, data2, weight], conv2d_3)
         return func
 
     def check_storage_and_device_types():
         func = annotated()
-        func = run_opt_pass(func, [transform.RewriteAnnotatedOps(3),
-                                   transform.FuseOps(2)])
+        func = run_opt_pass(func, [transform.RewriteAnnotatedOps(3), transform.FuseOps(2)])
         smap = relay.backend._backend.GraphPlanMemory(func)
         storage_ids = []
         device_types = []
@@ -309,6 +291,70 @@ def test_conv_network():
     test_visitor_annotation()
 
 
+def test_propogation():
+    R""" The network and device type is as following:
+                  x           1
+                  |
+                 log          1
+                /   \
+              log2 log10      2
+                \   /
+                 add          2
+                  |
+                 tan          1
+    """
+    ctx1 = tvm.context(1)
+    ctx2 = tvm.context(2)
+
+    expected_dev_type = {"log": ctx1, "log2": ctx2, "log10": ctx2, "add": ctx2, "tan": ctx1}
+
+    x = relay.var("x", shape=(3,))
+
+    def annotated():
+        log = relay.log(x)
+        _log = relay.annotation.on_device(log, expected_dev_type["log"])
+        log2 = relay.log2(_log)
+        _log2 = relay.annotation.on_device(log2, expected_dev_type["log2"])
+        log10 = relay.log10(_log)
+        _log10 = relay.annotation.on_device(log10, expected_dev_type["log10"])
+        add = relay.add(_log2, _log10)
+        _add = relay.annotation.on_device(add, expected_dev_type["add"])
+        tan = relay.tan(_add)
+        _tan = relay.annotation.on_device(tan, expected_dev_type["tan"])
+
+        func = run_opt_pass(_tan, transform.RewriteAnnotatedOps(ctx1.device_type))
+        return func
+
+    def expected():
+        log = relay.log(x)
+        _log_left = relay.device_copy(log, ctx1, ctx2)
+        _log_right = relay.device_copy(log, ctx1, ctx2)
+        log2 = relay.log2(_log_left)
+        log10 = relay.log10(_log_right)
+        add = relay.add(log2, log10)
+        _add = relay.device_copy(add, ctx2, ctx1)
+        tan = relay.tan(_add)
+
+        func = run_opt_pass(tan, transform.InferType())
+        return func
+
+    annotated_expr = annotated()
+    expected_expr = expected()
+    assert tvm.ir.structural_equal(annotated_expr, expected_expr)
+
+    smap = relay.backend._backend.GraphPlanMemory(annotated_expr)
+    for expr, storage_dev_type in smap.items():
+        # x is ctx1 as output is ctx1
+        if isinstance(expr, tvm.relay.expr.Var):
+            assert storage_dev_type[1][0] == ctx1.device_type
+        else:
+            # device_copy op should be its dst_dev_type
+            if isinstance(expr.attrs, tvm.relay.op.op_attrs.DeviceCopyAttrs):
+                assert storage_dev_type[1][0] == expr.attrs.dst_dev_type
+            else:
+                assert storage_dev_type[1][0] == expected_dev_type[expr.op.name].device_type
+
+
 def run_fusible_network(dev, tgt):
     R""" The network is as following:
                x     y
@@ -323,13 +369,14 @@ def run_fusible_network(dev, tgt):
     """
     x = relay.var("x", shape=(1, 10))
     y = relay.var("y", shape=(10, 10))
-    x_data = np.random.rand(1, 10).astype('float32')
-    y_data = np.random.rand(10, 10).astype('float32')
+    x_data = np.random.rand(1, 10).astype("float32")
+    y_data = np.random.rand(10, 10).astype("float32")
     tmp_add = x_data + y_data
     tmp_sqrt = np.sqrt(tmp_add)
     tmp_log = np.log(tmp_add)
     tmp_sub = np.subtract(tmp_sqrt, tmp_log)
     ref_res = np.exp(tmp_sub)
+    params = {"x": x_data, "y": y_data}
 
     def get_func():
         add = relay.add(x, y)
@@ -341,28 +388,6 @@ def run_fusible_network(dev, tgt):
         func = relay.Function([x, y], exp)
         return func
 
-    def test_runtime(target, device, func, fallback_device=None,
-                     expected_index=None):
-        params = {"x": x_data, "y": y_data}
-        config = {"opt_level": 1}
-        if fallback_device:
-            config["fallback_device"] = fallback_device
-        with relay.build_config(**config):
-            graph, lib, params = relay.build(
-                func,
-                target,
-                params=params)
-            contexts = [tvm.cpu(0), tvm.context(device)]
-            graph_json = json.loads(graph)
-            if "device_index" in graph_json["attrs"]:
-                device_index = graph_json["attrs"]["device_index"][1]
-                assert device_index == expected_index
-            mod = graph_runtime.create(graph, lib, contexts)
-            mod.set_input(**params)
-            mod.run()
-            res = mod.get_output(0).asnumpy()
-            tvm.testing.assert_allclose(res, ref_res, rtol=1e-5, atol=1e-5)
-
     def test_fuse_log_add(device, tgt):
         """ Only log and add are fused."""
         fallback_device = tvm.context("cpu")
@@ -380,8 +405,7 @@ def run_fusible_network(dev, tgt):
             _exp = relay.annotation.on_device(exp, dev_ctx)
 
             func = relay.Function([x, y], _exp)
-            func = run_opt_pass(
-                func, transform.RewriteAnnotatedOps(cpu_ctx.device_type))
+            func = run_opt_pass(func, transform.RewriteAnnotatedOps(cpu_ctx.device_type))
             return func
 
         def expected():
@@ -403,8 +427,15 @@ def run_fusible_network(dev, tgt):
         dev_idx = ctx.device_type
         expected_index = [1, 1, 1, dev_idx, dev_idx, 1, 1, dev_idx, dev_idx]
         check_annotated_graph(annotated_func, expected_func)
-        test_runtime(target, device, annotated_func, fallback_device,
-                     expected_index)
+        opt_level = 1
+        config = {"relay.fallback_device_type": fallback_device.device_type}
+        check_graph_runtime(
+            target, ref_res, device, annotated_func, params, config, opt_level, expected_index
+        )
+        opt_level = 2
+        check_vm_runtime(
+            target, ref_res, device, annotated_func, params, config, opt_level, expected_index
+        )
 
     def test_fuse_all(device, tgt):
         """Fuse all operators."""
@@ -426,14 +457,17 @@ def run_fusible_network(dev, tgt):
             _exp = relay.annotation.on_device(exp, dev_ctx)
 
             func = relay.Function([x, y], _exp)
-            func = run_opt_pass(
-                func, transform.RewriteAnnotatedOps(cpu_ctx.device_type))
+            func = run_opt_pass(func, transform.RewriteAnnotatedOps(cpu_ctx.device_type))
             return func
 
         annotated_func = annotated()
         expected_func = get_func()
         check_annotated_graph(annotated_func, expected_func)
-        test_runtime(target, device, annotated_func, fallback_device)
+        opt_level = 1
+        config = {"relay.fallback_device_type": fallback_device.device_type}
+        check_graph_runtime(target, ref_res, device, annotated_func, params, config, opt_level)
+        opt_level = 2
+        check_vm_runtime(target, ref_res, device, annotated_func, params, config, opt_level)
 
     def test_fallback_exp(device, tgt):
         fallback_device = tvm.context("cpu")
@@ -450,8 +484,7 @@ def run_fusible_network(dev, tgt):
             _exp = relay.annotation.on_device(exp, cpu_ctx)
 
             func = relay.Function([x, y], _exp)
-            func = run_opt_pass(
-                func, transform.RewriteAnnotatedOps(dev_ctx.device_type))
+            func = run_opt_pass(func, transform.RewriteAnnotatedOps(dev_ctx.device_type))
             return func
 
         def expected():
@@ -470,40 +503,49 @@ def run_fusible_network(dev, tgt):
         ctx = tvm.context(device, 0)
         dev_idx = ctx.device_type
         expected_index = [dev_idx, dev_idx, dev_idx, 1, 1]
+        opt_level = 1
+        config = {"relay.fallback_device_type": fallback_device.device_type}
         check_annotated_graph(annotated_func, expected_func)
-        test_runtime(target, device, annotated_func, fallback_device,
-                     expected_index)
+        check_graph_runtime(
+            target, ref_res, device, annotated_func, params, config, opt_level, expected_index
+        )
+        opt_level = 2
+        check_vm_runtime(
+            target, ref_res, device, annotated_func, params, config, opt_level, expected_index
+        )
 
     def test_fallback_all_operators(device, tgt):
         target = {device: tgt, "cpu": "llvm"}
         annotated_func = get_func()
         expected_func = get_func()
         check_annotated_graph(annotated_func, expected_func)
-        test_runtime(target, device, annotated_func)
-
+        opt_level = 2
+        check_graph_runtime(target, ref_res, device, annotated_func, params, {}, opt_level)
+        check_vm_runtime(target, ref_res, device, annotated_func, params, {}, opt_level)
 
     test_fuse_log_add(dev, tgt)
     test_fuse_all(dev, tgt)
     test_fallback_exp(dev, tgt)
     test_fallback_all_operators(dev, tgt)
 
+
 def run_unpropagatable_graph(dev, tgt):
-    R""" The network is as following:
-            a     b  c     d
-             \   /    \   /
-              add      mul
-                \      /
-                subtract
+    R"""The network is as following:
+    a     b  c     d
+     \   /    \   /
+      add      mul
+        \      /
+        subtract
     """
 
     a = relay.var("a", shape=(10, 10))
     b = relay.var("b", shape=(10, 10))
     c = relay.var("c", shape=(10, 10))
     d = relay.var("d", shape=(10, 10))
-    a_data = np.random.rand(10, 10).astype('float32')
-    b_data = np.random.rand(10, 10).astype('float32')
-    c_data = np.random.rand(10, 10).astype('float32')
-    d_data = np.random.rand(10, 10).astype('float32')
+    a_data = np.random.rand(10, 10).astype("float32")
+    b_data = np.random.rand(10, 10).astype("float32")
+    c_data = np.random.rand(10, 10).astype("float32")
+    d_data = np.random.rand(10, 10).astype("float32")
     tmp_add = a_data + b_data
     tmp_mul = np.multiply(c_data, d_data)
     ref_res = np.subtract(tmp_add, tmp_mul)
@@ -521,8 +563,7 @@ def run_unpropagatable_graph(dev, tgt):
         sub = relay.subtract(_add, _mul)
         _sub = relay.annotation.on_device(sub, dev_ctx)
         func = relay.Function([a, b, c, d], _sub)
-        func = run_opt_pass(
-            func, transform.RewriteAnnotatedOps(dev_ctx.device_type))
+        func = run_opt_pass(func, transform.RewriteAnnotatedOps(dev_ctx.device_type))
         return func
 
     def expected():
@@ -538,38 +579,44 @@ def run_unpropagatable_graph(dev, tgt):
     expected_index = [2, 2, 2, 1, 1, 1, 2, 2]
     check_annotated_graph(annotated_func, expected_func)
     params = {"a": a_data, "b": b_data, "c": c_data, "d": d_data}
-    config = {"opt_level": 0}
-    config["fallback_device"] = fallback_device
-    with relay.build_config(**config):
-        graph, lib, params = relay.build(annotated_func, target, params=params)
-        contexts = [tvm.cpu(0), tvm.context(dev)]
-        graph_json = json.loads(graph)
-        if "device_index" in graph_json["attrs"]:
-            device_index = graph_json["attrs"]["device_index"][1]
-            assert device_index == expected_index
-        mod = graph_runtime.create(graph, lib, contexts)
-        mod.set_input(**params)
-        mod.run()
-        res = mod.get_output(0).asnumpy()
-        tvm.testing.assert_allclose(res, ref_res, rtol=1e-5, atol=1e-5)
+    opt_level = 0
+    config = {"relay.fallback_device_type": fallback_device.device_type}
+
+    check_graph_runtime(
+        target, ref_res, dev, annotated_func, params, config, opt_level, expected_index
+    )
+
+    opt_level = 2
+    check_vm_runtime(target, ref_res, dev, annotated_func, params, config, opt_level)
 
 
-def test_check_run():
-    for dev, tgt in [("opencl", "opencl"), ("cuda", "cuda"),
-                 ("opencl", str(tvm.target.intel_graphics()))]:
-        if not tvm.module.enabled(dev):
-            print("Skip test because %s is not enabled." % dev)
-            continue
-        run_fusible_network(dev, tgt)
-        run_unpropagatable_graph(dev, tgt)
+@tvm.testing.requires_opencl
+def test_check_run_opencl():
+    dev = "opencl"
+    tgt = "opencl"
+    run_fusible_network(dev, tgt)
+    run_unpropagatable_graph(dev, tgt)
 
 
+@tvm.testing.requires_opencl
+def test_check_run_opencl_intel():
+    dev = "opencl"
+    tgt = str(tvm.target.intel_graphics())
+    run_fusible_network(dev, tgt)
+    run_unpropagatable_graph(dev, tgt)
+
+
+@tvm.testing.requires_cuda
+def test_check_run_cuda():
+    dev = "cuda"
+    tgt = "cuda"
+    run_fusible_network(dev, tgt)
+    run_unpropagatable_graph(dev, tgt)
+
+
+@tvm.testing.requires_cuda
 def test_tuple_get_item():
     dev = "cuda"
-    if not tvm.module.enabled(dev):
-        print("Skip test because %s is not enabled." % dev)
-        return
-
     cpu_ctx = tvm.cpu(0)
     gpu_ctx = tvm.context(dev)
 
@@ -590,13 +637,12 @@ def test_tuple_get_item():
         split = relay.TupleWrapper(split, 3)
         sub = split[0] - split[1]
         func = relay.Function(relay.analysis.free_vars(sub), sub)
-        func = run_opt_pass(
-            func, transform.RewriteAnnotatedOps(cpu_ctx.device_type))
+        func = run_opt_pass(func, transform.RewriteAnnotatedOps(cpu_ctx.device_type))
         return func
 
     annotated_func = annotated()
     expected_func = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.graph_equal(annotated_func, expected_func)
+    assert tvm.ir.structural_equal(annotated_func, expected_func)
 
 
 if __name__ == "__main__":
@@ -605,5 +651,4 @@ if __name__ == "__main__":
     test_annotate_all()
     test_annotate_none()
     test_conv_network()
-    test_check_run()
     test_tuple_get_item()
