@@ -16,14 +16,17 @@
 # under the License.
 import numpy as np
 import tvm
+from tvm import te
 from tvm import relay
 from tvm.relay import transform
+from tvm.relay.build_module import bind_params_by_name
+from tvm.relay.testing import run_infer_type, create_workload
 
 
 def run_opt_pass(expr, opt_pass):
-    assert isinstance(opt_pass, transform.Pass)
+    assert isinstance(opt_pass, tvm.transform.Pass)
 
-    mod = relay.Module.from_expr(expr)
+    mod = tvm.IRModule.from_expr(expr)
     mod = opt_pass(mod)
     entry = mod["main"]
     return entry if isinstance(expr, relay.Function) else entry.body
@@ -45,12 +48,13 @@ def test_concatenate_const():
 
     zz = run_opt_pass(before(), transform.FoldConstant())
     zexpected = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.graph_equal(zz, zexpected)
+    assert tvm.ir.structural_equal(zz, zexpected)
 
 
 def test_fold_const():
     c_data = np.array([1, 2, 3]).astype("float32")
     t = relay.TensorType([1, 2, 3], "float32")
+
     def before():
         c = relay.const(c_data)
         x = relay.var("x", t)
@@ -67,20 +71,17 @@ def test_fold_const():
         z = relay.add(y, relay.const(c_data))
         return relay.Function([x], z)
 
-    def fail(x):
-        raise RuntimeError()
-
     # the fold constant should work on any context.
-    with tvm.build_config(add_lower_pass=[(0, fail)]):
-        with tvm.target.create("cuda"):
-            zz = run_opt_pass(before(), transform.FoldConstant())
+    with tvm.target.Target("cuda"):
+        zz = run_opt_pass(before(), transform.FoldConstant())
     zexpected = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.alpha_equal(zz, zexpected)
+    assert tvm.ir.structural_equal(zz, zexpected)
 
 
 def test_fold_let():
     c_data = np.array(1).astype("float32")
     t = relay.TensorType([1], "float32")
+
     def before():
         sb = relay.ScopeBuilder()
         x = relay.var("x", t)
@@ -93,19 +94,20 @@ def test_fold_let():
     def expected():
         sb = relay.ScopeBuilder()
         x = relay.var("x", t)
-        c_folded = (c_data + c_data)
+        c_folded = c_data + c_data
         t3 = sb.let("t3", relay.add(relay.const(c_folded), x))
         sb.ret(t3)
         return relay.Function([x], sb.get())
 
     zz = run_opt_pass(before(), transform.FoldConstant())
     zexpected = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.graph_equal(zz, zexpected)
+    assert tvm.ir.structural_equal(zz, zexpected)
 
 
 def test_fold_tuple():
     c_data = np.array(1).astype("float32")
     t = relay.TensorType([1], "float32")
+
     def before():
         c = relay.const(c_data)
         x = relay.var("x", t)
@@ -122,7 +124,7 @@ def test_fold_tuple():
 
     zz = run_opt_pass(before(), transform.FoldConstant())
     zexpected = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.graph_equal(zz, zexpected)
+    assert tvm.ir.structural_equal(zz, zexpected)
 
 
 def test_fold_concat():
@@ -141,11 +143,12 @@ def test_fold_concat():
 
     zz = run_opt_pass(before(), transform.FoldConstant())
     zexpected = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.graph_equal(zz, zexpected)
+    assert tvm.ir.structural_equal(zz, zexpected)
 
 
 def test_fold_shape_of():
     c_shape = (8, 9, 10)
+
     def before(dtype):
         x = relay.var("x", shape=c_shape, dtype="float32")
         y = relay.var("y", shape=c_shape, dtype="float32")
@@ -162,13 +165,36 @@ def test_fold_shape_of():
     for dtype in ["int32", "float32"]:
         zz = run_opt_pass(before(dtype), transform.FoldConstant())
         zexpected = run_opt_pass(expected(dtype), transform.InferType())
-        assert relay.analysis.graph_equal(zz, zexpected)
+        assert tvm.ir.structural_equal(zz, zexpected)
+
+
+def test_fold_ndarray_size():
+    c_shape = (8, 9, 10)
+
+    def before(dtype):
+        x = relay.var("x", shape=c_shape, dtype="float32")
+        y = relay.var("y", shape=c_shape, dtype="float32")
+        z = relay.ndarray_size(x + y, dtype)
+        return relay.Function([x, y], z)
+
+    def expected(dtype):
+        x = relay.var("x", shape=c_shape, dtype="float32")
+        y = relay.var("y", shape=c_shape, dtype="float32")
+        z = relay.const(np.size(np.zeros(c_shape)), dtype=dtype)
+        func = relay.Function([x, y], z)
+        return func
+
+    for dtype in ["int32", "float32"]:
+        zz = run_opt_pass(before(dtype), transform.FoldConstant())
+        zexpected = run_opt_pass(expected(dtype), transform.InferType())
+        assert tvm.ir.structural_equal(zz, zexpected)
 
 
 def test_fold_full():
     c_shape = (8, 9, 10)
+
     def before():
-        dtype = 'float32'
+        dtype = "float32"
         return relay.full(relay.const(1.0, dtype), c_shape, dtype=dtype)
 
     def expected():
@@ -177,7 +203,52 @@ def test_fold_full():
 
     zz = run_opt_pass(before(), transform.FoldConstant())
     zexpected = run_opt_pass(expected(), transform.InferType())
-    assert relay.analysis.graph_equal(zz, zexpected)
+    assert tvm.ir.structural_equal(zz, zexpected)
+
+
+def test_fold_batch_norm():
+    def expected():
+        data = relay.var("data", relay.TensorType((1, 3, 224, 224), "float32"))
+        weight = relay.const(np.zeros((16, 3, 3, 3)))
+        bias = relay.const(np.zeros((16, 1, 1)))
+        conv = relay.nn.conv2d(
+            data=data, weight=weight, kernel_size=(3, 3), channels=16, padding=(1, 1)
+        )
+        add = relay.add(conv, bias)
+        return relay.Function(relay.analysis.free_vars(add), add)
+
+    remove_bn_pass = tvm.transform.Sequential(
+        [
+            relay.transform.InferType(),
+            relay.transform.SimplifyInference(),
+            relay.transform.FoldConstant(),
+            relay.transform.FoldScaleAxis(),
+        ]
+    )
+
+    data = relay.var("data", relay.TensorType((1, 3, 224, 224), "float32"))
+    weight = relay.var("weight")
+    bn_gamma = relay.var("bn_gamma")
+    bn_beta = relay.var("bn_beta")
+    bn_mmean = relay.var("bn_mean")
+    bn_mvar = relay.var("bn_var")
+
+    conv = relay.nn.conv2d(
+        data=data, weight=weight, kernel_size=(3, 3), channels=16, padding=(1, 1)
+    )
+    bn_output = relay.nn.batch_norm(conv, bn_gamma, bn_beta, bn_mmean, bn_mvar)
+
+    def initializer(_, param):
+        param = np.zeros(param.shape)
+
+    mod, params = create_workload(bn_output[0], initializer)
+    mod["main"] = bind_params_by_name(mod["main"], params)
+
+    with tvm.transform.PassContext(opt_level=3):
+        mod = remove_bn_pass(mod)
+
+    expect = run_infer_type(expected())
+    assert tvm.ir.structural_equal(mod["main"], expect)
 
 
 if __name__ == "__main__":
@@ -187,3 +258,5 @@ if __name__ == "__main__":
     test_fold_concat()
     test_fold_shape_of()
     test_fold_full()
+    test_fold_batch_norm()
+    test_fold_ndarray_size()
