@@ -19,19 +19,14 @@ import numpy as np
 import pytest
 
 import tvm
+from tvm import te
 from tvm import relay
 from tvm.relay import ExprFunctor
 from tvm.relay import Function, Call
 from tvm.relay import analysis
 from tvm.relay import transform as _transform
-from tvm.relay.testing import ctx_list
-
-
-def run_infer_type(expr):
-    mod = relay.Module.from_expr(expr)
-    mod = _transform.InferType()(mod)
-    entry = mod["main"]
-    return entry if isinstance(expr, relay.Function) else entry.body
+from tvm.relay.testing import run_infer_type
+import tvm.testing
 
 
 def get_var_func():
@@ -74,21 +69,18 @@ def update_func(func):
 
         def visit_function(self, fn):
             new_body = self.visit(fn.body)
-            return Function(
-                list(fn.params), new_body, fn.ret_type, fn.type_params,
-                fn.attrs)
+            return Function(list(fn.params), new_body, fn.ret_type, fn.type_params, fn.attrs)
 
     double_value = DoubleValues()
     return double_value.visit(func)
 
 
-class OptTester():
+class OptTester:
     """A helper class for testing the pass manager."""
 
     def __init__(self, mod):
-        if not isinstance(mod, relay.Module):
-            raise TypeError("mod is expected to be the type of "
-                            "relay.Module")
+        if not isinstance(mod, tvm.IRModule):
+            raise TypeError("mod is expected to be the type of " "tvm.IRModule")
         self.mod = mod
 
     def analysis(self):
@@ -98,10 +90,10 @@ class OptTester():
     @staticmethod
     def transform(node, ctx=None):
         """Perform optimization on node."""
-        if isinstance(node, relay.Module):
+        if isinstance(node, tvm.IRModule):
             # Add a function to the module and return an updated module.
             gv, func = get_var_func()
-            mod = relay.Module({gv: func})
+            mod = tvm.IRModule({gv: func})
             mod.update(node)
             return mod
         if isinstance(node, relay.Function):
@@ -110,38 +102,39 @@ class OptTester():
         raise TypeError("Found not supported node type.")
 
 
-def get_rand(shape, dtype='float32'):
+def get_rand(shape, dtype="float32"):
     return tvm.nd.array(np.random.rand(*shape).astype(dtype))
 
 
 def check_func(func, ref_func):
     func = run_infer_type(func)
     ref_func = run_infer_type(ref_func)
-    assert analysis.graph_equal(func, ref_func)
+    assert tvm.ir.structural_equal(func, ref_func)
 
 
+@tvm.testing.uses_gpu
 def test_module_pass():
     shape = (5, 10)
-    dtype = 'float32'
+    dtype = "float32"
     tp = relay.TensorType(shape, dtype)
     x = relay.var("x", tp)
     y = relay.var("y", tp)
     v_add = relay.GlobalVar("myAdd")
     func = relay.Function([x, y], x + y)
-    mod = relay.Module({v_add: func})
+    mod = tvm.IRModule({v_add: func})
 
     pass_name = "module_pass_test"
     opt_level = 0
     opt_tester = OptTester(mod)
     pass_ctx = None
 
-    @_transform.module_pass(opt_level=opt_level, name=pass_name)
+    @tvm.transform.module_pass(opt_level=opt_level, name=pass_name)
     def transform(expr, ctx):
         return opt_tester.transform(expr, ctx)
 
     def test_pass_registration():
         mod_pass = transform
-        assert isinstance(mod_pass, _transform.ModulePass)
+        assert isinstance(mod_pass, tvm.transform.ModulePass)
         pass_info = mod_pass.info
         assert pass_info.name == pass_name
         assert pass_info.opt_level == opt_level
@@ -149,18 +142,19 @@ def test_module_pass():
     def test_pass_registration_no_decorator():
         def direct_transform(expr, ctx):
             return opt_tester.transform(expr, ctx)
-        mod_pass = _transform.module_pass(direct_transform, opt_level=3)
-        assert isinstance(mod_pass, _transform.ModulePass)
+
+        mod_pass = tvm.transform.module_pass(direct_transform, opt_level=3)
+        assert isinstance(mod_pass, tvm.transform.ModulePass)
         pass_info = mod_pass.info
         assert pass_info.name == "direct_transform"
         assert pass_info.opt_level == 3
 
     def test_pass_run():
         module_pass = transform
-        assert pass_name in module_pass.astext()
+        assert pass_name in str(module_pass)
 
         updated_mod = module_pass(mod)
-        assert isinstance(updated_mod, relay.Module)
+        assert isinstance(updated_mod, tvm.IRModule)
 
         # Check the abs function in the updated module.
         v_abs, myabs = get_var_func()
@@ -184,7 +178,7 @@ def test_module_pass():
         x_nd = get_rand(shape, dtype)
         y_nd = get_rand(shape, dtype)
         ref_res = x_nd.asnumpy() + y_nd.asnumpy()
-        for target, ctx in ctx_list():
+        for target, ctx in tvm.testing.enabled_targets():
             exe1 = relay.create_executor("graph", ctx=ctx, target=target)
             exe2 = relay.create_executor("debug", ctx=ctx, target=target)
             res1 = exe1.evaluate(new_add)(x_nd, y_nd)
@@ -201,6 +195,7 @@ def test_function_class_pass():
     @relay.transform.function_pass(opt_level=1)
     class TestReplaceFunc:
         """Simple test function to replace one argument to another."""
+
         def __init__(self, new_func):
             self.new_func = new_func
 
@@ -213,21 +208,22 @@ def test_function_class_pass():
     fpass = TestReplaceFunc(f1)
     assert fpass.info.opt_level == 1
     assert fpass.info.name == "TestReplaceFunc"
-    mod = relay.Module.from_expr(f2)
+    mod = tvm.IRModule.from_expr(f2)
     mod = fpass(mod)
     # wrap in expr
-    mod2 = relay.Module.from_expr(f1)
-    assert relay.alpha_equal(mod["main"], mod2["main"])
+    mod2 = tvm.IRModule.from_expr(f1)
+    assert tvm.ir.structural_equal(mod["main"], mod2["main"])
 
 
+@tvm.testing.uses_gpu
 def test_function_pass():
-    shape = (10, )
-    dtype = 'float32'
+    shape = (10,)
+    dtype = "float32"
     tp = relay.TensorType(shape, dtype)
     x = relay.var("x", tp)
     v_log = relay.GlobalVar("myLog")
     log = relay.Function([x], relay.log(x))
-    mod = relay.Module({v_log: log})
+    mod = tvm.IRModule({v_log: log})
 
     pass_name = "function_pass_test"
     opt_level = 1
@@ -252,6 +248,7 @@ def test_function_pass():
     def test_pass_registration_no_decorator():
         def direct_transform(expr, ctx):
             return opt_tester.transform(expr, ctx)
+
         mod_pass = _transform.function_pass(direct_transform, opt_level=0)
         assert isinstance(mod_pass, _transform.FunctionPass)
         pass_info = mod_pass.info
@@ -260,10 +257,10 @@ def test_function_pass():
 
     def test_pass_run():
         function_pass = transform
-        assert pass_name in function_pass.astext()
+        assert pass_name in str(function_pass)
 
         updated_mod = function_pass(mod)
-        assert isinstance(updated_mod, relay.Module)
+        assert isinstance(updated_mod, tvm.IRModule)
 
         # Check the log function in the updated module.
         new_v_log = updated_mod.get_global_var(v_log.name_hint)
@@ -277,7 +274,7 @@ def test_function_pass():
         # Execute the add function.
         x_nd = get_rand(shape, dtype)
         ref_res = np.log(x_nd.asnumpy() * 2)
-        for target, ctx in ctx_list():
+        for target, ctx in tvm.testing.enabled_targets():
             exe1 = relay.create_executor("graph", ctx=ctx, target=target)
             exe2 = relay.create_executor("debug", ctx=ctx, target=target)
             res1 = exe1.evaluate(new_log)(x_nd)
@@ -291,9 +288,10 @@ def test_function_pass():
 
 
 def test_module_class_pass():
-    @relay.transform.module_pass(opt_level=1)
+    @tvm.transform.module_pass(opt_level=1)
     class TestPipeline:
         """Simple test function to replace one argument to another."""
+
         def __init__(self, new_mod, replace):
             self.new_mod = new_mod
             self.replace = replace
@@ -304,8 +302,8 @@ def test_module_class_pass():
             return mod
 
     x = relay.var("x", shape=(10, 20))
-    m1 = relay.Module.from_expr(relay.Function([x], x))
-    m2 = relay.Module.from_expr(relay.Function([x], relay.log(x)))
+    m1 = tvm.IRModule.from_expr(relay.Function([x], x))
+    m2 = tvm.IRModule.from_expr(relay.Function([x], relay.log(x)))
     fpass = TestPipeline(m2, replace=True)
     assert fpass.info.name == "TestPipeline"
     mod3 = fpass(m1)
@@ -315,14 +313,15 @@ def test_module_class_pass():
 
 
 def test_pass_info():
-    info = relay.transform.PassInfo(opt_level=1, name="xyz")
+    info = tvm.transform.PassInfo(opt_level=1, name="xyz")
     assert info.opt_level == 1
     assert info.name == "xyz"
 
 
+@tvm.testing.uses_gpu
 def test_sequential_pass():
-    shape = (10, )
-    dtype = 'float32'
+    shape = (10,)
+    dtype = "float32"
     tp = relay.TensorType(shape, dtype)
     x = relay.var("x", tp)
     y = relay.var("y", tp)
@@ -333,16 +332,14 @@ def test_sequential_pass():
     v_log = relay.GlobalVar("myLog")
     log = relay.Function([z], relay.log(z))
 
-    mod = relay.Module({v_sub: sub, v_log: log})
+    mod = tvm.IRModule({v_sub: sub, v_log: log})
 
     def get_ref_log():
         ref_log = relay.Function([x], relay.log(relay.add(x, x)))
         return ref_log
 
     def get_ref_sub():
-        ref_sub = relay.Function([x, y],
-                                 relay.subtract(
-                                     relay.add(x, x), relay.add(y, y)))
+        ref_sub = relay.Function([x, y], relay.subtract(relay.add(x, x), relay.add(y, y)))
         return ref_sub
 
     def get_ref_abs():
@@ -356,7 +353,7 @@ def test_sequential_pass():
     opt_tester = OptTester(mod)
     pass_ctx = None
 
-    @_transform.module_pass(opt_level=1)
+    @tvm.transform.module_pass(opt_level=1)
     def mod_transform(expr, ctx):
         return opt_tester.transform(expr, ctx)
 
@@ -373,22 +370,22 @@ def test_sequential_pass():
         passes = [module_pass, function_pass]
         opt_level = 2
         pass_name = "sequential"
-        sequential = _transform.Sequential(passes=passes, opt_level=opt_level)
+        sequential = tvm.transform.Sequential(passes=passes, opt_level=opt_level)
         pass_info = sequential.info
         assert pass_info.name == pass_name
         assert pass_info.opt_level == opt_level
 
     def test_no_pass():
         passes = []
-        sequential = _transform.Sequential(opt_level=1, passes=passes)
+        sequential = tvm.transform.Sequential(opt_level=1, passes=passes)
         ret_mod = sequential(mod)
         mod_func = ret_mod[v_sub]
         check_func(sub, mod_func)
 
     def test_only_module_pass():
         passes = [module_pass]
-        sequential = _transform.Sequential(opt_level=1, passes=passes)
-        with relay.build_config(required_pass=["mod_transform"]):
+        sequential = tvm.transform.Sequential(opt_level=1, passes=passes)
+        with tvm.transform.PassContext(required_pass=["mod_transform"]):
             ret_mod = sequential(mod)
         # Check the subtract function.
         sub_var, new_sub = extract_var_func(ret_mod, v_sub.name_hint)
@@ -402,8 +399,8 @@ def test_sequential_pass():
     def test_only_function_pass():
         # Check the subtract function.
         passes = [function_pass]
-        sequential = _transform.Sequential(opt_level=1, passes=passes)
-        with relay.build_config(required_pass=["func_transform"]):
+        sequential = tvm.transform.Sequential(opt_level=1, passes=passes)
+        with tvm.transform.PassContext(required_pass=["func_transform"]):
             ret_mod = sequential(mod)
         _, new_sub = extract_var_func(ret_mod, v_sub.name_hint)
         check_func(new_sub, get_ref_sub())
@@ -415,11 +412,11 @@ def test_sequential_pass():
     def test_multiple_passes():
         # Reset the current module since mod has been polluted by the previous
         # function pass.
-        mod = relay.Module({v_sub: sub, v_log: log})
+        mod = tvm.IRModule({v_sub: sub, v_log: log})
         passes = [module_pass, function_pass]
-        sequential = _transform.Sequential(opt_level=1, passes=passes)
+        sequential = tvm.transform.Sequential(opt_level=1, passes=passes)
         required = ["mod_transform", "func_transform"]
-        with relay.build_config(required_pass=required):
+        with tvm.transform.PassContext(required_pass=required):
             ret_mod = sequential(mod)
 
         # Check the abs function is added.
@@ -439,7 +436,7 @@ def test_sequential_pass():
         x_nd = get_rand(shape, dtype)
         y_nd = get_rand(shape, dtype)
         ref_res = np.subtract(x_nd.asnumpy() * 2, y_nd.asnumpy() * 2)
-        for target, ctx in ctx_list():
+        for target, ctx in tvm.testing.enabled_targets():
             exe1 = relay.create_executor("graph", ctx=ctx, target=target)
             exe2 = relay.create_executor("debug", ctx=ctx, target=target)
             res1 = exe1.evaluate(new_sub)(x_nd, y_nd)
@@ -450,7 +447,7 @@ def test_sequential_pass():
         # Execute the updated abs function.
         x_nd = get_rand((5, 10), dtype)
         ref_res = np.abs(x_nd.asnumpy() * 2)
-        for target, ctx in ctx_list():
+        for target, ctx in tvm.testing.enabled_targets():
             exe1 = relay.create_executor("graph", ctx=ctx, target=target)
             exe2 = relay.create_executor("debug", ctx=ctx, target=target)
             res1 = exe1.evaluate(new_abs)(x_nd)
@@ -469,6 +466,7 @@ def test_sequential_with_scoping():
     shape = (1, 2, 3)
     c_data = np.array(shape).astype("float32")
     tp = relay.TensorType(shape, "float32")
+
     def before():
         c = relay.const(c_data)
         x = relay.var("x", tp)
@@ -488,21 +486,23 @@ def test_sequential_with_scoping():
         z1 = relay.add(z, z)
         return relay.Function([x], z1)
 
-    seq = _transform.Sequential([
-        relay.transform.InferType(),
-        relay.transform.FoldConstant(),
-        relay.transform.EliminateCommonSubexpr(),
-        relay.transform.AlterOpLayout()
-    ])
+    seq = tvm.transform.Sequential(
+        [
+            relay.transform.InferType(),
+            relay.transform.FoldConstant(),
+            relay.transform.EliminateCommonSubexpr(),
+            relay.transform.AlterOpLayout(),
+        ]
+    )
 
-    mod = relay.Module({"main": before()})
-    with relay.build_config(opt_level=3):
-        with tvm.target.create("llvm"):
+    mod = tvm.IRModule({"main": before()})
+    with tvm.transform.PassContext(opt_level=3):
+        with tvm.target.Target("llvm"):
             mod = seq(mod)
 
     zz = mod["main"]
     zexpected = run_infer_type(expected())
-    assert analysis.alpha_equal(zz, zexpected)
+    assert tvm.ir.structural_equal(zz, zexpected)
 
 
 def test_print_ir(capfd):
@@ -513,22 +513,59 @@ def test_print_ir(capfd):
     y = relay.multiply(y, relay.const(2, "float32"))
     func = relay.Function([x], y)
 
-    seq = _transform.Sequential([
-        relay.transform.InferType(),
-        relay.transform.FoldConstant(),
-        relay.transform.PrintIR(),
-        relay.transform.DeadCodeElimination()
-    ])
+    seq = tvm.transform.Sequential(
+        [
+            relay.transform.InferType(),
+            relay.transform.FoldConstant(),
+            tvm.transform.PrintIR(),
+            relay.transform.DeadCodeElimination(),
+        ]
+    )
 
-    mod = relay.Module({"main": func})
-    with relay.build_config(opt_level=3):
+    mod = tvm.IRModule({"main": func})
+    with tvm.transform.PassContext(opt_level=3):
         mod = seq(mod)
 
     out = capfd.readouterr().err
 
-    assert "Dumping the module IR" in out
+    assert "PrintIR" in out
     assert "multiply" in out
 
 
+__TRACE_COUNTER__ = 0
+
+
+def _tracer(module, info, is_before):
+    global __TRACE_COUNTER__
+    if bool(is_before):
+        __TRACE_COUNTER__ += 1
+
+
+def test_print_debug_callback():
+    global __TRACE_COUNTER__
+    shape = (1, 2, 3)
+    tp = relay.TensorType(shape, "float32")
+    x = relay.var("x", tp)
+    y = relay.add(x, x)
+    y = relay.multiply(y, relay.const(2, "float32"))
+    func = relay.Function([x], y)
+
+    seq = tvm.transform.Sequential(
+        [
+            relay.transform.InferType(),
+            relay.transform.FoldConstant(),
+            relay.transform.DeadCodeElimination(),
+        ]
+    )
+
+    assert __TRACE_COUNTER__ == 0
+    mod = tvm.IRModule({"main": func})
+
+    with tvm.transform.PassContext(opt_level=3, trace=_tracer):
+        mod = seq(mod)
+
+    assert __TRACE_COUNTER__ == 3
+
+
 if __name__ == "__main__":
     pytest.main()
