@@ -14,27 +14,27 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-import os
-import tvm
 import numpy as np
 
-import pytest
+from tvm.runtime import profiler_vm
 from tvm import relay
-from tvm.relay.testing import resnet
+from tvm.relay.testing import resnet, enabled_targets
+
 
 def test_basic():
     mod, params = resnet.get_workload()
-    target = 'llvm'
-    ctx = tvm.cpu()
-    if not relay.profiler_vm.enabled():
+    if not profiler_vm.enabled():
         return
-    exe = relay.vm.compile(mod, target, params=params)
-    vm = relay.profiler_vm.VirtualMachineProfiler(exe)
-    vm.init(ctx)
 
-    data = np.random.rand(1, 3, 224, 224).astype('float32')
-    res = vm.invoke("main", [data])
-    print("\n{}".format(vm.get_stat()))
+    for target, ctx in enabled_targets():
+        exe = relay.vm.compile(mod, target, params=params)
+        vm = profiler_vm.VirtualMachineProfiler(exe, ctx)
+
+        data = np.random.rand(1, 3, 224, 224).astype("float32")
+        res = vm.invoke("main", [data])
+        print("\n{}".format(vm.get_stat()))
+        print("\n{}".format(vm.get_stat(False)))
+
 
 if __name__ == "__main__":
     test_basic()
