@@ -20,11 +20,24 @@ set -e
 set -u
 
 export LD_LIBRARY_PATH="lib:${LD_LIBRARY_PATH:-}"
+# NOTE: important to use abspath, when VTA is enabled.
+export VTA_HW_PATH=`pwd`/3rdparty/vta-hw
+
+# to avoid CI thread throttling.
+export TVM_BIND_THREADS=0
+export OMP_NUM_THREADS=1
 
 # Remove existing testcases
 rm -f build/*_test
 
-make cpptest -j8
+make cpptest -j2
+make crttest  # NOTE: don't parallelize, due to issue with build deps.
 for test in build/*_test; do
     ./$test
 done
+
+# Test MISRA-C runtime
+cd apps/bundle_deploy
+rm -rf build
+make test_dynamic test_static
+cd ../..
