@@ -36,25 +36,26 @@ from __future__ import absolute_import, print_function
 
 import os
 import tvm
+from tvm import te
 import vta
 import numpy as np
 from tvm import rpc
 from tvm.contrib import util
 from vta.testing import simulator
 
-# Load VTA parameters from the vta/config/vta_config.json file
+# Load VTA parameters from the 3rdparty/vta-hw/config/vta_config.json file
 env = vta.get_env()
 
 # We read the Pynq RPC host IP address and port number from the OS environment
-host = os.environ.get("VTA_PYNQ_RPC_HOST", "192.168.2.99")
-port = int(os.environ.get("VTA_PYNQ_RPC_PORT", "9091"))
+host = os.environ.get("VTA_RPC_HOST", "192.168.2.99")
+port = int(os.environ.get("VTA_RPC_PORT", "9091"))
 
 # We configure both the bitstream and the runtime system on the Pynq
 # to match the VTA configuration specified by the vta_config.json file.
-if env.TARGET == "pynq":
+if env.TARGET == "pynq" or env.TARGET == "de10nano":
 
     # Make sure that TVM was compiled with RPC=1
-    assert tvm.module.enabled("rpc")
+    assert tvm.runtime.enabled("rpc")
     remote = rpc.connect(host, port)
 
     # Reconfigure the JIT runtime
@@ -167,13 +168,13 @@ n = 16
 # Batch factor o (we use single batch inference)
 o = 1
 # A placeholder tensor in tiled data format
-A = tvm.placeholder((o, n, env.BATCH, env.BLOCK_IN), name="A", dtype=env.inp_dtype)
+A = te.placeholder((o, n, env.BATCH, env.BLOCK_IN), name="A", dtype=env.inp_dtype)
 # B placeholder tensor in tiled data format
-B = tvm.placeholder((m, n, env.BLOCK_OUT, env.BLOCK_IN), name="B", dtype=env.wgt_dtype)
+B = te.placeholder((m, n, env.BLOCK_OUT, env.BLOCK_IN), name="B", dtype=env.wgt_dtype)
 # A copy buffer
-A_buf = tvm.compute((o, n, env.BATCH, env.BLOCK_IN), lambda *i: A(*i), "A_buf")
+A_buf = te.compute((o, n, env.BATCH, env.BLOCK_IN), lambda *i: A(*i), "A_buf")
 # B copy buffer
-B_buf = tvm.compute((m, n, env.BLOCK_OUT, env.BLOCK_IN), lambda *i: B(*i), "B_buf")
+B_buf = te.compute((m, n, env.BLOCK_OUT, env.BLOCK_IN), lambda *i: B(*i), "B_buf")
 
 ######################################################################
 # Matrix Multiplication
@@ -186,8 +187,8 @@ B_buf = tvm.compute((m, n, env.BLOCK_OUT, env.BLOCK_IN), lambda *i: B(*i), "B_bu
 # In order to implement matrix multiplication, the lambda function needs to
 # include a reduction formula over the input channel dimension axes.
 # To create a reduction formula, we can declare a reduction axis using
-# :code:`tvm.reduce_axis`, which takes in the range of reductions.
-# :code:`tvm.sum` takes in the expression to be reduced as well as
+# :code:`te.reduce_axis`, which takes in the range of reductions.
+# :code:`te.sum` takes in the expression to be reduced as well as
 # the reduction axes to compute the sum of value over all k in the declared
 # ranges.
 #
@@ -198,17 +199,18 @@ B_buf = tvm.compute((m, n, env.BLOCK_OUT, env.BLOCK_IN), lambda *i: B(*i), "B_bu
 # the computation should be done.
 
 # Outer input feature reduction axis
-ko = tvm.reduce_axis((0, n), name="ko")
+ko = te.reduce_axis((0, n), name="ko")
 # Inner input feature reduction axis
-ki = tvm.reduce_axis((0, env.BLOCK_IN), name="ki")
+ki = te.reduce_axis((0, env.BLOCK_IN), name="ki")
 # Describe the in-VTA matrix multiplication
-C_buf = tvm.compute(
+C_buf = te.compute(
     (o, m, env.BATCH, env.BLOCK_OUT),
-    lambda bo, co, bi, ci:
-        tvm.sum(A_buf[bo, ko, bi, ki].astype(env.acc_dtype) *
-                B_buf[co, ko, ci, ki].astype(env.acc_dtype),
-                axis=[ko, ki]),
-    name="C_buf")
+    lambda bo, co, bi, ci: te.sum(
+        A_buf[bo, ko, bi, ki].astype(env.acc_dtype) * B_buf[co, ko, ci, ki].astype(env.acc_dtype),
+        axis=[ko, ki],
+    ),
+    name="C_buf",
+)
 
 ######################################################################
 # Casting the Results
@@ -234,10 +236,9 @@ C_buf = tvm.compute(
 # input activation data format.
 
 # Cast to output type, and send to main memory
-C = tvm.compute(
-    (o, m, env.BATCH, env.BLOCK_OUT),
-    lambda *i: C_buf(*i).astype(env.inp_dtype),
-    name="C")
+C = te.compute(
+    (o, m, env.BATCH, env.BLOCK_OUT), lambda *i: C_buf(*i).astype(env.inp_dtype), name="C"
+)
 
 ######################################################################
 # This concludes the computation declaration part of this tutorial.
@@ -265,7 +266,7 @@ C = tvm.compute(
 # :code:`C` in the following way:
 
 # Let's take a look at the generated schedule
-s = tvm.create_schedule(C.op)
+s = te.create_schedule(C.op)
 print(tvm.lower(s, [A, B, C], simple_mode=True))
 
 ######################################################################
@@ -368,12 +369,8 @@ print(tvm.lower(s, [A, B, C], simple_mode=True))
 # by the VTA runtime JIT compiler.
 
 s[C_buf].reorder(
-    ko,
-    s[C_buf].op.axis[0],
-    s[C_buf].op.axis[1],
-    s[C_buf].op.axis[2],
-    s[C_buf].op.axis[3],
-    ki)
+    ko, s[C_buf].op.axis[0], s[C_buf].op.axis[1], s[C_buf].op.axis[2], s[C_buf].op.axis[3], ki
+)
 s[C_buf].tensorize(s[C_buf].op.axis[2], env.gemm)
 
 # Let's take a look at the finalized schedule
@@ -421,16 +418,12 @@ f = remote.load_module("gemm.o")
 ctx = remote.ext_dev(0)
 
 # Initialize the A and B arrays randomly in the int range of (-128, 128]
-A_orig = np.random.randint(
-    -128, 128, size=(o * env.BATCH, n * env.BLOCK_IN)).astype(A.dtype)
-B_orig = np.random.randint(
-    -128, 128, size=(m * env.BLOCK_OUT, n * env.BLOCK_IN)).astype(B.dtype)
+A_orig = np.random.randint(-128, 128, size=(o * env.BATCH, n * env.BLOCK_IN)).astype(A.dtype)
+B_orig = np.random.randint(-128, 128, size=(m * env.BLOCK_OUT, n * env.BLOCK_IN)).astype(B.dtype)
 
 # Apply packing to the A and B arrays from a 2D to a 4D packed layout
-A_packed = A_orig.reshape(
-    o, env.BATCH, n, env.BLOCK_IN).transpose((0, 2, 1, 3))
-B_packed = B_orig.reshape(
-    m, env.BLOCK_OUT, n, env.BLOCK_IN).transpose((0, 2, 1, 3))
+A_packed = A_orig.reshape(o, env.BATCH, n, env.BLOCK_IN).transpose((0, 2, 1, 3))
+B_packed = B_orig.reshape(m, env.BLOCK_OUT, n, env.BLOCK_IN).transpose((0, 2, 1, 3))
 
 # Format the input/output arrays with tvm.nd.array to the DLPack standard
 A_nd = tvm.nd.array(A_packed, ctx)
@@ -451,10 +444,8 @@ f(A_nd, B_nd, C_nd)
 # matrix multiplication indeed is correct
 
 # Compute reference result with numpy
-C_ref = np.dot(A_orig.astype(env.acc_dtype),
-               B_orig.T.astype(env.acc_dtype)).astype(C.dtype)
-C_ref = C_ref.reshape(
-    o, env.BATCH, m, env.BLOCK_OUT).transpose((0, 2, 1, 3))
+C_ref = np.dot(A_orig.astype(env.acc_dtype), B_orig.T.astype(env.acc_dtype)).astype(C.dtype)
+C_ref = C_ref.reshape(o, env.BATCH, m, env.BLOCK_OUT).transpose((0, 2, 1, 3))
 np.testing.assert_equal(C_ref, C_nd.asnumpy())
 
 # Print stats
